C+++
C	CTS - the CAMAC Topology Supervisor
C
C This program maintains a data base of logical names and their physical CAMAC
C device equivalents.  The data base is a single index file with two keys.  The
C primary key is in logical name order.  The secondary key is in CAMAC device
C specifier order, with duplicates and changes permitted.
C
C The following commands are understood:
C
C	ASSIGN		creates a logical name - physical device equivalence
C
C	AUTOCONFIGURE	recreates equivalences
C
C	DEASSIGN	deletes an equivalence
C
C	EXIT		terminates the program
C
C	HELP		provides on-line help on the use of this program
C
C	MODIFY		modifies equivalences
C
C	SET		sets a CAMAC support unit (crate) on- or off-line,
C			enables/disables LAM support
C
C	SHOW		lists equivalences
C
C
C Source modules:
C
C	CTS.FOR		main routine, plus autoconfigure, exit and help
C			commands;  report & get_keyword_val utility routines
C
C	CTSASSIGN.FOR	assign command
C
C	CTSDEASGN.FOR	deassign command
C
C	CTSMODIFY.FOR	modify command
C
C	CTSSET.FOR	set command
C
C	CTSSHOW.FOR	show command
C
C	CTSUSROPN.MAR	user open routine to set database file protection
C
C	CTSCLD.CLD	command language definition source file
C
C	CTSHELP.HLP	help text source file
C
C
C Compilation and Link Requirements:
C
C	FOR CTS, CTSASSIGN, CTSDEASGN, CTSMODIFY, CTSSET, CTSSHOW
C
C	MAC CTSUSROPN
C
C	SET COMMAND/OBJ CTSCLD
C
C	LINK/NOTRACEBACK CTS, CTSASSIGN, CTSDEASGN, CTSMODIFY, CTSSET, -
C	CTSSHOW, CTSUSROPN, CTSCLD, WCCOPY, WCMATCH, SYS$LIBRARY:CAMLIB/LIB
C
C	LIBRARY/HELP SYS$LIBRARY:CTS CTSHELP
C
C CTS must be installed with SYSPRV, SYSNAM and PHY_IO privleges
C
C
C Revision History
C   Date	By		Comments
C  9/01/82	D.E. Greenwood	Original
C 10/07/82	D.E.Greenwood	^Z treated like EXIT cmd; LAM enable/disable
C				moved to SET cmd; SET cmd source moved to
C				separate module (CTSSET); subroutine
C				GET_KEYWORD_VAL added
C 10/08/82	D.E.Greenwood	Correction to GET_KEYWORD_VAL to return the
C				last value of the legal value string correctly
C  1/04/83	D.E.Greenwood	AUTOCONFIGURE allows list of match specs, full
C				wildcards, logical or physical specifications;
C				REPORT takes second argument
C
C---

C Main routine
C
	PARAMETER FILENAME='SYS$SYSTEM:SYSCAMAC.DAT'
	CHARACTER*80 A
	EXTERNAL CTSUSROPN, CAMAC_EQUIVALENCES
	INTEGER*4 CMD_LEN,FRGN_CMD_LEN
	INTEGER*4 CLI$DCL_PARSE, LIB$GET_INPUT
C
C Create (if necessary) and open the file containing the equivalences.
C
	OPEN (1, FILE=FILENAME, ACCESS='KEYED', ERR=1000, FORM='UNFORMATTED',
	1	ORGANIZATION='INDEXED', RECL=30,  SHARED, STATUS='OLD')
	GOTO 10000
C
1000	TYPE *, '%CTS-I-CREAT, creating new ',FILENAME,' file'
	OPEN (1, FILE=FILENAME, ACCESS='KEYED', FORM='UNFORMATTED',
	1	ORGANIZATION='INDEXED', RECL=30, SHARED, STATUS='NEW',
	2	KEY=(1:32:CHARACTER,37:47:CHARACTER), USEROPEN=CTSUSROPN)
C
C Get and parse the command string.  Then dispatch to the appropriate subroutine.
C If a foreign command string was entered, we'll execute that command only and
C then exit.  Else, we'll continue to prompt for commands until the EXIT
C command (or ^Z) is entered.
C
10000	CALL LIB$GET_FOREIGN(A,,FRGN_CMD_LEN)
	IF (FRGN_CMD_LEN .GT. 0) GOTO 10200
C
10100	IF (.NOT. LIB$GET_INPUT(A,'CTS> ',CMD_LEN)) CALL DONE
	IF (CMD_LEN .EQ. 0) GOTO 10100
C
10200	IF (.NOT. CLI$DCL_PARSE(A,CAMAC_EQUIVALENCES) .AND. 1) GOTO 10100
	CALL CLI$DISPATCH
	IF (FRGN_CMD_LEN .EQ. 0) GOTO 10100
	CALL DONE
C
	END

C AUTOCONFIGURE command
C
C This subroutine reassigns the CAMAC logical names, based on the data base.
C The normal use for this subroutine is to recreate all CAMAC logical name
C equivalences each time VMS is reloaded.
C
	SUBROUTINE AUTO
C
	CHARACTER MATCH*32, MIN_MATCH*32, PHY_SPEC*32, TEMP_STR*2
	INTEGER*2 LOG_LEN, MATCH_LEN, MIN_MATCH_LEN,
     &		PHY_SPEC_LEN, PRCNT_LOC, STAR_LOC
	INTEGER*4 ISTAT
	INTEGER*4 CLI$GET_VALUE, CLI$PRESENT, SYS$CRELOG, WCMATCH
C
	INCLUDE 'CTSDEF'
	INCLUDE '($SSDEF)'
C
C
C Get the (next) match specification and the minimum number of characters
C required to match.
C
100	IF (.NOT. CLI$GET_VALUE('NAME',MATCH)) RETURN
	CALL STR$UPCASE(MATCH,MATCH)
	CALL STR$TRIM(MATCH,MATCH,MATCH_LEN)
	STAR_LOC = INDEX(MATCH,'*')
	PRCNT_LOC = INDEX(MATCH,'%')
	MIN_MATCH_LEN = MATCH_LEN
	IF (STAR_LOC .GT. 0) MIN_MATCH_LEN = STAR_LOC - 1
	IF (PRCNT_LOC .GT. 0 .AND. PRCNT_LOC-1 .LT. MIN_MATCH_LEN)
     &		MIN_MATCH_LEN = PRCNT_LOC - 1
	MIN_MATCH = MATCH(1:MIN_MATCH_LEN)
	IF (MIN_MATCH_LEN .EQ. 0) MIN_MATCH = ' '
C
C Determine the sorting order (logical name order, unless "/physical" was
C specified.
C
	IF (CLI$PRESENT('PHYSICAL')) GOTO 2000
C
C Select assignments in logical name order.
C
1000	READ (1,KEYGE=MIN_MATCH,KEYID=0,ERR=89000) CAMDEF_RECORD
	UNLOCK 1
C
	DO WHILE (MIN_MATCH_LEN .EQ. 0 .OR.
     &		LOGICAL_NAME(1:MIN_MATCH_LEN) .EQ. MIN_MATCH)
	  CALL STR$TRIM(LOGICAL_NAME,LOGICAL_NAME,LOG_LEN)
	  IF (MATCH_LEN .EQ. 0 .OR.
     &		WCMATCH( MATCH(1:MATCH_LEN), LOGICAL_NAME(1:LOG_LEN))) THEN
	    CALL MAKE_FULL_SPEC(CAMDEF_RECORD, PHY_SPEC, PHY_SPEC_LEN)
	    ISTAT = SYS$CRELOG(%VAL(0), LOGICAL_NAME(1:LOG_LEN),
     &		PHY_SPEC(1:PHY_SPEC_LEN),)
	    IF (ISTAT .NE. SS$_NORMAL)
     &		CALL REPORT('Error assigning '//LOGICAL_NAME(1:LOG_LEN)//': ',
     &		ISTAT)
	  ENDIF
	  READ (1,END=89000) CAMDEF_RECORD
	  UNLOCK 1
	END DO
C
	GOTO 89000
C
C Choose assignments in physical specifier order.
C
2000	IF (MIN_MATCH_LEN .GT. 3) THEN			! This kludge req'd
	    MIN_MATCH_LEN = 3				!  because crate may be
	    MIN_MATCH(4:) = ' '				!  either 1 or 2 chars
	ENDIF						!  long
	READ (1,KEYGE=MIN_MATCH(1:11),KEYID=1,ERR=89000) CAMDEF_RECORD
	UNLOCK 1
C
	DO WHILE (.TRUE.)
	  CALL MAKE_SPECIFIER(CAMDEF_RECORD,PHY_SPEC)
	  IF (MIN_MATCH_LEN .GT. 0 .AND.
     &		PHY_SPEC(1:MIN_MATCH_LEN) .GT. MIN_MATCH) GOTO 89000
	  CALL STR$TRIM(PHY_SPEC,PHY_SPEC,PHY_SPEC_LEN)
	  IF (MATCH_LEN .EQ. 0 .OR.
     &		WCMATCH(MATCH(1:MATCH_LEN),PHY_SPEC(1:PHY_SPEC_LEN))) THEN
	    CALL MAKE_FULL_SPEC(CAMDEF_RECORD, PHY_SPEC, PHY_SPEC_LEN)
	    ISTAT = SYS$CRELOG(%VAL(0),LOGICAL_NAME(1:LOG_NAM_LEN),
	1	PHY_SPEC(1:PHY_SPEC_LEN),)
	    IF (ISTAT .NE. SS$_NORMAL)
     &		CALL REPORT('Error assigning '//PHY_SPEC(1:PHY_SPEC_LEN)//': ',
     &		ISTAT)
	  ENDIF
	  READ (1,END=89000) CAMDEF_RECORD
	  UNLOCK 1
	END DO
C
89000	GOTO 100
C
	END

C EXIT command
C
C This subroutine closes the data base file and terminates the supervisor.
C
	SUBROUTINE DONE
	CLOSE (1)
	CALL EXIT
	END

C HELP command
C
C This subroutine calls the VMS help routines to provide the user with
C information on how to use the supervisor
C
	SUBROUTINE HELP
	CHARACTER*80 LINE
	EXTERNAL LIB$PUT_OUTPUT,LIB$GET_INPUT
	INTEGER*4 CLI$PRESENT,CLI$GET_VALUE,LBR$OUTPUT_HELP
C
	LINE = ' '
	IF (CLI$PRESENT('TEXT')) CALL CLI$GET_VALUE('TEXT',LINE)
	ISTAT=LBR$OUTPUT_HELP(LIB$PUT_OUTPUT,,LINE,
     &		'SYS$HELP:CTS',,LIB$GET_INPUT)
	IF (.NOT. ISTAT) CALL LIB$SIGNAL(%VAL(ISTAT))
	RETURN
	END

C REPORT - a utility subroutine for the supervisor
C
C This subroutine types an error message on SYS$OUTPUT.
C
C Calling Sequence:
C
C	CALL REPORT(MSG_ID, MSG_NUM)
C
C   where
C
C	MSG_ID		is a character expression to be typed before the system
C			error message (an all-blank string will not be typed)
C	MSG_NUM		is the address of an integer containing the message
C			number
C
C
C
	SUBROUTINE REPORT(MSG_ID, MSG_NUM)
C
	CHARACTER*132 MESSAGE, MSG_ID*(*)
	INTEGER*2 MSG_LEN
	INTEGER*4 SYS$GETMSG
C
	ISTAT = SYS$GETMSG(%VAL(MSG_NUM), MSG_LEN, MESSAGE, %VAL(15),)
	IF (ISTAT) THEN
	    IF (MSG_ID .NE. ' ') THEN
		TYPE *, MSG_ID, MESSAGE(1:MSG_LEN)
	    ELSE
		TYPE *, MESSAGE(1:MSG_LEN)
	    ENDIF
	ELSE
	    IF (MSG_ID .NE. ' ') THEN
		TYPE *, MSG_ID, '%CTS-W-ERROR: ', MSG_NUM
	    ELSE
		TYPE *, 'CTS-W-ERROR: ',MSG_NUM
	    ENDIF
	END IF
	RETURN
	END

C	GET_KEYWORD_VAL
C
C Function to check that the value of a parameter or qualifier returned by the
C CLI is contained uniquely in a list of legal values.
C
C Calling sequence:
C
C	ISTATUS = GET_KEYWORD_VAL(NAME,LEGAL_LIST,VALUE)
C   where:
C	NAME		is the address of a string descriptor containing the
C			name of the qualifier or parameter (the name must
C			be recognizable by CLI$GET_VALUE)
C	LEGAL_LIST	is the address of a string descriptor containing the
C			list of legal keyword values, separated by a "/"
C	VALUE		is the address of a string descriptor defining where
C			the full spelling of the value will be placed
C
C
	INTEGER*4 FUNCTION GET_KEYWORD_VAL(NAME, LEGAL_LIST, VALUE)
C
C
	CHARACTER ENTRY*64, LEGAL_LIST*(*), NAME*(*), VALUE*(*)
	INTEGER*2 ENTRY_LEN, LEGAL_LIST_LEN, MATCH_LOC, SLASH_LOC
	INTEGER*4 STATUS
	INTEGER*4 CLI$GET_VALUE
C
	LEGAL_LIST_LEN = LEN(LEGAL_LIST)
	STATUS = CLI$GET_VALUE(NAME, ENTRY)
	IF (.NOT. STATUS) GOTO 89900
	CALL STR$TRIM(ENTRY, ENTRY, ENTRY_LEN)
C
C Search the list of legal values for a unique match with the entered value.
C
	MATCH_LOC = 0
	START_LOC = 1
	DO 10050 WHILE (.TRUE.)
	   IF (INDEX(LEGAL_LIST(START_LOC:),ENTRY(1:ENTRY_LEN)) .EQ. 1) THEN
		IF (MATCH_LOC .NE. 0) THEN
		   STATUS = .FALSE.			! not unique
		   TYPE *, '%CTS-W-ABKEYVAL, ambiguous keyword value'
		   GOTO 89900
		ENDIF
		MATCH_LOC = START_LOC			! found match
	   ENDIF
	   SLASH_LOC = INDEX(LEGAL_LIST(START_LOC:),'/')
	   IF (SLASH_LOC .EQ. 0) GOTO 20000		! no '/' means done
	   START_LOC = START_LOC + SLASH_LOC
10050	CONTINUE
C
C Done searching.  Return full spelling of value if we found a match.
C
20000	IF (MATCH_LOC .EQ. 0) THEN
	   STATUS = .FALSE.				! illegal value
	   TYPE *, '%CTS-W-ILLVAL, illegal keyword value'
	   GOTO 89900
	ENDIF
	SLASH_LOC = INDEX(LEGAL_LIST(MATCH_LOC:),'/')
	IF (SLASH_LOC .EQ. 0) SLASH_LOC = LEN(LEGAL_LIST)-MATCH_LOC+2
	VALUE = LEGAL_LIST(MATCH_LOC:SLASH_LOC+MATCH_LOC-2)
	STATUS = .TRUE.
C
C Return
C
89900	GET_KEYWORD_VAL = STATUS
	RETURN
	END
