C+++
C	CTSASSIGN.FOR
C
C This module contains the code for the ASSIGN command of the CAMAC Topology
C Supervisor (CTS).
C
C
C   Date	By		Comments
C    ??		D.E.Greenwood	Original
C 10/07/82	D.E.Greenwood	This header added; order of prompts for
C				physical_module name and logical name reversed
C				(physical name first) to correspond with order
C				in CLD file
C  1/03/83	D.E.Greenwood	Calls to REPORT changed to 2 arguments
C
C---
C
C
	SUBROUTINE ASSIGN
C
	CHARACTER PHY_NAME*32, VALUE*32
	INTEGER*2 LOG_NAM_LEN, PHY_NAM_LEN, TEMP_NUM, VALUE_LEN
	INTEGER*4 CAM$PHYMOD, CLI$GET_VALUE, CLI$PRESENT
	INTEGER*4 LIB$SYS_TRNLOG, LIB$SYS_GETMSG, SYS$CRELOG
C
	INCLUDE 'CTSDEF/LIST'			! Define standard vars.
	INCLUDE '($SSDEF)'			! System status code defs.

C
C Get physical specification and it's length.  If we must prompt for the
C spec., remove leading and/or trailing double-quotes (") so as to be
C consistent with the way the CLI handles quoted parameters.
C
	IF (CLI$PRESENT('PHY_NAME')) THEN
	  CALL CLI$GET_VALUE('PHY_NAME',PHY_NAME)
	ELSE
	  CALL LIB$GET_INPUT(PHY_NAME,'Physical specification: ',PHY_NAM_LEN)
	  IF (PHY_NAME(PHY_NAM_LEN:) .EQ. '"')
     &		PHY_NAME = PHY_NAME(:PHY_NAM_LEN-1)
	  IF (PHY_NAME(1:1) .EQ. '"') PHY_NAME = PHY_NAME(2:)
	END IF
	CALL STR$UPCASE(PHY_NAME,PHY_NAME)
	CALL STR$TRIM(PHY_NAME,PHY_NAME,PHY_NAM_LEN)
C
C Verify and parse physical specification into its components.
C
	ISTAT = CAM$PHYMOD(PHY_NAME(1:PHY_NAM_LEN),CSU_NAME,PHY_MOD)
	IF (.NOT. ISTAT) THEN
	  CALL REPORT(' ', ISTAT)
	  GOTO 89000
	END IF
C
C Get the logical name (may have to prompt for it) and its length.
C
	IF (CLI$PRESENT('LOG_NAME')) THEN
	  CALL CLI$GET_VALUE('LOG_NAME',LOGICAL_NAME)
	ELSE
	  CALL LIB$GET_INPUT(LOGICAL_NAME,'Logical name: ',LOG_NAM_LEN)
	END IF
	CALL STR$UPCASE(LOGICAL_NAME,LOGICAL_NAME)
	CALL STR$TRIM(LOGICAL_NAME,LOGICAL_NAME,LOG_NAM_LEN)
C
C Insure logical name is legal and does not duplicate any existing one in the
C system logical name table.
C
	ISTAT = LIB$SYS_TRNLOG(LOGICAL_NAME(1:LOG_NAM_LEN),,LOGICAL_NAME,,,6)
	IF (ISTAT .EQ. SS$_NOTRAN) GOTO 2000
	IF (ISTAT .EQ. SS$_NORMAL .OR. ISTAT .EQ. SS$_STRTRU) THEN
	  TYPE *, '%CTS-W-INUSE, logical name already in use'
	ELSE
	  CALL REPORT(' ', ISTAT)
	END IF
	GOTO 89000
C
C Get the optional comment.
C
2000	COMMENT = ' '
	IF (CLI$PRESENT('COMMENT')) CALL CLI$GET_VALUE('COMMENT',COMMENT)
C
C Get the optional reference uic
C
	IF (CLI$PRESENT('REFERENCE_UIC')) THEN
	  CALL GET_REF_UIC(PHY_NAME,PHY_NAM_LEN)
	  IF (.NOT. CAM$PHYMOD(PHY_NAME(1:PHY_NAM_LEN),CSU_NAME,PHY_MOD))
	1	 THEN
	    TYPE *, '%CTS-W-ILLUIC, illegal reference uic'
	    GOTO 89000
	  END IF
	END IF
C
C Create the physical specification portion of the file record, create the
C logical name, write the record to the file and we're done.
C
	CALL MAKE_PHY_REC(CSU_NAME,PHY_MOD,CAMDEF_RECORD)
	CALL MAKE_FULL_SPEC(CAMDEF_RECORD,PHY_NAME,PHY_NAM_LEN)
	ISTAT = SYS$CRELOG(%VAL(0),LOGICAL_NAME(1:LOG_NAM_LEN),
	1	PHY_NAME(1:PHY_NAM_LEN),)
	IF (ISTAT .EQ. SS$_NORMAL) THEN
	  WRITE (1) CAMDEF_RECORD
	ELSE
	  CALL REPORT(' ', ISTAT)
	END IF
89000	RETURN
	END

	SUBROUTINE MAKE_PHY_REC(CSU_ID,PHY_MOD_SPEC,RECORD)
C
	CHARACTER*(*) CSU_ID,RECORD
	INTEGER*2 COLON_LOC
	INTEGER*4 PHY_MOD_SPEC(*)
C
	INCLUDE 'CTSDEF'
C
	CAMDEF_RECORD = RECORD
	DO 10000 I=1,4
	  PHY_MOD(I) = PHY_MOD_SPEC(I)
10000	END DO
	CSU(1:3) = CSU_ID(1:3)
	ENCODE (2,90000,CSU_UNIT) PM_UNIT
	CR_COLON = ':'
	ENCODE (3,90010,STATION) PM_STATION
	REF_UIC_GROUP = PM_RUG
	REF_UIC_MEMBER = PM_RUM
	RECORD = CAMDEF_RECORD
	RETURN
C
90000	FORMAT(I2)
90010	FORMAT('N',I2)
	END

	SUBROUTINE GET_REF_UIC(SPEC_STR,SPEC_STR_LEN)
C
	CHARACTER*(*) SPEC_STR, UIC*15
	INTEGER*2 COLON_LOC, LFBRKT_LOC, SPEC_STR_LEN, UIC_LEN
C
	CALL CLI$GET_VALUE('REFERENCE_UIC',UIC)
	CALL STR$TRIM(UIC,UIC,UIC_LEN)
	LFBRKT_LOC = INDEX(SPEC_STR,'[')
	IF (LFBRKT_LOC .GT. 0) SPEC_STR =
	1	SPEC_STR(1:LFBRKT_LOC-1)//SPEC_STR(INDEX(SPEC_STR,']')+1:)
	COLON_LOC = INDEX(SPEC_STR,':')
	SPEC_STR = SPEC_STR(1:COLON_LOC)//UIC(1:UIC_LEN)
	1	//SPEC_STR(COLON_LOC+1:)
	CALL STR$TRIM(SPEC_STR,SPEC_STR,SPEC_STR_LEN)
	RETURN
	END

	SUBROUTINE MAKE_FULL_SPEC(RECORD,PHY_SPEC,PHY_SPEC_LEN)
C
	CHARACTER PHY_SPEC*(*), RECORD*(*), REFERENCE_UIC*15
	INTEGER*2 PHY_SPEC_LEN, REF_UIC_LEN
	INCLUDE 'CTSDEF'
C
	CAMDEF_RECORD = RECORD
	CALL MAKE_REF(CAMDEF_RECORD,REFERENCE_UIC)
	CALL MAKE_SPECIFIER(CAMDEF_RECORD,PHY_SPEC)
	CALL STR$TRIM(REFERENCE_UIC,REFERENCE_UIC,REF_UIC_LEN)
	IF (REF_UIC_LEN .GT. 0) THEN
	  COLON_LOC = INDEX(PHY_SPEC,':')
	  PHY_SPEC = PHY_SPEC(1:COLON_LOC)//REFERENCE_UIC(1:REF_UIC_LEN)
	1	//PHY_SPEC(COLON_LOC+1:)
	END IF
	CALL STR$TRIM(PHY_SPEC,PHY_SPEC,PHY_SPEC_LEN)
	RETURN
	END
