C+++
C
C	Modify Subroutine for CAMAC Topology Supervisor
C
C This routine is called in response to the CTS command MODIFY.
C
C
C   Date	By		Comments
C		D.E.Greenwood	Original
C  1/05/83	D.E.Greenwood	Wildcard changes permitted;  /[NO]PHYSICAL,
C				/[NO]LOG, /BRANCH, /CRATE and /HIGHWAY
C				switches added
C  2/17/83	D.E.Greenwood	Correction to update system logical name in
C				addition to database when physical specification
C				is modified;  documentation updates
C
C---
C
	SUBROUTINE MODIFY
C
	IMPLICIT NONE
C
	CHARACTER MATCH*32, MIN_MATCH*32, PHY_SPECIFIER*32
	INTEGER*2 LOG_LEN, MATCH_LEN, MIN_MATCH_LEN, PHY_SPEC_LEN,
     &		PRCNT_LOC, STAR_LOC
	INTEGER*4 CLI$GET_VALUE, CLI$PRESENT, MODIFY_ONE_REC, WCMATCH
	LOGICAL ANY_MATCHES, LOG_CHANGES
C
	INCLUDE 'CTSDEF'
C
	COMMON /CTS_MODIFY/ MATCH, MATCH_LEN, CAMDEF_RECORD, CSU_NAME, PHY_MOD
C
	LOG_CHANGES = CLI$PRESENT('LOG')
C
C Get the (next) match specification and determine the minimum number of
C characters required for a match.  The minimum match string is used to
C minimize the number of file reads we do while searching for records which
C fit the complete match pattern.
C
	DO WHILE (CLI$GET_VALUE('NAME', MATCH))
	    CALL STR$UPCASE(MATCH, MATCH)
	    CALL STR$TRIM( MATCH, MATCH, MATCH_LEN)		! get length
	    STAR_LOC = INDEX(MATCH, '*')			! min. match str.
	    PRCNT_LOC = INDEX( MATCH, '%')			!  is the part
	    MIN_MATCH_LEN = MATCH_LEN				!  of the match
	    IF (STAR_LOC .GT. 0) MIN_MATCH_LEN = STAR_LOC - 1	!  string before
	    IF (PRCNT_LOC .GT. 0 .AND. PRCNT_LOC-1 .LT. MIN_MATCH_LEN)	!  the
     &			MIN_MATCH_LEN = PRCNT_LOC - 1		!  1st wildcard
	    MIN_MATCH = MATCH(1:MIN_MATCH_LEN)			!  character
	    IF (MIN_MATCH_LEN .EQ. 0) MIN_MATCH = ' '
C
	    ANY_MATCHES = .FALSE.
	    IF (CLI$PRESENT('PHYSICAL')) THEN
C
C /PHYSICAL specified, search for a match on the physical specifier
C
		IF (MIN_MATCH_LEN .GT. 3) THEN		! this kluge insures that
		    MIN_MATCH_LEN = 3			!  match specs which
		    MIN_MATCH(4:) = ' '			!  include crate nos work
		ENDIF					!  correctly (at the cost
							!  of extra file reads)
		READ (1,KEYGE=MIN_MATCH(1:11),KEYID=1,ERR=18050) CAMDEF_RECORD
		DO WHILE (.TRUE.)
		    CALL MAKE_SPECIFIER( CAMDEF_RECORD, PHY_SPECIFIER)
		    IF (MIN_MATCH_LEN .GT. 0 .AND.
     &			PHY_SPECIFIER(1:MIN_MATCH_LEN) .NE.
     &			MIN_MATCH(1:MIN_MATCH_LEN)) GOTO 18000
		    CALL STR$TRIM( PHY_SPECIFIER, PHY_SPECIFIER, PHY_SPEC_LEN)
		    IF (MATCH_LEN .EQ. 0 .OR.
     &			WCMATCH( MATCH(1:MATCH_LEN),
     &				PHY_SPECIFIER(1:PHY_SPEC_LEN)))
     &			THEN
			    IF (.NOT. MODIFY_ONE_REC('PHYSICAL',LOG_CHANGES))
     &				RETURN
			    ANY_MATCHES = .TRUE.
		    ENDIF
		    READ (1,END=18050) CAMDEF_RECORD
C
		ENDDO			! end of physical spec match loop
C
18000		UNLOCK (1)
18050		IF (.NOT. ANY_MATCHES) TYPE *, 'No physical spec. matches ',
     &			MATCH(1:MATCH_LEN)
C
	    ELSE
C
C Search for match on logical name
C
		READ (1,KEYGE=MIN_MATCH,KEYID=0,ERR=28000) CAMDEF_RECORD
		DO WHILE (MIN_MATCH_LEN .EQ. 0 .OR.
     &			LOGICAL_NAME(1:MIN_MATCH_LEN) .EQ. MIN_MATCH)
		    CALL STR$TRIM( LOGICAL_NAME, LOGICAL_NAME, LOG_LEN)
		    IF (MATCH_LEN .EQ. 0 .OR.
     &			WCMATCH(MATCH(1:MATCH_LEN),LOGICAL_NAME(1:LOG_LEN)))
     &			THEN
			    IF (.NOT. MODIFY_ONE_REC('LOGICAL',LOG_CHANGES))
     &				RETURN
			    ANY_MATCHES = .TRUE.
		    ENDIF
		    READ (1,END=28000) CAMDEF_RECORD
C
		ENDDO			! end of logical name match loop
		UNLOCK (1)
C
28000		IF (.NOT. ANY_MATCHES) TYPE *, 'No logical name matches ',
     &			MATCH(1:MATCH_LEN)
	    ENDIF			! end of log./phy. if-then-else
C
	ENDDO				! end of list of match strings loop
	RETURN
C
	END

	INTEGER*4 FUNCTION MODIFY_ONE_REC( SEARCH_ORDER, LOG_CHANGES )
C
C This subroutine modifies a single record in the data base.  The record to be
C changed is passed via the common area CTS_MODIFY.  A call is made to a CLI
C routine for each part of the record which may be changed.  The calling
C parameters are:
C
C	SEARCH_ORDER	char. string indicating whether we are search via
C			logical name ('LOGICAL') or physical specification
C			('PHYSICAL')
C	LOG_CHANGES	is a logical variable indicating whether changes are
C			to be logged to the terminal
C
C
C The basic idea for making modifications is to test each possible MODIFY command
C switch.  When we find one which has been specified, we put its value into the
C string VALUE and trim it to find its real length.  If the switch will cause a
C change to the physical specifier, we patch the value into the string
C PHY_SPECIFIER (which contains the current physical specifier string) and
C pass it to CAM$PHYMOD.  That subroutine verifies the legality of the physical
C specifier and parses it into the character string CSU_NAME and the array
C PHY_MOD.  Thus, PHY_SPECIFIER and, together, CSU_NAME and PHY_MOD contain the
C most up-to-date physical specifier info.
C
	IMPLICIT NONE
C
	INTEGER*4 CAM$PHYMOD, CLI$GET_VALUE, CLI$PRESENT, SYS$CRELOG,
     &		SYS$DELLOG, SYS$TRNLOG, WCCOPY
	INTEGER*4 ISTAT
	LOGICAL LOG_CHANGES, LOG_NAM_CHNG, PHY_MOD_CHNG
	INTEGER*2 COLON_LOC, DOT_LOC, LOG_NAM_LEN, MATCH_LEN, NEW_LOG_LEN,
     &		OLD_LOG_LEN, OLD_PHY_LEN, PHY_SPEC_LEN, STATION_LOC,
     &		TEMP_NUM, VALUE_LEN
	CHARACTER MATCH*32, MIN_MATCH*32, NEW_LOG_NAM*32, OLD_LOG_NAM*32,
     &		OLD_PHY_SPEC*32, PHY_SPECIFIER*32, SEARCH_ORDER*(*), VALUE*40
	EXTERNAL CLI$_NEGATED
C
	INCLUDE 'CTSDEF'
	INCLUDE '($SSDEF)'
C
	COMMON /CTS_MODIFY/ MATCH, MATCH_LEN, CAMDEF_RECORD, CSU_NAME, PHY_MOD
C
	LOG_NAM_CHNG = .FALSE.
	PHY_MOD_CHNG = .FALSE.
	CALL MAKE_FULL_SPEC(CAMDEF_RECORD, PHY_SPECIFIER, PHY_SPEC_LEN)
	CALL STR$TRIM( OLD_LOG_NAM, LOGICAL_NAME, LOG_NAM_LEN)
	OLD_LOG_LEN = LOG_NAM_LEN
	OLD_PHY_SPEC = PHY_SPECIFIER
	OLD_PHY_LEN = PHY_SPEC_LEN
C
C May want to change the camac highway BRANCH
C
	IF (CLI$GET_VALUE( 'BRANCH', VALUE)) THEN
	    CALL STR$TRIM( VALUE, VALUE, VALUE_LEN)
	    PHY_SPECIFIER =
     &		PHY_SPECIFIER(1:2)//VALUE(1:VALUE_LEN)//PHY_SPECIFIER(4:)
	    CALL STR$TRIM(PHY_SPECIFIER, PHY_SPECIFIER, PHY_SPEC_LEN)
	    ISTAT = CAM$PHYMOD(PHY_SPECIFIER(1:PHY_SPEC_LEN),CSU_NAME,PHY_MOD)
	    IF (ISTAT) THEN
		PHY_MOD_CHNG = .TRUE.
	    ELSE
		TYPE *, 'Illegal branch'
		GOTO 89000
	    ENDIF
	ENDIF
C
C May want to replace or delete the COMMENT.
C
	IF (CLI$PRESENT('COMMENT')) CALL CLI$GET_VALUE('COMMENT',COMMENT)
	IF (CLI$PRESENT('COMMENT') .EQ. %LOC(CLI$_NEGATED)) COMMENT = ' '
C
C May want to change the camac CRATE number.
C
	IF (CLI$GET_VALUE( 'CRATE', VALUE)) THEN
	    CALL STR$TRIM( VALUE, VALUE, VALUE_LEN)
	    COLON_LOC = INDEX( PHY_SPECIFIER, ':')
	    PHY_SPECIFIER =
     &		PHY_SPECIFIER(1:3)//VALUE(1:VALUE_LEN)//
     &		PHY_SPECIFIER(COLON_LOC:)
	    CALL STR$TRIM(PHY_SPECIFIER, PHY_SPECIFIER, PHY_SPEC_LEN)
	    ISTAT = CAM$PHYMOD(PHY_SPECIFIER(1:PHY_SPEC_LEN),CSU_NAME,PHY_MOD)
	    IF (ISTAT) THEN
		PHY_MOD_CHNG = .TRUE.
	    ELSE
		TYPE *, 'Illegal crate'
		GOTO 89000
	    ENDIF
	ENDIF
C
C May want to replace the entire CSU
C
	IF (CLI$GET_VALUE('CSU',VALUE)) THEN
	    CALL STR$TRIM(VALUE,VALUE,VALUE_LEN)
	    IF (VALUE(VALUE_LEN:) .EQ. ':') VALUE_LEN = VALUE_LEN-1
	    COLON_LOC = INDEX(PHY_SPECIFIER,':')
	    PHY_SPECIFIER = VALUE(1:VALUE_LEN)//PHY_SPECIFIER(COLON_LOC:)
	    CALL STR$TRIM(PHY_SPECIFIER,PHY_SPECIFIER,PHY_SPEC_LEN)
	    ISTAT = CAM$PHYMOD(PHY_SPECIFIER(1:PHY_SPEC_LEN),CSU_NAME,PHY_MOD)
	    IF (ISTAT) THEN
		PHY_MOD_CHNG = .TRUE.
	    ELSE
		TYPE *,'Illegal csu'
		GOTO 89000
	    END IF
	END IF
C
C May want to change the camac HIGHWAY.
C
	IF (CLI$GET_VALUE( 'HIGHWAY', VALUE)) THEN
	    CALL STR$TRIM( VALUE, VALUE, VALUE_LEN)
	    PHY_SPECIFIER = VALUE(1:VALUE_LEN)//PHY_SPECIFIER(3:)
	    CALL STR$TRIM(PHY_SPECIFIER, PHY_SPECIFIER, PHY_SPEC_LEN)
	    ISTAT = CAM$PHYMOD(PHY_SPECIFIER(1:PHY_SPEC_LEN),CSU_NAME,PHY_MOD)
	    IF (ISTAT) THEN
		PHY_MOD_CHNG = .TRUE.
	    ELSE
		TYPE *, 'Illegal highway'
		GOTO 89000
	    ENDIF
	ENDIF
C
C May want to change the LOGICAL NAME.  The manner in which we do the rename 
C depends on whether we are searching modules in logical name or physical spec.
C order.  If we're doing the search by logical name, we do a wildcard rename.
C Otherwise, we simply replace the logical name with the one specified.
C
	IF (CLI$GET_VALUE('LOGICAL_NAME',VALUE)) THEN
	    CALL STR$TRIM(VALUE,VALUE,VALUE_LEN)
	    IF (SEARCH_ORDER .EQ. 'LOGICAL') THEN
		IF (.NOT. WCCOPY(MATCH(1:MATCH_LEN),
     &			LOGICAL_NAME(1:LOG_NAM_LEN), VALUE(1:VALUE_LEN),
     &			NEW_LOG_NAM, NEW_LOG_LEN)) THEN
		    TYPE *, 'Internal CTS error'
		    GOTO 89000
		ENDIF
	    ELSE
		IF (INDEX(VALUE(1:VALUE_LEN),'*') .GT. 0 .OR.
     &			INDEX(VALUE(1:VALUE_LEN),'%') .GT. 0) THEN
		    TYPE *, 'Illegal use of wildcard char. in logical name'
		    GOTO 89000
		ENDIF
		NEW_LOG_NAM = VALUE
		NEW_LOG_LEN = VALUE_LEN
	    ENDIF
	    IF (NEW_LOG_NAM .NE. LOGICAL_NAME) THEN
		ISTAT = SYS$TRNLOG(NEW_LOG_NAM(1:NEW_LOG_LEN),,
     &			NEW_LOG_NAM,,,6)
		IF (ISTAT .EQ. SS$_NOTRAN) THEN
		    LOGICAL_NAME = NEW_LOG_NAM
		    LOG_NAM_CHNG = .TRUE.
		ELSE IF (ISTAT .EQ. SS$_NORMAL .OR.
     &			 ISTAT .EQ. SS$_RESULTOVF) THEN
		    TYPE *, 'Logical name already in use'
		    GOTO 89100
	    	ELSE
		    CALL REPORT(' ', ISTAT)
		    GOTO 89000
		END IF
	    END IF
	END IF
C
C May want to change the REFERENCE UIC.
C
	ISTAT = CLI$PRESENT('REFERENCE_UIC')
	IF (ISTAT .OR. ISTAT .EQ. %LOC(CLI$_NEGATED)) THEN
	    CALL GET_REF_UIC(PHY_SPECIFIER,PHY_SPEC_LEN)
	    ISTAT = CAM$PHYMOD(PHY_SPECIFIER(1:PHY_SPEC_LEN),CSU_NAME,PHY_MOD)
	    IF (ISTAT) THEN
		PHY_MOD_CHNG = .TRUE.
	    ELSE
		TYPE *,'Illegal reference uic'
		GOTO 89000
	    END IF
	END IF
C
C May want to change the STATION number.
C
	IF (CLI$GET_VALUE('STATION',VALUE)) THEN
	    IF (VALUE(1:1) .NE. 'N') VALUE = 'N'//VALUE
	    STATION_LOC = INDEX(PHY_SPECIFIER,']')
	    IF (STATION_LOC .EQ. 0) STATION_LOC = INDEX(PHY_SPECIFIER,':')
	    PHY_SPECIFIER = PHY_SPECIFIER(1:STATION_LOC)//VALUE
	    CALL STR$TRIM(PHY_SPECIFIER,PHY_SPECIFIER,PHY_SPEC_LEN)
	    ISTAT = CAM$PHYMOD(PHY_SPECIFIER(1:PHY_SPEC_LEN),CSU_NAME,PHY_MOD)
	    IF (ISTAT) THEN
		PHY_MOD_CHNG = .TRUE.
		ELSE
	    TYPE *,'Illegal station'
		GOTO 89000
	    END IF
	END IF
C
C Done - rewrite the record
C
C ...first, incorporate any changes to the physical specifier into our copy
C    of the data base record (CAMDEF_RECORD).
C
	IF (PHY_MOD_CHNG) CALL MAKE_PHY_REC(CSU_NAME,PHY_MOD,CAMDEF_RECORD)
C
C ...if any change (other than to the comment string) was made, we must
C    update the system logical name table.  Note that interpretation of the
C    CRELOG status code depends on whether the logical name was changed.  Also
C    note that the old logical name need be deleted only when the name is
C    changed...
C
	IF (PHY_MOD_CHNG .OR. LOG_NAM_CHNG) THEN
	    CALL STR$TRIM( LOGICAL_NAME, LOGICAL_NAME, LOG_NAM_LEN)
	    ISTAT = SYS$CRELOG( %VAL(0), LOGICAL_NAME(1:LOG_NAM_LEN),
     &		PHY_SPECIFIER(1:PHY_SPEC_LEN),)
	    IF (LOG_NAM_CHNG) THEN
		IF (ISTAT .NE. SS$_NORMAL) GOTO 88000
		ISTAT = SYS$DELLOG( %VAL(0), OLD_LOG_NAM(1:OLD_LOG_LEN),)
		IF (ISTAT .EQ. SS$_NOLOGNAM) THEN
		    TYPE *, 'Warning - '//OLD_LOG_NAM(1:OLD_LOG_LEN)//
     &			' was not found in system logical name table'
		ELSE IF (ISTAT .NE. SS$_NORMAL) THEN
		    CALL REPORT( 'Error deleting '//OLD_LOG_NAM(1:OLD_LOG_LEN)
     &			//' from system logical name table: ', ISTAT)
		ENDIF
	    ELSE
		IF (ISTAT .EQ. SS$_NORMAL) THEN
		    TYPE *, 'Warning - '//OLD_LOG_NAM(1:OLD_LOG_LEN)//
     &			' was not found in system logical name table'
		ELSE IF (ISTAT .NE. SS$_SUPERSEDE) THEN
		    GOTO 88000
		ENDIF
	    ENDIF
	ENDIF
C
C ...create the updated record - the logical name is the primary key, so if
C    the logical name was changed we must delete the old record and write a
C    new one...
C
	IF (LOG_NAM_CHNG) THEN
	    DELETE (1)
	    WRITE (1) CAMDEF_RECORD
	ELSE
	    REWRITE (1) CAMDEF_RECORD
	END IF
C
C ...finally, if changes were made to the logical name or physical specification
C    and we were so requested, log the changes...
C
	IF (LOG_CHANGES .AND. (PHY_MOD_CHNG .OR. LOG_NAM_CHNG)) THEN
	    CALL MAKE_FULL_SPEC(CAMDEF_RECORD,PHY_SPECIFIER,PHY_SPEC_LEN)
	    TYPE *, ' ', OLD_LOG_NAM(1:OLD_LOG_LEN),
     &		' ('//OLD_PHY_SPEC(1:OLD_PHY_LEN)//')',' changed to ',
     &		LOGICAL_NAME(1:LOG_NAM_LEN),
     &		' ('//PHY_SPECIFIER(1:PHY_SPEC_LEN)//')'
	END IF
	GOTO 89100
C
C Error creating new logical name
C
88000	CALL REPORT('Error assigning logical name '//
     &		LOGICAL_NAME(1:LOG_NAM_LEN)//' - ',ISTAT)
C
C Exit
C
89000	MODIFY_ONE_REC = 0			! error return
	GOTO 89900
89100	MODIFY_ONE_REC = 1			! success
89900	RETURN
	END
