C+++
C	CTSSET.FOR
C
C This module contains the SET subroutine for the CAMAC Topology Supervisor.
C
C The SET command requires a CAMAC Support Unit (CSU or crate) as a
C parameter and can set the unit on-/off-line and enable/disable lam support.
C If lam support is enabled, the station number of the lam grader may be
C specified.
C
C LAM_SUPPORT=ENABLE and ONLINE may be specified in the same SET command, as
C may LAM_SUPPORT=DISABLE and OFFLINE.
C
C
C   Date	By		Comments
C 10/07/82	D.E.Greenwood	Source moved from CTS.FOR; lam support added to
C				command
C  1/03/82	D.E.Greenwood	Calls to REPORT changed to 2 arguments
C
C---
	SUBROUTINE SET
C
	CHARACTER CSU*10, LAM_STATION*10, SUPPORT_TYPE*20
	INTEGER*2 CSU_LEN, IOSB(4), LAMG_N, LAM_STA_LEN
	INTEGER*4 CAM$DABLAMS, CAM$ENLAMS, CAM$ERROR, ISTAT
	INTEGER*4 CAM$CONL, CAM$COFL, CLI$PRESENT, GET_KEYWORD_VAL
	PARAMETER Q=.TRUE., X=.TRUE.
	PARAMETER LAM_KEYWRDS='DISABLE/ENABLE/OFF/ON'
C
C Get Camac Support Unit (crate number)
C
	CALL CLI$GET_VALUE('CSU',CSU)
	CALL STR$TRIM(CSU,CSU,CSU_LEN)
C
C If LAM_SUPPORT qualifier was specified, make sure a legal value was given.
C
	IF (CLI$PRESENT('LAM_SUPPORT')) THEN
	   IF (.NOT. GET_KEYWORD_VAL('LAM_SUPPORT', LAM_KEYWRDS, SUPPORT_TYPE))
     1		GOTO 89000
	ENDIF
C
	ISTAT = 1
	IOSB(1) = 1
C
C Disable lam support.
C
	IF (CLI$PRESENT('LAM_SUPPORT')) THEN
	   IF (SUPPORT_TYPE .EQ. 'DISABLE' .OR. SUPPORT_TYPE .EQ. 'OFF') THEN
		ISTAT =  CAM$DABLAMS(CSU(1:CSU_LEN),IOSB)
		IF (CAM$ERROR(,,IOSB) .OR. .NOT. ISTAT) GOTO 80000
	   ENDIF
	ENDIF
C
C Set crate offline.
C
	IF (CLI$PRESENT('OFFLINE')) THEN
	   ISTAT = CAM$COFL(CSU(1:CSU_LEN),IOSB)
	   IF (CAM$ERROR(X,,IOSB) .OR. .NOT. ISTAT) GOTO 80000
	ENDIF
C
C Set crate online.
C
	IF (CLI$PRESENT('ONLINE')) THEN
	   ISTAT = CAM$CONL(CSU(1:CSU_LEN),IOSB)
	   IF (CAM$ERROR(X,,IOSB) .OR. .NOT. ISTAT) GOTO 80000
	ENDIF
C
C Enable Lam support.
C
	IF (CLI$PRESENT('LAM_SUPPORT')) THEN
	   IF (SUPPORT_TYPE .EQ. 'ENABLE' .OR. SUPPORT_TYPE .EQ. 'ON') THEN
		CALL CLI$GET_VALUE('STATION',LAM_STATION)
		IF (LAM_STATION(1:1) .EQ. 'N') LAM_STATION = LAM_STATION(2:)
		CALL STR$TRIM(LAM_STATION,LAM_STATION,LAM_STA_LEN)
		DECODE (LAM_STA_LEN,90000,LAM_STATION,ERR=88000) LAMG_N
		ISTAT = CAM$ENLAMS(CSU(1:CSU_LEN), %VAL(LAMG_N), IOSB)
		IF (CAM$ERROR(X,Q,IOSB) .OR. .NOT. ISTAT) GOTO 80000
	   ENDIF
	ENDIF
C
	GOTO 89000
C
C Error reporting
C
80000	IF (ISTAT) THEN
	  IF (CAM$ERROR(X,,IOSB)) CALL CAM$SHOW_STAT(IOSB)
	ELSE
	  CALL REPORT(' ', ISTAT)
	END IF
	GOTO 89000
C
88000	TYPE *, '%CTS-W-ILLSTA, illegal station number'
C
89000	RETURN
C
C
90000	FORMAT(I)
	END
