	INTEGER*4 FUNCTION WCMATCH(MATCH_STRING, STRING)
C
C This function compares two strings for matching patterns.  The first string
C (the "match_string") specifies the pattern and may contain the wildcard
C characters "*" and/or "%".  The function returns the following values:
C	1	if "string" is of the pattern specified by "match_string"
C	0	otherwise
C
C
C This routine is implemented essentially as a code-driven finite state machine,
C except that a little bit of additional "memory" is kept in the temporary
C variables M_TEMP and S_TEMP.  These variables retain the starting points for
C pattern checking after a "*" in the match string.  If a pattern match fails,
C the index variables can be quickly reset using the temporary variables.
C
C
C   Date	By		Comments
C 12/14/82	D.E.Greenwood	Original
C
C
	CHARACTER M_CHAR*1, MATCH_STRING*(*), STRING*(*)
	INTEGER*2 M_INDEX, MATCH_LEN, M_TEMP, S_INDEX, STR_LEN, S_TEMP
C
C
	M_INDEX = 0
	MATCH_LEN = LEN(MATCH_STRING)
	S_INDEX = 0
	STR_LEN = LEN(STRING)
C
C "State 1" - String is of form specified by match string, so far.  We check
C the strings on a character by character basis, until the end of a string
C or a "*" is reached.
C
10100	M_INDEX = M_INDEX + 1
	S_INDEX = S_INDEX + 1
	M_CHAR = MATCH_STRING(M_INDEX:M_INDEX)
	IF (M_INDEX .GT. MATCH_LEN .AND. S_INDEX .GT. STR_LEN) GOTO 80000
	IF (M_INDEX .GT. MATCH_LEN) GOTO 82000
	IF (M_CHAR .EQ. '*') GOTO 10200
	IF (S_INDEX .GT. STR_LEN) GOTO 82000
	IF (M_CHAR .EQ. '%' .OR. M_CHAR .EQ. STRING(S_INDEX:S_INDEX))
     &		GOTO 10100
	GOTO 82000
C
C "State 2" - Last match char was a "*"
C
10200	M_INDEX = M_INDEX + 1
	IF (M_INDEX .GT. MATCH_LEN) GOTO 80000
	M_CHAR = MATCH_STRING(M_INDEX:M_INDEX)
	IF (M_CHAR .EQ. '*') GOTO 10200
	IF (M_CHAR .NE. '%') GOTO 10400
C
C "State 3" - Last two match chars were "*%".  We treat that as if only the
C "%" had been specified, i.e., we match only the current string char.
C
	IF (S_INDEX .GT. STR_LEN) GOTO 82000
	GOTO 10100
C
C "State 4" - Last match char was a "*", this match char is not - we want to
C find a char in the string which equals this match char.
C
10400	IF (S_INDEX .GT. STR_LEN) GOTO 82000
	IF (M_CHAR .EQ. STRING(S_INDEX:S_INDEX)) THEN
	    M_TEMP = M_INDEX				! found - save starting
	    S_TEMP = S_INDEX				!  index of string which
	    GOTO 10500					!  may terminate "*" matching
	ENDIF
	S_INDEX = S_INDEX + 1
	GOTO 10400
C
C "State 5" - First match char after "*" is same as current string char.  We
C want to know if we have found a string which will terminate the wildcard match.
C The things which identify such a string are end of both strings or another
C wildcard char.
C
10500	M_INDEX = M_INDEX + 1
	S_INDEX = S_INDEX + 1
	IF (M_INDEX .GT. MATCH_LEN .AND. S_INDEX .GT. STR_LEN) GOTO 80000
	IF (M_INDEX .GT. MATCH_LEN) GOTO 10600
	M_CHAR = MATCH_STRING(M_INDEX:M_INDEX)
	IF (M_CHAR .EQ. '*') GOTO 10200
	IF (S_INDEX .GT. STR_LEN) GOTO 82000
	IF (M_CHAR .EQ. '%') GOTO 10100
	IF (M_CHAR .EQ. STRING(S_INDEX:S_INDEX)) GOTO 10500
C
C "State 6" - Failed to match string after "*" - restart test one char further
C in the string
C
10600	M_INDEX = M_TEMP
	S_INDEX = S_TEMP + 1
	M_CHAR = MATCH_STRING(M_INDEX:M_INDEX)
	GOTO 10400
C
C SUCCESS
C
80000	WCMATCH = 1
	GOTO 89900
C
C FAIL
C
82000	WCMATCH = 0
C
89900	RETURN
	END
