	program l2551
	implicit integer(a-z)
c++
c
c	Define program parameters
c
c--
	parameter l2551_nchan=12		! maximum LRS 2551 channel
	parameter mem_l=24			! memory format= 24 bit
c++
c
c	Input data buffer for LRS 2551
c
c--
	integer*4 l2551_data(l2551_nchan)
c++
c
c	Use a CHARACTER variable for the logical module name
c
c--
	character logmod*32


	type *,'LRS 2551 Module Exercisor'
c++
c
c	Set default CAMAC state for event flag and memory format.
c
c--
	status=cam$cstate(%val(0),,,mem_l)
	if (.not.status) then
		call lib$stop(%val(status))
	end if
c++
c
c	Prompt user for logical module name.
c
c--
	type 1999
1999	format(1x,'Enter LRS 2551 module name ',$)
	accept 1998,loglen,logmod
1998	format(q,a)
	if (loglen.eq.0) call exit
c++
c
c	Assign key to module for keyed module reference.
c
c--
	status=cam$assign(logmod(1:loglen),keyv)
	if (.not.status) then
		call lib$stop(%val(status))
	end if
c++
c
c	Initialize counter and the LRS 2551 scaler.
c
c--
	countl=0
	a=0
	f=9
	status=cam$piow(%descr(keyv),a,f)
	if (.not.cam$go(status,)) then
		call exit
	end if
c++
c
c	Q-Scan test loop.
c
c--
1001	continue
c++
c
c	Increment all 12 channels by 1 count.
c
c--
	f=25
	status=cam$piow(%descr(keyv),a,f)
	if (.not.cam$go(status,)) go to 1001
	countl=countl+1
c++
c
c	Perform a Q-Scan request to read back all 12 channels.
c	Note that this CAMAC request
c		1. Uses keyed module reference vs. direct module
c		   reference.
c		2. Uses the I/O status block in the CAMAC image block.
c		3. Uses the default memory format from the CAMAC image
c		   block(specified above via CAM$CSTATE).
c	
c--
	f=0
	status=cam$qscanw(%descr(keyv),a,f,l2551_nchan,l2551_data)
	if (.not.cam$go(status,)) go to 1001
c++
c
c	Verify all channels of data read from the LRS 2551.
c
c--
	call verify(countl,l2551_data,l2551_nchan)
	go to 1001
	end
	logical function cam$go(status,iosb)
	implicit integer(a-z)
	integer*4 iosb(*),liosb(2)
	logical*4 cam$error
	character response*4
c++
c
c	Assume failure by setting 'GO' false.
c
c--
	cam$go=.false.
c++
c
C	Test return status from CAMAC call. If failure indicated,
c	signal condition and prompt for action if control returns
c	after signal.
c
c--
	if (.not.status) then
		call lib$signal(%val(status))
		go to 1001
	end if
c++
c
c	If IOSB parameter specified as null, then sample the CAMAC
c	image block IOSB.
c--
	if (%loc(iosb).eq.0) then
		call cam$get_stat(liosb)
	else
		liosb(1)=iosb(1)
		liosb(2)=iosb(2)
	end if
c++
c
c	Test for any CAMAC errors. If so, then display nature of
c	the CAMAC error via CAM$SHOW_STAT and prompt for action.
c
c--
	if (cam$error(.true.,,liosb)) then
		call cam$show_stat(liosb)
		go to 1001
	end if
c++
c
c	Change assumed failure to success and return.
c
c--
	cam$go=.true.
	return
c++
c
c	Prompt for failure override response.
c
c--
1001	continue
	type 1999
1999	format(1x,'Override Failure? Y/[N] ',$)
	accept 1998,resp_len,response
1998	format(q,a)
	if (resp_len.eq.1 .and. 
	9	(response(1:1).eq.'y' .or.
	9	response(1:1).eq.'Y')) cam$go=.true.
	return
	end
	subroutine verify(countl,datal,nchan)
	implicit integer(a-z)
	integer*4 datal(*)
c++
c
c	Verify all channels of data in the input buffer(datal) aganist the
c	expected count value(countl). Data from a channel failing to match 
c	is displayed.
c
c--
	do i=1,nchan
	if (countl.ne.datal(i)) then
		type 1999,i,countl,datal(i)
		return
	end if
1999	format(1x,'Mismatch: Chan=',i,2x,i,' Expected; ',i,' Read')
	end do
	return
	end
