c+++
c Facility:
c	ORNL Fusion Energy Division
c	Tokamak Experimental Section
c
c Abstract:
c	This library contains a number of FORTRAN procedures enabling
c	the use of the VAX terminal driver control-c attention AST
c	capability. Specifically, these procedures support the folowing
c	types of operations:
c		Set a control-c attention AST,
c		Cancel a control-c attention AST,
c		Service a control-c AST and toggle a LOGICAL variable.
c		
c
c Author:
c	E.T. Blair, ORNL Computer Sciences Division
c
c Revision History:
c	V00-000, E.T. Blair, 28-OCT-1982
c	Initial development completed and package tested.
c
c--



c++
c
c	set_ctlc(ctlc_handler,handler_parameter)
c
c	Description:
c	A control-c AST is set for the logical device 'SYS$INPUT' using 
c	the procedure specified procedure address, 'ctlc_handler', as the
c	AST service procedure and the specified handler parameter,
c	'handler_parameter, as the AST parameter.
c--
	integer function set_ctlc(ctlc_handler,handler_parm)
	implicit integer(a-z)
	include '($iodef)/nolist'
	include '($ssdef)/nolist'
	include '($psldef)/nolist'

c++
c
c	control_c ast common data and parameters
c
c--
	common/ctlc_blk/ tt_chan,ast_state

c++
c
c	qiow data and parameters
c
c--
	parameter ctlc_efn=0			! qiow event flag for ctlc request
	integer*4 ctlc_iosb(2)
	character tt*9/'SYS$INPUT'/
c
c	If 'SYS$INPUT' is not a terminal device, abort the call.
c
	if (.not. term_dev(tt)) then
		setc_ctlc=ss$_cancel
		return
	end if
c++
c
c	If 'SYS$INPUT' channel not assigned then,assign channel 
c	and enable process ast delivery.
c
c--
	if (tt_chan.eq.0) then
		set_ctlc=sys$assign(tt,tt_chan,,)
		if (.not.set_ctlc) then
			return
		end if
		ast_state=sys$setast(%val(1))
	end if
c++
c
c	set control_c attention ast for sys$input
c
c--
	set_ctlc=sys$qiow(%val(ctlc_efn),%val(tt_chan),
	9	%val(io$_setmode .or. io$m_ctrlcast),
	9	ctlc_iosb,,,ctlc_handler,handler_parm,%val(psl$c_user),,,)
	if (set_ctlc) set_ctlc=ctlc_iosb(1)
	if (.not.set_ctlc) then
		call sys$dassgn(%val(tt_chan))
		if (ast_state.eq.ss$_wasclr) then
			call sys$setast(%val(0))
		end if
	end if
	return
	end
c++
c
c	can_ctlc()
c
c	Description:
c	Any pending control-c AST requests issued via the 'set_ctlc' procedure
c	call are cancelled.
c--
	integer function can_ctlc
	implicit integer(a-z)
	include '($iodef)/nolist'
	include '($ssdef)/nolist'
	include '($psldef)/nolist'

c++
c
c	control_c ast common data and parameters
c
c--
	common/ctlc_blk/ tt_chan,ast_state

c++
c
c	qiow data and parameters
c
c--
	parameter ctlc_efn=1			! qiow event flag for ctlc request
	integer*4 ctlc_iosb(2)
c++
c
c	Cancel control_c attention ast for sys$input
c
c--
	can_ctlc=sys$qiow(%val(ctlc_efn),%val(tt_chan),
	9	%val(io$_setmode .or. io$m_ctrlcast),
	9	ctlc_iosb,,,,,,,,)
	if (can_ctlc) can_ctlc=ctlc_iosb(1)
	return
	end
c++
c
c	ctlc_ast(ctlc_flag)
c
c	Description:
c	This procedure is a generic control-c AST service procedure. It
c	assumes that the control-c AST parameter is the address of a LOGICAL
c	variable and it's action in response to the control-c from the
c	terminal device is to toggle the state of the logical variable.
c	To utilize it, specify the following sequence in the calling program:
c		external ctlc_ast
c		logical*1 ctlc_flag
c		.
c		.
c		call set_ctlc(ctlc_ast,ctlc_flag)
c		
c
c--
	subroutine ctlc_ast(ctlc_flag)
	implicit integer(a-z)
	include '($ssdef)/nolist'
	logical*1 ctlc_flag

c++
c
c	Toggle state of logical variable
c
c--
	ctlc_flag=.not.ctlc_flag
	return
	end

c+++
c
c***Terminal device class routine.
c
c---

	logical function term_dev(dev_name)
	implicit integer(a-z)
	character dev_name*(*)
	include '($dcdef)/nolist'
c
c	getdvi item list
c
	parameter dvi$_devclass=4		! device class item code
	integer*4 dvi_itmlst(4),dvi_iosb(2)
	data dvi_itmlst(4)/0/
c
c	Assume not terminal device.
c
	term_dev = .false.
c
c	Set up GETDVI item list to sample the device class.
c
	dvi_itmlst(1) = ishft(dvi$_devclass,16) .or. 4
	dvi_itmlst(2) = %loc(dev_class)
	dvi_itmlst(3) = 0
c
c	Get device class.
c
	status = sys$getdvi(%val(dvi_efn),,dev_name,dvi_itmlst,
	9	dvi_iosb,,,,)
	call sys$waitfr(%val(dvi_efn))
	if (status) status = dvi_iosb(1)
	if (.not. status) then
		return
	end if
c
c	If class is 'TERMINAL', return .TRUE. indication.
c
	if (dev_class .eq. dc$_term) term_dev = .true.
	return
	end
