C++
C
C	This program is designed to demonstrate the use of the VAX CAMAC system
C	list mode access procedures. The program uses a Le Croy Systems 8801/16
C	memory module to exercise the CAM$DCLST, CAM$LPIO, CAM$LSTOP and 
C	CAM$EXECUTEW CAMAC library procedures. The program also uses the
C	CAM$ASSIGN and CAM$PIOW procedures. The program initializes the
C	LRS 8801 module, then enters a test loop to repetitively build an
C	command list and then execute it. Specifically, the program performs
C	the following steps.
C		1. Prompts for user input to direct execution.
C		2. Initializes the LRS 8801 module.
C		3. Declares(initializes) the execute list buffer via
C		   a CAM$DCLST call.
C		4. Generates a data buffer to write to the LRS 8801.
C		5. Generates a command frame to set the LRS 8801 MAP 
C		   register via a CAM$LPIO call.
C		6. Generates a command frame to write a data buffer to the
C		   LSR 8801 module via a CAM$LSTOP call.
C		7. Generates a command frame to reset the MAP via a
C		   CAM$LPIO call.
C		8. Generates a command frame to perform a dummy read
C		   from the 8801 via a CAM$LPIO call.
C		9. Executes the command list via a CAM$EXECUTEW call.
C		10. Verifies the input buffer read from the LRS 8801 module
C		    with the output buffer written to the LRS 8801.
C		11. Loop back to step 3 and repeat.
C
C
C	This represents one way to handle a case where the data in a command 
C	list changes and must be updated before executing the desired command 
C	sequence. It does not, however,	demonstrate the desired method for 
C	doing this!!! See LISTMODEB.FOR	for a better way to handle this.
C
C--
	PROGRAM L8801M4
	IMPLICIT INTEGER(A-Z)
	PARAMETER MAXB=4096				! MAX XFER SIZE(BYTES)
	PARAMETER MAX16=MAXB/2				! MAX XFER SIZE(I*2)
	PARAMETER MAX24=MAXB/4				! MAX XFER SIZE(I*4)
C++
C
C	EXECUTE AND REPLY LIST PARAMETERS
C
C--
	PARAMETER EXLHDR=10				! EXE LIST HDR LEN(I*2)		 
	PARAMETER CMAX=10				! MAX COMMAND FRAMES
	PARAMETER MAXEXE=EXLHDR+CMAX*3+MAX16		! EXE LIST LENGTH(I*2)
	PARAMETER MAXREP=MAX16+8			! REP LIST LENGTH(I*2)
C++
C
C	EXECUTE AND REPLY LIST DECLARATIONS
C	NOTE: EXECUTE LIST(EXELST) AND REPLY LIST(REPLST) ARE ZERO
C	RELATIVE ARRAYS IN ORDER TO ALLOW THE DIRECT USE OF FIB POINTERS
C	WITHOUT OFFSETING BY 1.
C
C--
	INTEGER*2 CAMOUT(MAX16),CAMINP(MAX16),EXELST(0:MAXEXE),REPLST(0:MAXREP)
C++
C
C	FRAME INFORMATION BLOCK(FIB) PARAMETERS
C
C--
	PARAMETER FIBLEN=4				! FIB STRUC LENGTH(I*2)
	PARAMETER FIB_W_CNDX=1				! FIB COMMAND INDEX OFFSET
	PARAMETER FIB_B_FCODE=2				! FIB FCODE OFFSET
	PARAMETER FIB_W_DNDX=3				! FIB DATA INDEX OFFSET
	PARAMETER FIB_W_DSIZ=4				! FIB DATA SIZE OFFSET
C++
C
C	FIB LIST FOR THE EXECUTE LIST
C	FIB ENTRIES ARE INDEXED BY THE COMMAND FRAME NUMBER
C	THE SYMBOLIC OFFSETS(DEFINED AS PARAMETERS) ARE USED TO REFERENCE
C	THE FIELDS OF THE FIB FOR A GIVEN COMMAND FRAME.
C
C--
	INTEGER*2 FIB_LIST(FIBLEN,CMAX)
C++
C
C	CAMAC STATUS BUFFER
C
C--
	INTEGER*4 CAMSTATUS(3),IOSB(2)
	EQUIVALENCE(STATUS,CAMSTATUS(1))
	EQUIVALENCE(IOSB,CAMSTATUS(2))

	LOGICAL*4 CAM$GO
	CHARACTER*32 LOGMOD
	DATA MAP/0/

C++
C
C	PROMPT USER FOR LRS 8801 LOGICAL NAME
C
C--
	TYPE 1999
1999	FORMAT(1X,'ENTER LRS 8801 MODULE ',$)
	ACCEPT 1998,LOGLEN,LOGMOD
1998	FORMAT(Q,A)
	IF (LOGLEN.EQ.0) CALL EXIT
C++
C
C	ASSIGN KEY TO LRS 8801 MODULE
C
C--
	STATUS=CAM$ASSIGN(LOGMOD,KEYV)
	IF (.NOT.STATUS) THEN
		CALL LIB$STOP(%VAL(STATUS))
	END IF
	TYPE 1991,LOGMOD(1:LOGLEN),KEYV
1991	FORMAT(1X,'MODULE: ',A,' KEY VALUE=',Z8)
C++
C
C	PROMPT USER FOR DESIRED MEMORY FORMAT(16/24 BIT)
C
C--
2001	CONTINUE
	TYPE 1997
1997	FORMAT(1X,'ENTER 16 OR 24 BIT MEMORY FORMAT ',$)
	ACCEPT 1996,MEM
1996	FORMAT(I)
	TCMAX=-1
	IF (MEM.EQ.16) TCMAX=MAX16
	IF (MEM.EQ.24) TCMAX=MAX24
	IF (TCMAX.LT.0) THEN
		TYPE 1994,MEM
		GO TO 2001
	END IF
1994	FORMAT(1X,'INVALID MEMORY FORMAT=',I)
C++
C
C	PROMPT USER FOR DESIRED TRANSFER COUNT
C
C--
3001	CONTINUE
	TYPE 1995,TCMAX
1995	FORMAT(1X,'ENTER TRANSFER COUNT;',I,' LIMIT ',$)
	ACCEPT	1996,TC
	IF (TC.LE.0 .OR. TC.GT.TCMAX) THEN
		TYPE 1993,TC
		GO TO 3001
	END IF
1993	FORMAT(1X,'INVALID TRANSFER COUNT=',I)
C++
C
C	INITIALIZE THE LRS 8801 MODULE
C	PERFORM MODULE CLEAR(A0,F9)
C	DISABLE EXTERNAL MEMORY PORT(A0,F19)
C	ENABLE AUTOINCREMENT MODE(A0,F25)
C
C--
	A=0
	F=9
	STATUS=CAM$PIOW(%DESCR(KEYV),A,F,,,IOSB)
	IF (.NOT.CAM$GO(CAMSTATUS)) CALL EXIT
	F=19
	STATUS=CAM$PIOW(%DESCR(KEYV),A,F,0,16,IOSB)
	IF (.NOT.CAM$GO(CAMSTATUS)) CALL EXIT
	F=25
	STATUS=CAM$PIOW(%DESCR(KEYV),A,F,,,IOSB)
	IF (.NOT.CAM$GO(CAMSTATUS)) CALL EXIT
D	TYPE 2998,'INIT COMPLETE'
D2998	FORMAT(1X,A)
D	ACCEPT 2999,MORE
D2999	FORMAT(I)
C++
C
C	INITIALIZE EXECUTE LIST BUFFER
C
C--
	CALL CAM$DCLST(EXELST,MAXEXE,MEM,FIB_LIST)
D	TYPE 2998,'DCLST'
D	ACCEPT 2999,MORE
C++
C
C***EXELST: FRAME 1
C	GENERATE LIST FRAME TO SET LRS 8801 MAP
C
C--
	F=18
	STATUS=CAM$LPIO(EXELST,%DESCR(KEYV),A,F,MAP)
D	CALL LIB$SIGNAL(%VAL(STATUS))
D	TYPE 2998,'WRITE MAP'
D	ACCEPT 2999,MORE
C++
C
C***EXELST: FRAME 2
C	GENERATE LIST FRAME TO WRITE DATA BUFFER TO LRS 8801
C
C--
	F=16
	STATUS=CAM$LSTOP(EXELST,%DESCR(KEYV),A,F,TC)
D	CALL LIB$SIGNAL(%VAL(STATUS))
D	TYPE 2998,'WRITE DATA'
D	ACCEPT 2999,MORE
C++
C
C***EXELST: FRAME 3
C	GENERATE LIST FRAME TO RESET LRS 8801 MAP
C
C--
	F=18
	STATUS=CAM$LPIO(EXELST,%DESCR(KEYV),A,F,MAP)
D	CALL LIB$SIGNAL(%VAL(STATUS))
D	TYPE 2998,'READ MAP'
D	ACCEPT 2999,MORE
C++
C
C***EXELST: FRAME 4
C	GENERATE LIST FRAME TO PERFORM DUMMY READ FROM LRS 8801
C
C--
	F=2
	STATUS=CAM$LPIO(EXELST,%DESCR(KEYV),A,F)
D	CALL LIB$SIGNAL(%VAL(STATUS))
D	TYPE 2998,'DUMMY READ'
D	ACCEPT 2999,MORE
C++
C
C***EXELST: FRAME 5
C	GENERATE LIST FRAME TO READ BACK DATA PATTERN FROM LRS 8801
C
C--
	STATUS=CAM$LSTOP(EXELST,%DESCR(KEYV),A,F,TC)
D	CALL LIB$SIGNAL(%VAL(STATUS))
D	TYPE 2998,'READ DATA'
D	ACCEPT 2999,MORE
4001	CONTINUE
C++
C
C	GENERATE DATA PATTERN TO WRITE TO LRS 8801
C
C--
	IF (MEM.EQ.16) CALL GEN16(CAMOUT,TC)
	IF (MEM.EQ.24) CALL GEN24(CAMOUT,TC)
C
C	UPDATE DATA FOR FRAME 2 OF EXECUTE LIST
C
C--
	CALL CAM$WDATA(EXELST,2,CAMOUT)
C++
C
C	EXECUTE COMMAND LIST
C
C--
	STATUS=CAM$EXECUTEW(EXELST,REPLST,MAXREP,IOSB)
D	TYPE 2998,'EXECUTE'
D	ACCEPT 2999,MORE
	IF (.NOT.CAM$GO(CAMSTATUS)) CALL EXIT
C++
C
C	RETRIEVE READ DATA FOR FRAME 5 FROM REPLY LIST
C
C--
	CALL CAM$RDATA(EXELST,5,REPLST,CAMINP)
C++
C
C	VERIFY CAMAC DATA READ FROM LRS 8801 WITH DATA WRITTEN TO
C	LRS 8801.
C
C--
	WC=TC
	IF (MEM.EQ.24) WC=WC*2
	CALL VERIFY(CAMOUT,CAMINP,WC)
	GO TO 4001
	END
	LOGICAL FUNCTION CAM$GO(CAMSTATUS)
	IMPLICIT INTEGER(A-Z)
	INTEGER*4 CAMSTATUS(*)
	LOGICAL*4 CAM$X,CAM$ERROR
C++
C
C	ASSUME FAILURE
C
C--
	CAM$GO=.FALSE.
C++
C
C	CHECK FOR SYSTEM ERROR
C
C--
	IF (.NOT.CAMSTATUS(1)) THEN
		CALL LIB$SIGNAL(%VAL(CAMSTATUS(1)))
		RETURN
	END IF
C++
C
C	CHECK FOR CAMAC TRANSMISSION ERROR.
C	NOTE THAT THE STATE OF X AND Q RESPONSE ARE
C	"DON'T CARE" IN THIS CAM$ERROR CALL.
C
C--
	IF (CAM$ERROR(,,CAMSTATUS(2))) THEN
		CALL CAM$SHOW_STAT(CAMSTATUS(2))
		RETURN
	END IF
C++
C
C	CHECK FOR CAMAC X RESPONSE.
C
C--
	IF (.NOT.CAM$X(CAMSTATUS(2))) THEN
		CALL CAM$SHOW_STAT(CAMSTATUS(2))
		RETURN
	END IF
C++
C
C	CHANGE TO SUCCESS
C
C--
	CAM$GO=.TRUE.
	RETURN
	END
	SUBROUTINE GEN16(CAMOUT,TC)
	IMPLICIT INTEGER(A-Z)
	INTEGER*2 CAMOUT(1)
	INTEGER*2 TPAT16/'3333'X/
C++
C
C	GENERATE TEST PATTERN INTO SPECIFIED INTEGER*2 BUFFER 
C	ACCORDING TO THE LENGTH SPECIFIED BY THE TRANSFER COUNT(TC). 
C	ALL WORDS OF THE TEST PATTERN RECEIVE THE SAME VALUE ON A
C	GIVEN ITERATION. THE PATTERN IS INVERTED AFTER THE BUFFER
C	IS LOADED.
C
C--
	DO TNDX=1,TC
	CAMOUT(TNDX)=TPAT16
	END DO
	TPAT16=-TPAT16-1
	RETURN
	END
	SUBROUTINE GEN24(CAMOUT,TC)
	IMPLICIT INTEGER(A-Z)
	INTEGER*4 CAMOUT(1)
	DATA TPAT24/'3333'X/
	DATA LOW16/'FFFF'X/
C++
C
C	GENERATE TEST PATTERN INTO THE SPECIFIED INTEGER*4 BUFFER ACCORDING
C	TO THE LENGTH SPECIFIED BY THE TRARANSFER COUNT(TC). ALL ELEMENTS OF 
C	THE BUFFER RECEIVE THE SAME PATTERN ON A GIVEN ITERATION. THE PATTERN
C	IS INVERTED AFTER THE BUFFER IS LOADED.
C
C--
	DO TNDX=1,TC
	CAMOUT(TNDX)=TPAT24
	END DO
	TPAT24=(-TPAT24-1) .AND. LOW16
	RETURN
	END
	SUBROUTINE VERIFY(OUTDAT,INPDAT,WC)
	IMPLICIT INTEGER(A-Z)
	INTEGER*2 OUTDAT(*),INPDAT(*)
C++
C
C	VERIFY EACH WORD OF THE SPECIFIED OUTPUT DATA BUFFER(OUTDAT)
C	AGANIST EACH WORD OF THE INPUT DATA BUFFER(INPDAT) ACCORDING
C	TO THE LENGTH SPECIFIED BY THE WORD COUNT(WC).
C
C--
	DO NDX=1,WC
	IF (INPDAT(NDX).NE.OUTDAT(NDX)) THEN
		TYPE 1999,NDX,OUTDAT(NDX),INPDAT(NDX)
		RETURN
	END IF
	END DO
1999	FORMAT(1X,'NDX=',I,' OUTPUT=',Z4,' INPUT=',Z4)
	RETURN
	END
