C+++
C
C	Procedure:
C	CAM$CAMAC(MOD_ID,A,F,TC,DATA,MEM)
C
C	Description:
C	This procedure implements a general CAMAC call. This call
C	offers none of the more sophisticated features of the direct CAMAC
C	calls. It does offer the advantage of being particularly simple to
C	use. The module identitier(MOD_ID) must either be the module
C	logical name descriptor or the module key descriptor(i.e. a CHARACTER 
C	type variable containing the CAMAC logical module name or an INTEGER*4
C	variable containing the module key assigned by CAM$ASSIGN. The CAMAC
C	subaddress(A) and function code(F) must be specified. The remaining
C	arguments may be specified as null or actual parameters. If the transfer
C	count(TC) is specified as null or has a value of 1, a programmed I/O 
C	operation is performed. If greater than 1, a Stop mode transaction
C	is performed. CAMAC status is analyzed for any generic anomolous
C	conditions and if any are detected, appropriate action is taken.
C
C	Parameters:
C	MOD_ID is the address of a CHARACTER string descriptor containing the
C		logical module name or an I*4 descriptor containing the module
C		key assigned by CAM$ASSIGN.
C	A is the address of a byte containing the CAMAC sudabbress.
C	F is the address of a byte containing the CAMAC function code.
C	TC is the address of a word containing the transfer count. If specified
C		as null or with a value of 1, a programmed I/O operation is
C		requested. If greater than 1, a stop mode transaction is
C		requested.
C	DATA is the address of a buffer containing/to receive CAMAC data. May
C		be specified as null for CAMAC control function codes.
C	MEM is the address of a byte containing a value of 16 to indicate
C		16-bit memory format or 24 to indicate 24-bit memory format
C		for the CAMAC transaction. May be specified as null, in which
C		case, the default memory format from the CAMAC Image Block is
C		used.
C
C
C---
	INTEGER FUNCTION CAM$CAMAC(MOD_ID,A,F,TC,DATA,MEM)
	IMPLICIT INTEGER(A-Z)
	INTEGER*2 TC
	IF (%LOC(TC).NE.0 .AND. TC.GT.1) THEN
		STATUS=CAM$STOPW(MOD_ID,A,F,TC,DATA,MEM)
	ELSE
		STATUS=CAM$PIOW(MOD_ID,A,F,DATA,MEM)
	END IF
	IF (.NOT.STATUS) THEN
		CALL LIB$STOP(%VAL(STATUS))
	END IF
	IF (CAM$ERROR(.TRUE.,)) THEN
		CALL CAM$SHOW_STAT
	END IF
	RETURN
	END
