!
!			  COPYRIGHT (c) 1982 BY
!	      MOSTEK CORPORATION, CARROLLTON TX
!
! This software is furnished under a license and may be used and  copied
! only  in  accordance  with  the  terms  of  such  license and with the
! inclusion of the above copyright notice.  This software or  any  other
! copies  thereof may not be provided or otherwise made available to any
! other person.  No title to and ownership of  the  software  is  hereby
! transferred.
!
! The information in this software is subject to change  without  notice
! and  should  not  be  construed  as  a commitment by MOSTEK
! CORPORATION.
!
! MOSTEK assumes no responsibility for the use or  reliability  of  its
! software on equipment which is not supplied by MOSTEK.
!

!++
! FACILITY:
!   MOSTEK CORPORATION
!
! ABSTRACT: CRT_AXIS  an X or Y axis on a VT-100 compatible terminal
!   
!
! ENVIRONMENT: VAX/VMS unpriv
!   
!
! AUTHOR: Gary Smith, CREATION DATE: 3/14/1983
!
! MODIFIED BY:
!
!--
!
! TABLE OF CONTENTS:
!
!
! INCLUDE FILES: none
!
!
! MACROS: none
!
!
! EQUATED SYMBOLS: none
!
!
! EXTERNAL REFERENCES:
!

!++
! FUNCTIONAL DESCRIPTION: CRT_AXIS takes the information and produces an
!                         axis on the screen
!
!
!
! FORMAL PARAMETERS:
!        X_ORG      - The X coordinate of the axis origin.
!        Y_ORG      - The Y coordinate of the axis origin.
!        TITLE      - The label associated with this axis.
!        AXIS_LEN   - The length of the axis (in inches).
!        ORIENT     - The orientation of the axis (either 'H' or 'V')
!        AXIS_MIN   - The value of the lowest point on the axis.
!        AXIS_DELTA - The axis increment per inch.
!
!
!
! IMPLICIT INPUTS: none
!
!
!
! IMPLICIT OUTPUTS: a plot on the screen
!
!
!
! ROUTINE VALUE and
! COMPLETION CODES: none
!
!
!
! SIDE EFFECTS: none to date
!
!
!
!--
      SUBROUTINE CRT_AXIS(X_ORG,Y_ORG,TITLE,AXIS_LEN,ORIENT,
     .     AXIS_MIN,AXIS_DELTA,FORM)

      INTEGER HOURS
      CHARACTER*(*) TITLE, FORM, ORIENT*1
      REAL X_ORG, Y_ORG, AXIS_LEN, AXIS_MIN, AXIS_DELTA, FP_LABEL
      LOGICAL HOUR
C----------------------------------------------------------------------*
C      COMMON BLOCK                                                    *
C----------------------------------------------------------------------*
      COMMON /SYSINFO/ IX_NEW,IY_NEW,X_ORIGIN,Y_ORIGIN,SCALE_FACTOR,
     .     X_RESOL,Y_RESOL,X_CURRENT,Y_CURRENT
C----------------------------------------------------------------------*
C      LOCAL VARIABLES                                                 *
C----------------------------------------------------------------------*
      CHARACTER*9 TICK_MARK, AXIS_SYMB
      CHARACTER*7 LABEL      
      REAL DELTA_X, DELTA_Y, X_OFFSET, Y_OFFSET, X_LABEL, Y_LABEL
      INTEGER NUM_STEPS, NUM_BETWEEN
      DATA RND /1.0005/, SMALL /1.E-30/
C...... First set up the parameters that depend upon orientation
      HOUR = .FALSE.
      IF(ORIENT.EQ.'H') THEN
        DELTA_X = 1.
        DELTA_Y = 0.
        X_OFFSET = -0.4
        Y_OFFSET = -0.3875
        X_LABEL = 0.5 * (AXIS_LEN - .1*LENGTH(TITLE))
        Y_LABEL = -1.0
        TICK_MARK = 'UPPER_T'
        AXIS_SYMB = 'SCAN5'
        NUM_STEPS = AXIS_LEN
        NUM_BETWEEN = 9
        HOUR = ( INDEX(FORM, '$HOUR') .NE. 0 )
        IF ( HOUR ) FORM = '(F5.2)'
      ELSE
        DELTA_X = 0.
        DELTA_Y = 1.
        X_OFFSET = -1.
        Y_OFFSET = 0.
        X_LABEL = -1.
        Y_LABEL = AXIS_LEN+2.
        TICK_MARK = 'RIGHT_T'
        AXIS_SYMB = 'V_BAR'
        NUM_STEPS = AXIS_LEN
        NUM_BETWEEN = 1
        END IF
        FP_LABEL = AXIS_MIN - AXIS_DELTA
C...... Now set up the special variables needed
      AXIS_LABEL = AXIS_MIN
      X_AXIS = X_ORG
      Y_AXIS = Y_ORG
C...... Plot a CROSS at the axis origin
      CALL CRT_SYMBOL_SPECIAL(X_AXIS,Y_AXIS,'CROSS')
C...... Run through the number of tick marks
      DO 100 I = 1,NUM_STEPS
        FP_LABEL = FP_LABEL + AXIS_DELTA
        IF ( HOUR ) THEN
           FP_SAVE = FP_LABEL
           HOURS = FP_LABEL / 60
           FMINUTES = FP_LABEL - HOURS * 60
           HOURS = JMOD(HOURS, 24)
           FP_LABEL = HOURS + FMINUTES / 100.0 + 0.0001
           AXIS_LABEL = FP_LABEL
         END IF
        LABEL = '         '
        ENCODE (7,FORM,LABEL)  AXIS_LABEL
        IF ( HOUR ) THEN
           LABEL(3:3) = ':'
           FP_LABEL = FP_SAVE
        END IF
        CALL CRT_SYMBOL(X_AXIS+X_OFFSET,Y_AXIS+Y_OFFSET,LABEL)
C...... Between tick marks plot the axis symbol
        DO 50 J = 1,NUM_BETWEEN
          X_AXIS = X_AXIS + .1*DELTA_X
          Y_AXIS = Y_AXIS + .3875*DELTA_Y
          CALL CRT_SYMBOL_SPECIAL(X_AXIS,Y_AXIS,AXIS_SYMB)
 50       CONTINUE
        X_AXIS = X_AXIS + .1*DELTA_X
        Y_AXIS = Y_AXIS + .3875*DELTA_Y
        AXIS_LABEL = AXIS_LABEL + AXIS_DELTA
        CALL CRT_SYMBOL_SPECIAL(X_AXIS,Y_AXIS,TICK_MARK)
100     CONTINUE
C...... Clean up at the end
      LABEL = '         '
      IF ( HOUR ) THEN
         FP_LABEL = FP_LABEL + AXIS_DELTA
         HOURS = FP_LABEL / 60
         FMINUTES = FP_LABEL - HOURS * 60
         HOURS = JMOD(HOURS, 24)
         FP_LABEL = HOURS + FMINUTES / 100.0 + 0.0001
         AXIS_LABEL = FP_LABEL
      END IF
      ENCODE (7,FORM,LABEL)  AXIS_LABEL
        IF ( HOUR ) THEN
           LABEL(3:3) = ':'
           FP_LABEL = FP_SAVE
        END IF
      CALL CRT_SYMBOL(X_AXIS+X_OFFSET,Y_AXIS+Y_OFFSET,LABEL)
      CALL CRT_SYMBOL(X_ORG+X_LABEL, Y_ORG+Y_LABEL,
     .     TITLE(1:LENGTH(TITLE)))
      RETURN
      END
