!
!			  COPYRIGHT (c) 1982 BY
!	      MOSTEK CORPORATION, CARROLLTON TX
!
! This software is furnished under a license and may be used and  copied
! only  in  accordance  with  the  terms  of  such  license and with the
! inclusion of the above copyright notice.  This software or  any  other
! copies  thereof may not be provided or otherwise made available to any
! other person.  No title to and ownership of  the  software  is  hereby
! transferred.
!
! The information in this software is subject to change  without  notice
! and  should  not  be  construed  as  a commitment by MOSTEK
! CORPORATION.
!
! MOSTEK assumes no responsibility for the use or  reliability  of  its
! software on equipment which is not supplied by MOSTEK.
!

!++
! FACILITY:
!   MOSTEK CORPORATION
!
! ABSTRACT: Display the components collected by MOUSE
!   
!
! ENVIRONMENT: VAX VMS unpriv
!   
!
! AUTHOR: Gary Smith, CREATION DATE: 8/23/1982
!
! MODIFIED BY:
!
!--
!
! TABLE OF CONTENTS:
!
!
! INCLUDE FILES: statsubs.for
!                mousesubs.for
!                defrec.for
!                catdef.for
!
!
! MACROS: none
!
!
! EQUATED SYMBOLS: see statsubs.for
!                      mousesubs.for
!                      defrec.for
!                      catdef.for
!
!
! EXTERNAL REFERENCES: read_nest, MK$LENGTH
!

!++
! FUNCTIONAL DESCRIPTION: DISPLAY puts the information gleaned from
!                         the RATSNEST file and diplays it to the user
!                         in a hopefully comprehensible form. The RATSNEST
!                         file is keyed by time. DISPLAY uses start_time as
!                         the beginning with keyed read. It then reads 
!                         sequentially until the key returned is greater
!                         than the end_time.
!
!
!
! FORMAL PARAMETERS: start_time  -> the beginning key to start reading
!                    end_time    -> the end point of reading
!                    display_op  -> the user's choice of options
!
!
!
! IMPLICIT INPUTS: info read from the RATSNEST file
!
!
!
! IMPLICIT OUTPUTS: a coherent display of the gathered information
!
!
!
! ROUTINE VALUE and
! COMPLETION CODES: none
!
!
!
! SIDE EFFECTS: none to date
!
!
!
!--
          SUBROUTINE DISPLAY( start_time, end_time, display_op )

          include 'mousesubs.for'
          include 'statsubs.for'
          include 'defrec.for'
          include 'catdef.for'

          common / io_com / in_unit, out_unit, batch
          common / category / cat_state, cat_title, cat_len

          character *  8 per_sec
          character * 16 start_time, end_time, nest_key
          character * 16 sav_start_time
          character * 23 begin, finish
          character * 23 key_to_date
          character * 63 dis_title(max_cat)
          character * 63 cat_title(rec_size)

          logical batch

          integer display_op, count, dif_time, deltat, current_val
          integer in_unit, out_unit, column, true_length
          integer cat_state(max_cat,max_ent), cat_len(rec_size)
          integer max(max_ent), min(max_ent)
          integer rec_sizes(max_cat)

          real * 8 set(max_ent), bar(max_ent)

          data dis_title( 1) / 'CPU Statistics' /
          data dis_title( 2) / 'File Control Statistics' /
          data dis_title( 3) / 'Miscellaneous Statistics' /
          data dis_title( 4) / 'Memory Statistics' /
          data dis_title( 5) / 'Page Fault Statistics' /
          data dis_title( 6) 
     .         / 'Number of Processes Per Scheduler State' /
          data dis_title( 7) / 'Swapping Statistics' /
          data dis_title( 8) / 'NOT IMPLEMENTED YET' /
          data dis_title( 9) / 'DECnet Statistics' /
          data dis_title(10) / 'ENQ/DEQ Statistics' /
          data dis_title(11) / 'System Response' /
          data dis_title(12) / 'Non-paged Pool Statistics' /

          data rec_sizes( 1) / cpu_size      /
          data rec_sizes( 2) / fcp_size      /
          data rec_sizes( 3) / misc_size     /
          data rec_sizes( 4) / mem_size      /
          data rec_sizes( 5) / pgflt_size    /
          data rec_sizes( 6) / sch_size      /
          data rec_sizes( 7) / swap_size     /
          data rec_sizes( 8) / time_size     /
          data rec_sizes( 9) / net_size      /
          data rec_sizes(10) / lock_size     /
          data rec_sizes(11) / response_size /
          data rec_sizes(12) / pool_size     /

          data per_sec / ' Per Sec' /

10        format( /, t<column>, A, / )

120       format( /, t36, 'Rate', T47, 'Max', T57, 'Min',
     .            t67, 'Std', / )

130       format(' ', A, t30, f10.2, 
     .             t40, i10, t50, i10, t60, f10.2 )

140       format( t15, a, ' to ', a )

150       format( /, t33, 'Average', T47, 'Max', T57, 'Min',
     .            t67, 'Std', / )

160       format( / )

170       format(' ', A, t30, f10.2, 
     .             t40, f10.2, t50, f10.2, t60, f10.2 )

          nest_key = start_time

!++
! do the calculations to put out the msg and center it 
!--

          column = 39 - nint( real( MK$LENGTH(dis_title(display_op))
     .               / 2 ) )
          true_length = MK$LENGTH(dis_title(display_op))
          write( out_unit, 10) dis_title(display_op)(1:true_length)

!++
! run thru all the values until we either run outside the range of 
! the times or until we hit EOF. Note the special casing of response
!--

          do i = 1, max_ent
             set(i) = 0.0
             bar(i) = 0.0
             max(i) = 0.0
             min(i) = maxint
          end do

          count = 0
          call read_nest( keyed_read, nest_key, ios )
          sav_start_time = nest_key
          begin = key_to_date( nest_key )
          response_max = 0.0
          response_min = maxint

          do while ( key_compare(end_time, nest_key) .eq. greater )
             do i = 1, rec_sizes(display_op)
                if ( cat_state(display_op,1) .eq. c_response ) then
                   response = real(stats_rec(cat_state(display_op,1)))
     .                         / 100.0
                   set(i) = set(i) + response
                   bar(i) = bar(i) + response ** 2
                   if ( response .gt. response_max ) 
     .                response_max = response
                   if ( response .lt. response_min )
     .                response_min = response
                else
                   current_val = stats_rec(cat_state(display_op,i))
                   set(i) = set(i) + current_val
                   bar(i) = bar(i) + real(current_val) ** 2
                   if ( current_val .gt. max(i) ) 
     .                max(i) = current_val
                   if ( current_val .lt. min(i) )
     .                min(i) = current_val
               end if
             end do
             count = count + 1
             call read_nest( seq_read, nest_key, ios )
             if ( ios .eq. eof ) goto 20
          end do

!++
! all thru with math, now write it out
!--
          
20        continue

          finish = key_to_date( nest_key )
          dif_time = deltat( sav_start_time, nest_key )

          write( out_unit, 140 ) begin, finish

          goto ( 100, 200,  300,  400,  500,  600, 700,
     .           800, 900, 1000, 1100, 1200      ) display_op

!++
! command was display the time spent in the different cpu modes
!--

100       continue


          write( out_unit, 150 )

          write( out_unit, 130 ) cat_title(c_kernel),
     .                           set(kernel) / count,
     .                           max(kernel),
     .                           min(kernel),
     .                           std( bar(kernel),
     .                                set(kernel),count )

          write( out_unit, 130 ) cat_title(c_super),
     .                           set(super) / count,
     .                           max(super),
     .                           min(super),
     .                           std( bar(super),
     .                                set(super),count )

          write( out_unit, 130 ) cat_title(c_exec),
     .                           set(exec) / count,
     .                           max(exec),
     .                           min(exec),
     .                           std( bar(exec),
     .                                set(exec),count )

          write( out_unit, 130 ) cat_title(c_user),
     .                           set(user) / count,
     .                           max(user),
     .                           min(user),
     .                           std( bar(user),
     .                                set(user),count )

          write( out_unit, 130 ) cat_title(c_int),
     .                           set(int) / count,
     .                           max(int),
     .                           min(int),
     .                           std( bar(int),
     .                                set(int),count )

          write( out_unit, 130 ) cat_title(c_compat),
     .                           set(compat) / count,
     .                           max(compat),
     .                           min(compat),
     .                           std( bar(compat),
     .                                set(compat),count )

          write( out_unit, 130 ) cat_title(c_idle),
     .                           set(idle) / count,
     .                           max(idle),
     .                           min(idle),
     .                           std( bar(idle),
     .                                set(idle),count )
          write ( out_unit, 160 )

          write( out_unit, 130 ) cat_title(c_bjobcnt),
     .                           set(bjobcnt) / count,
     .                           max(bjobcnt),
     .                           min(bjobcnt),
     .                           std( bar(bjobcnt),
     .                                set(bjobcnt),count )

          write( out_unit, 130 ) cat_title(c_ijobcnt),
     .                           set(ijobcnt) / count,
     .                           max(ijobcnt),
     .                           min(ijobcnt),
     .                           std( bar(ijobcnt),
     .                                set(ijobcnt),count )

          
          return

!++
! display FCP stats
!--

200       continue

          write( out_unit, 150 )

          write( out_unit, 130 ) cat_title(c_call)
     .                           (1:cat_len(c_call)) // per_sec,
     .                           set(fcp_calls) / dif_time,
     .                           max(fcp_calls),
     .                           min(fcp_calls),
     .                           std( bar(fcp_calls),
     .                                set(fcp_calls),count )

          write( out_unit, 130 ) cat_title(c_cache)
     .                           (1:cat_len(c_cache)) // per_sec,
     .                           set(fcp_cache) / dif_time,
     .                           max(fcp_cache),
     .                           min(fcp_cache),
     .                           std( bar(fcp_cache),
     .                                set(fcp_cache),count )

          write( out_unit, 130 ) cat_title(c_cpu)
     .                           (1:cat_len(c_cpu)) // per_sec,
     .                           set(fcp_cpu) / dif_time,
     .                           max(fcp_cpu),
     .                           min(fcp_cpu),
     .                           std( bar(fcp_cpu),
     .                                set(fcp_cpu),count )

          write( out_unit, 130 ) cat_title(c_read)
     .                           (1:cat_len(c_read)) // per_sec,
     .                           set(fcp_read) / dif_time,
     .                           max(fcp_read),
     .                           min(fcp_read),
     .                           std( bar(fcp_read),
     .                                set(fcp_read),count )

          write( out_unit, 130 ) cat_title(c_write)
     .                           (1:cat_len(c_write)) // per_sec,
     .                           set(fcp_write) / dif_time,
     .                           max(fcp_write),
     .                           min(fcp_write),
     .                           std( bar(fcp_write),
     .                                set(fcp_write),count )

          return

!++
! display misc stats
!--

300      continue

         write( out_unit, 120 )

         write( out_unit, 130 ) cat_title(c_dirio)
     .                          (1:cat_len(c_dirio)) // per_sec,
     .                          set(dirio) / dif_time,
     .                          max(dirio),
     .                          min(dirio),
     .                          std( bar(dirio),
     .                               set(dirio),count )

         write( out_unit, 130 ) cat_title(c_bufio)
     .                          (1:cat_len(c_bufio)) // per_sec,
     .                          set(bufio) / dif_time,
     .                          max(bufio),
     .                          min(bufio),
     .                          std( bar(bufio),
     .                               set(bufio),count )

         write( out_unit, 130 ) cat_title(c_lognam)
     .                          (1:cat_len(c_lognam)) // per_sec,
     .                          set(lognam) / dif_time,
     .                          max(lognam),
     .                          min(lognam),
     .                          std( bar(lognam),
     .                               set(lognam),count )

         write( out_unit, 130 ) cat_title(c_mbreads)
     .                          (1:cat_len(c_mbreads)) // per_sec,
     .                          set(mbreads) / dif_time,
     .                          max(mbreads),
     .                          min(mbreads),
     .                          std( bar(mbreads),
     .                               set(mbreads),count )

         write( out_unit, 130 ) cat_title(c_mbwrites)
     .                          (1:cat_len(c_mbwrites)) // per_sec,
     .                          set(mbwrites) / dif_time,
     .                          max(mbwrites),
     .                          min(mbwrites),
     .                          std( bar(mbwrites),
     .                               set(mbwrites),count )

         write( out_unit, 130 ) cat_title(c_opens)
     .                          (1:cat_len(c_opens)) // per_sec,
     .                          set(opens) / dif_time,
     .                          max(opens),
     .                          min(opens),
     .                          std( bar(opens),
     .                               set(opens),count )

         write( out_unit, 130 ) cat_title(c_turn)
     .                          (1:cat_len(c_turn)) // per_sec,
     .                          set(turn) / dif_time,
     .                          max(turn), 
     .                          min(turn),
     .                          std( bar(turn),
     .                               set(turn),count )

         write( out_unit, 130 ) cat_title(c_split)
     .                          (1:cat_len(c_split)) // per_sec,
     .                          set(split) / dif_time,
     .                          max(split),
     .                          min(split),
     .                          std( bar(split),
     .                               set(split),count )

         write( out_unit, 130 ) cat_title(c_hit)
     .                          (1:cat_len(c_hit)) // per_sec,
     .                          set(hit) / dif_time,
     .                          max(hit),
     .                          min(hit),
     .                          std( bar(hit),
     .                               set(hit),count )

         write( out_unit, 130 ) cat_title(c_dirhit)
     .                          (1:cat_len(c_dirhit)) // per_sec,
     .                          set(dirhit) / dif_time,
     .                          max(dirhit),
     .                          min(dirhit),
     .                          std( bar(dirhit),
     .                               set(dirhit),count )

         write( out_unit, 130 ) cat_title(c_dirmiss)
     .                          (1:cat_len(c_dirmiss)) // per_sec,
     .                          set(dirmiss) / dif_time,
     .                          max(dirmiss),
     .                          min(dirmiss),
     .                          std( bar(dirmiss),
     .                               set(dirmiss),count )

         write( out_unit, 130 ) cat_title(c_exthit)
     .                          (1:cat_len(c_exthit)) // per_sec,
     .                          set(exthit) / dif_time,
     .                          max(exthit),
     .                          min(exthit),
     .                          std( bar(exthit),
     .                               set(exthit),count )

         write( out_unit, 130 ) cat_title(c_extmiss)
     .                          (1:cat_len(c_extmiss)) // per_sec,
     .                          set(extmiss) / dif_time,
     .                          max(extmiss),
     .                          min(extmiss),
     .                          std( bar(extmiss),
     .                               set(extmiss),count )

         write( out_unit, 130 ) cat_title(c_fidhit)
     .                          (1:cat_len(c_fidhit)) // per_sec,
     .                          set(fidhit) / dif_time,
     .                          max(fidhit),
     .                          min(fidhit),
     .                          std( bar(fidhit),
     .                               set(fidhit),count )

         write( out_unit, 130 ) cat_title(c_fidmiss)
     .                          (1:cat_len(c_fidmiss)) // per_sec,
     .                          set(fidmiss) / dif_time,
     .                          max(fidmiss),
     .                          min(fidmiss),
     .                          std( bar(fidmiss),
     .                               set(fidmiss),count )

         write( out_unit, 130 ) cat_title(c_quohit)
     .                          (1:cat_len(c_quohit)) // per_sec,
     .                          set(quohit) / dif_time,
     .                          max(quohit),
     .                          min(quohit),
     .                          std( bar(quohit),
     .                               set(quohit),count )

         write( out_unit, 130 ) cat_title(c_quomiss)
     .                          (1:cat_len(c_quomiss)) // per_sec,
     .                          set(quomiss) / dif_time,
     .                          max(quomiss),
     .                          min(quomiss),
     .                          std( bar(quomiss),
     .                               set(quomiss),count )

         write( out_unit, 150 )

         write( out_unit, 130 ) cat_title(c_open),
     .                          set(open) / count ,
     .                          max(open),
     .                          min(open),
     .                          std( bar(open),
     .                               set(open),count )

         return

!++
! display memory statistics
!--

400       continue

          write( out_unit, 150 )

          write( out_unit, 130 ) cat_title(c_freecnt),
     .                           set(freecnt) / count,
     .                           max(freecnt),
     .                           min(freecnt),
     .                           std( bar(freecnt),
     .                                set(freecnt),count )

          write( out_unit, 130 ) cat_title(c_mfycnt),
     .                           set(mfycnt) / count,
     .                           max(mfycnt),
     .                           min(mfycnt),
     .                           std( bar(mfycnt),
     .                                set(mfycnt),count )


          return

!++
! command was display page fault stats
!--

500       continue

          write( out_unit, 120 )

          write( out_unit, 130 ) cat_title(c_dzroflts)
     .                           (1:cat_len(c_dzroflts)) // per_sec,
     .                           set(dzroflts) / dif_time,
     .                           max(dzroflts),
     .                           min(dzroflts),
     .                           std( bar(dzroflts),
     .                                set(dzroflts),count )

          write( out_unit, 130 ) cat_title(c_faults)
     .                           (1:cat_len(c_faults)) // per_sec,
     .                           set(faults) / dif_time,
     .                           max(faults),
     .                           min(faults),
     .                           std( bar(faults),
     .                                set(faults),count )

          write( out_unit, 130 ) cat_title(c_rdflts)
     .                           (1:cat_len(c_rdflts)) // per_sec,
     .                           set(rdflts) / dif_time,
     .                           max(rdflts),
     .                           min(rdflts),
     .                           std( bar(rdflts),
     .                                set(rdflts),count )

          write( out_unit, 130 ) cat_title(c_preadio)
     .                           (1:cat_len(c_preadio)) // per_sec,
     .                           set(preadio) / dif_time,
     .                           max(preadio),
     .                           min(preadio),
     .                           std( bar(preadio),
     .                                set(preadio),count )

          write( out_unit, 130 ) cat_title(c_pwritio)
     .                           (1:cat_len(c_pwritio)) // per_sec,
     .                           set(pwritio) / dif_time,
     .                           max(pwritio),
     .                           min(pwritio),
     .                           std( bar(pwritio),
     .                                set(pwritio),count )

          write( out_unit, 130 ) cat_title(c_preads)
     .                           (1:cat_len(c_preads)) // per_sec,
     .                           set(preads) / dif_time,
     .                           max(preads),
     .                           min(preads),
     .                           std( bar(preads),
     .                                set(preads),count )

          write( out_unit, 130 ) cat_title(c_pwrites)
     .                           (1:cat_len(c_pwrites)) // per_sec,
     .                           set(pwrites) / dif_time,
     .                           max(pwrites),
     .                           min(pwrites),
     .                           std( bar(pwrites),
     .                                set(pwrites),count )


          return
!++
! command was display scheduler states
!--

600       continue

          write( out_unit, 150 )

          write( out_unit, 130 ) cat_title(c_cef),
     .                           set(cef) / count,
     .                           max(cef),
     .                           min(cef),
     .                           std( bar(cef),
     .                                set(cef),count )

          write( out_unit, 130 ) cat_title(c_colpg),
     .                           set(colpg) / count,
     .                           max(colpg),
     .                           min(colpg),
     .                           std( bar(colpg),
     .                                set(colpg),count )

          write( out_unit, 130 ) cat_title(c_com),
     .                           set(com) / count,
     .                           max(com),
     .                           min(com),
     .                           std( bar(com),
     .                                set(com),count )

          write( out_unit, 130 ) cat_title(c_como),
     .                           set(como) / count,
     .                           max(como),
     .                           min(como),
     .                           std( bar(como),
     .                                set(como),count )

          write( out_unit, 130 ) cat_title(c_cur),
     .                           set(cur) / count,
     .                           max(cur),
     .                           min(cur),
     .                           std( bar(cur),
     .                                set(cur),count )

          write( out_unit, 130 ) cat_title(c_fpg),
     .                           set(fpg) / count,
     .                           max(fpg),
     .                           min(fpg),
     .                           std( bar(fpg),
     .                                set(fpg),count )

          write( out_unit, 130 ) cat_title(c_hib),
     .                           set(hib) / count,
     .                           max(hib),
     .                           min(hib),
     .                           std( bar(hib),
     .                                set(hib),count )

          write( out_unit, 130 ) cat_title(c_hibo),
     .                           set(hibo) / count,
     .                           max(hibo),
     .                           min(hibo),
     .                           std( bar(hibo),
     .                                set(hibo),count )

          write( out_unit, 130 ) cat_title(c_lef),
     .                           set(lef) / count,
     .                           max(lef),
     .                           min(lef),
     .                           std( bar(lef),
     .                                set(lef),count )

          write( out_unit, 130 ) cat_title(c_lefo),
     .                           set(lefo) / count,
     .                           max(lefo),
     .                           min(lefo),
     .                           std( bar(lefo),
     .                                set(lefo),count )

          write( out_unit, 130) cat_title(c_mwait),
     .                          set(mwait) / count,
     .                          max(mwait),
     .                          min(mwait),
     .                           std( bar(mwait),
     .                                set(mwait),count )

          write( out_unit, 130 ) cat_title(c_pfw),
     .                           set(pfw) / count,
     .                           max(pfw),
     .                           min(pfw),
     .                           std( bar(pfw),
     .                                set(pfw),count )

          write( out_unit, 130 ) cat_title(c_susp),
     .                           set(susp) / count,
     .                           max(susp),
     .                           min(susp),
     .                           std( bar(susp),
     .                                set(susp),count )

          write( out_unit, 130 ) cat_title(c_suspo),
     .                           set(suspo) / count,
     .                           max(suspo),
     .                           min(suspo),
     .                           std( bar(suspo),
     .                                set(suspo),count )


          return

!++
! command was display swap stats
!--

700       continue

          write( out_unit, 150 )

          write( out_unit, 130 ) cat_title(c_ispagcnt),
     .                           set(ispagcnt) / count,
     .                           max(ispagcnt),
     .                           min(ispagcnt),
     .                           std( bar(ispagcnt),
     .                                set(ispagcnt),count )

          write( out_unit, 130 ) cat_title(c_proccnt),
     .                           set(proccnt) / count,
     .                           max(proccnt),
     .                           min(proccnt),
     .                           std( bar(proccnt),
     .                                set(proccnt),count )

          write( out_unit, 120 )

          write( out_unit, 130 ) cat_title(c_iswpcnt)
     .                           (1:cat_len(c_iswpcnt))
     .                           // per_sec,
     .                           set(iswpcnt) / dif_time,
     .                           max(iswpcnt),
     .                           min(iswpcnt),
     .                           std( bar(iswpcnt),
     .                                set(iswpcnt),count )

          write( out_unit, 130 ) cat_title(c_oswpcnt)
     .                           (1:cat_len(c_oswpcnt))
     .                           // per_sec,
     .                           set(oswpcnt) / dif_time,
     .                           max(oswpcnt),
     .                           min(oswpcnt),
     .                           std( bar(oswpcnt),
     .                                set(oswpcnt),count )

          write( out_unit, 130 ) cat_title(c_hiswpcnt)
     .                           (1:cat_len(c_hiswpcnt))
     .                           // per_sec,
     .                           set(hiswpcnt) / dif_time,
     .                           max(hiswpcnt),
     .                           min(hiswpcnt),
     .                           std( bar(hiswpcnt),
     .                                set(hiswpcnt),count )

          write( out_unit, 130 ) cat_title(c_hoswpcnt)
     .                           (1:cat_len(c_hoswpcnt))
     .                            // per_sec,
     .                           set(hoswpcnt) / dif_time,
     .                           max(hoswpcnt),
     .                           min(hoswpcnt),
     .                           std( bar(hoswpcnt),
     .                                set(hoswpcnt),count )

          return

!++
! command was display the uptime, not implemented yet
!--

800       continue

          return

!++
! DECnet ( DEC not or DEC knot ) stats
!--

900       continue

          write( out_unit, 120 )

          write( out_unit, 130 ) cat_title(c_arrlocpk)
     .                           (1:cat_len(c_arrlocpk)),
     .                           set(arrlocpk) / dif_time,
     .                           max(arrlocpk),
     .                           min(arrlocpk),
     .                           std( bar(arrlocpk),
     .                                set(arrlocpk),count )


          write( out_unit, 130 ) cat_title(c_deplocpk)
     .                           (1:cat_len(c_deplocpk)),
     .                           set(deplocpk) / dif_time,
     .                           max(deplocpk),
     .                           min(deplocpk),
     .                           std( bar(deplocpk),
     .                                set(deplocpk),count )

          write( out_unit, 130 ) cat_title(c_arrtrapk)
     .                           (1:cat_len(c_arrtrapk)),
     .                           set(arrtrapk) / dif_time,
     .                           max(arrtrapk),
     .                           min(arrtrapk),
     .                           std( bar(arrtrapk),
     .                                set(arrtrapk),count )


          write( out_unit, 130 ) cat_title(c_rcvbuffl)
     .                           (1:cat_len(c_rcvbuffl)),
     .                           set(rcvbuffl) / dif_time,
     .                           max(rcvbuffl),
     .                           min(rcvbuffl),
     .                           std( bar(rcvbuffl),
     .                                set(rcvbuffl),count )

          write( out_unit, 130 ) cat_title(c_trcnglos)
     .                           (1:cat_len(c_trcnglos)),
     .                           set(trcnglos) / dif_time,
     .                           max(trcnglos),
     .                           min(trcnglos),
     .                           std( bar(trcnglos),
     .                                set(trcnglos),count )


          return

!++
! command was display the ENQ/DEQ stats
!--

1000      continue

          write( out_unit, 150 )

          write( out_unit, 130 ) cat_title(c_enqnew)
     .                           (1:cat_len(c_enqnew)) // per_sec,
     .                           set(enqnew) / dif_time,
     .                           max(enqnew),
     .                           min(enqnew),
     .                           std( bar(enqnew),
     .                                set(enqnew),count )

          write( out_unit, 130 ) cat_title(c_enqwait)
     .                           (1:cat_len(c_enqwait)) // per_sec,
     .                           set(enqwait) / dif_time,
     .                           max(enqwait),
     .                           min(enqwait),
     .                           std( bar(enqwait),
     .                                set(enqwait),count )

          write( out_unit, 130 ) cat_title(c_deq)
     .                           (1:cat_len(c_deq)) // per_sec,
     .                           set(deq) / dif_time,
     .                           max(deq),
     .                           min(deq),
     .                           std( bar(deq),
     .                                set(deq),count )

          write( out_unit, 130 ) cat_title(c_dlckfnd)
     .                           (1:cat_len(c_dlckfnd)) // per_sec,
     .                           set(dlckfnd) / dif_time,
     .                           max(dlckfnd),
     .                           min(dlckfnd),
     .                           std( bar(dlckfnd),
     .                                set(dlckfnd),count )

          write( out_unit, 130 ) cat_title(c_dlcksrch)
     .                           (1:cat_len(c_dlcksrch)) // per_sec,
     .                           set(dlcksrch) / dif_time,
     .                           max(dlcksrch),
     .                           min(dlcksrch),
     .                           std( bar(dlcksrch),
     .                                set(dlcksrch),count )

          write( out_unit, 130 ) cat_title(c_enqcvt)
     .                           (1:cat_len(c_enqcvt)) // per_sec,
     .                           set(enqcvt) / dif_time,
     .                           max(enqcvt),
     .                           min(enqcvt),
     .                           std( bar(enqcvt),
     .                                set(enqcvt),count )

          write( out_unit, 130 ) cat_title(c_enqnotqd)
     .                           (1:cat_len(c_enqnotqd)) // per_sec,
     .                           set(enqnotqd) / dif_time,
     .                           max(enqnotqd),
     .                           min(enqnotqd),
     .                           std( bar(enqnotqd),
     .                                set(enqnotqd),count )

          return

!++
! command was display the response
!--

1100      continue

          write( out_unit, 150 )

          write( out_unit, 170 ) cat_title(c_response),
     .                            set(res) / count,
     .                            response_max,
     .                            response_min,
     .                            std( bar(res),
     .                                 set(res),count )

          return

!++
! command was display the non-paged pool stats
!--

1200      continue

          write( out_unit, 150 )

          write( out_unit, 130 ) cat_title(c_irpcnt),
     .                           set(irpcnt) / count,
     .                           max(irpcnt),
     .                           min(irpcnt),
     .                           std( bar(irpcnt),
     .                                set(irpcnt),count )

          write( out_unit, 130 ) cat_title(c_lrpcnt),
     .                           set(lrpcnt) / count,
     .                           max(lrpcnt),
     .                           min(lrpcnt),
     .                           std( bar(lrpcnt),
     .                                set(lrpcnt),count )

          write( out_unit, 130 ) cat_title(c_srpcnt),
     .                           set(srpcnt) / count,
     .                           max(srpcnt),
     .                           min(srpcnt),
     .                           std( bar(srpcnt),
     .                                set(srpcnt),count )

          write( out_unit, 130 ) cat_title(c_holecnt),
     .                           set(holecnt) / count,
     .                           max(holecnt),
     .                           min(holecnt),
     .                           std( bar(holecnt),
     .                                set(holecnt),count )

          write( out_unit, 130 ) cat_title(c_bighole),
     .                           set(bighole) / count,
     .                           max(bighole),
     .                           min(bighole),
     .                           std( bar(bighole),
     .                                set(bighole),count )

          write( out_unit, 130 ) cat_title(c_smallhole),
     .                           set(smallhole) / count,
     .                           max(smallhole),
     .                           min(smallhole),
     .                           std( bar(smallhole),
     .                                set(smallhole),count )

          write( out_unit, 130 ) cat_title(c_smallcnt),
     .                           set(smallcnt) / count,
     .                           max(smallcnt),
     .                           min(smallcnt),
     .                           std( bar(smallcnt),
     .                                set(smallcnt),count )

          write( out_unit, 130 ) cat_title(c_holesum),
     .                           set(holesum) / count,
     .                           max(holesum),
     .                           min(holesum),
     .                           std( bar(holesum),
     .                                set(holesum),count )


          return


          end
