!
!			  COPYRIGHT (c) 1982 BY
!	      MOSTEK CORPORATION, CARROLLTON TX
!
! This software is furnished under a license and may be used and  copied
! only  in  accordance  with  the  terms  of  such  license and with the
! inclusion of the above copyright notice.  This software or  any  other
! copies  thereof may not be provided or otherwise made available to any
! other person.  No title to and ownership of  the  software  is  hereby
! transferred.
!
! The information in this software is subject to change  without  notice
! and  should  not  be  construed  as  a commitment by MOSTEK
! CORPORATION.
!
! MOSTEK assumes no responsibility for the use or  reliability  of  its
! software on equipment which is not supplied by MOSTEK.
!

!++
! FACILITY:
!   MOSTEK CORPORATION
!
! ABSTRACT:
!   DEC does not supply a very good program for getting the performance of the
!   system. Thus MOUSE was born as part of a two part system stats program.
!   MOUSE gathers the system information and stores it away in a file for
!   use by the program STATS. 
!
! ENVIRONMENT:
!   VAX11 VMS
!
! AUTHOR: Gary Smith, CREATION DATE: 6/24/82
!
! MODIFIED BY:
!
!--
!
! TABLE OF CONTENTS:
!
!
! INCLUDE FILES:
!   mousesubs.for
!   defrec.for
!
! MACROS:
!   none
!
! EQUATED SYMBOLS:
!   see mousesubs.for, defrec.for
!
! EXTERNAL REFERENCES:
!   SYS$TIMR, SYS$GETTIM, SYS$WAITFR, SYS$CANTIM, SYS$GETMSG
!
!++
! FUNCTIONAL DESCRIPTION:
!   MOUSE as part of the system stats package wakes up at regualr intervals
!   and scurries about the system gathering all the types of information
!   that the system keeps on itself. Once this information has been gathered
!   it is put into a keyed file for use by the STATS program. The file of
!   information is a binary file keyed by the system time of day.
!
!
! FORMAL PARAMETERS:
!   none
!
!
! IMPLICIT INPUTS:
!   none
!
!
! IMPLICIT OUTPUTS:
!   none
!
!
! ROUTINE VALUE and
! COMPLETION CODES:
!   none
!
!
! SIDE EFFECTS:
!   none to date
!
!
!--
        PROGRAM MOUSE ! MOnitor USage Evaluator


        include  'mousesubs.for'
        include  'defrec.for'

!++
! local subs
!--

        parameter init       = 1
        parameter end        = 2

        character * 16 time_key
        character * 80 msg

        integer status, rms_status, mouse_unit, err_unit
        integer SYS$GETTIM, SYS$GETMSG, SYS$CANTIM, SYS$SETIMR
        integer SYS$WAITFR, SYS$BINTIM
        

        logical first_time

        real * 8 timdel, current_time

        external SYS$GETTIM, SYS$GETMSG, SYS$CANTIM, SYS$SETIMR
        external SYS$WAITFR, SS$_NORMAL

        data         first_time / .true. /
        data         mouse_unit / 42     /
        data         err_unit   / 66     /

!++
! Open up the stats file. Note that it is keyed by system time in the
! first sixteen bytes. Note also that is opened SHARED so that STATS can
! read the file as MOUSE is writing new info into it. The gyrations
! with opening, closing etc. is to ensure that the RATSNEST file has
! the proper protection to it.
!--

          status = SYS$BINTIM( ascii_delta_time, timdel )

          open( unit=mouse_unit, status='UNKNOWN',
     .          organization='INDEXED', access='KEYED',
     .          recordtype='FIXED', form='UNFORMATTED',
     .          recl=(key_size+rec_size), key=(1:16:character),
     .          iostat=ios, err=999, shared,
     .          carriagecontrol='NONE',
     .          extendsize=3,
     .          file=standard_nest_file )

          close( unit=mouse_unit )
          status = MK$MOD_PROT( standard_nest_file, '10ff'x, '10ff'x )

          open( unit=mouse_unit, status='OLD',
     .          organization='INDEXED', access='KEYED',
     .          recordtype='FIXED', form='UNFORMATTED',
     .          recl=(key_size+rec_size), key=(1:16:character),
     .          iostat=ios, err=999, shared,
     .          carriagecontrol='NONE',
     .          extendsize=3,
     .          file=standard_nest_file )

40      continue
        do while ( .true. )  !! do this forever 

             status = SYS$SETIMR( %VAL(1),timdel,,) !!set delay unit

!+
! Cycle thru all the various stat gathering subroutines
!_
             status = SYS$GETTIM( current_time )         !! current time

             call STATRESP( init )  !! Initialize for response
             call STATCPU           !! Cpu
             call STATFCP           !! FCP usage
             call STATMISC          !! Miscellaneous
             call STATMEM           !! Memory
             call STATPGFLT         !! Page faults
             call STATSCH           !! Scheduler states
             call STATSWAP          !! Swapping
             call STATUP            !! Uptime
             call STATNET           !! DECnet 
             call STATLOCK          !! ENQ/DEQ stats
             call STATPOOL          !! Non-paged pool

             encode( 16, 100, time_key ) current_time 
100          format( z16 )


!++
! Throw away the first set of data since we don't know if MOUSE was
! started at the time the system was brought up. Unless this is done,
! erroneously high counts will be put into the performance file.
!--

             if ( .not. first_time ) then
                write( unit=mouse_unit, iostat=ios, err=999 ) time_key,
     .                                                        stats_rec
             end if 

             call STATRESP( end )

             status = SYS$WAITFR( %VAL(wait_ef) )
             status = SYS$CANTIM( , )

             first_time = .false. 

        end do

!++
! In case of failure due to writing or to initially opening the stats file,
! the "err=" conditional bounces you to here and all execution ceases.
! With any luck, the "ios" can be transmogrified to a meaningful
! system error msg and put away for later use.
!--

999     continue
        close( unit=mouse_unit )
        call ERRSNS( ios, rms_status , , mouse_unit, )
        status = SYS$GETMSG( %VAL(rms_status), , msg, , )
        open( unit=err_unit, file='MOUSE.ERR', status='NEW' )
        write( unit=err_unit, fmt=9991 ) msg(1:MK$LENGTH(msg)), 
     .                                   rms_status
9991    format( ' ', 'MOUSE died due to ', A, '. RMS status is', z8 )
        close( unit=err_unit )
        stop ' '
        end
