!
!			  COPYRIGHT (c) 1982 BY
!	      MOSTEK CORPORATION, CARROLLTON TX
!
! This software is furnished under a license and may be used and  copied
! only  in  accordance  with  the  terms  of  such  license and with the
! inclusion of the above copyright notice.  This software or  any  other
! copies  thereof may not be provided or otherwise made available to any
! other person.  No title to and ownership of  the  software  is  hereby
! transferred.
!
! The information in this software is subject to change  without  notice
! and  should  not  be  construed  as  a commitment by MOSTEK
! CORPORATION.
!
! MOSTEK assumes no responsibility for the use or  reliability  of  its
! software on equipment which is not supplied by MOSTEK.
!

!++
! FACILITY:
!   MOSTEK CORPORATION
!
! ABSTRACT: PDL_LINE creates a PDL file for later use by a plotter package
!   
!
! ENVIRONMENT: VAX/VMS unpriv
!   
!
! AUTHOR: Gary Smith, CREATION DATE: 9/20/1982
!
! MODIFIED BY:
!
!--
!
! TABLE OF CONTENTS:
!
!
! INCLUDE FILES: none
!
!
! MACROS: none
!
!
! EQUATED SYMBOLS: NPFIT
!
!
! EXTERNAL REFERENCES: none
!

!++
! FUNCTIONAL DESCRIPTION: PDL_LINE operates in the same manner as PLOT_LINE
!                         does. Only PDL_LINE creates a PDL file instead of
!                         writing  plot to the screen. 
!
!
!
! FORMAL PARAMETERS: x              -> array of x values
!                    y              -> array of y values
!                    n              -> number of values
!                    xmin           -> smallest x value
!                    xmax           -> biggest x value
!                    ymin           -> smallest y value
!                    ymax           -> biggest y value
!                    xtitle         -> title for the x axis
!                    ytitle         -> title for the y axis
!                    dif_time       -> the amount of time the plot 
!                                      covers in secs
!                    frame          -> which plot number this is
!                    since_mid_nite -> minutes since midnite this plot
!                                      begins at
!
!
!
! IMPLICIT INPUTS: none
!
!
!
! IMPLICIT OUTPUTS: a PDL file of the information to be plotted
!
!
!
! ROUTINE VALUE and
! COMPLETION CODES: none
!
!
!
! SIDE EFFECTS: none to date
!
!
!
!--
      SUBROUTINE PDL_LINE (X, Y, N, XMIN, XMAX, YMIN, YMAX, XTITLE,
     .                      YTITLE, DIF_TIME, SINCE_MID_NITE )
      include 'statsubs.for'

      character*80 xtitle, ytitle, factor_string
      dimension x(n), y(n)
      parameter (npfit = 100)
      dimension xx(npfit+2), yy(npfit+2)
      dimension sa(2000), sb(2000), sc(2000)
      character*40  form, time_format
      character*9   date_buf
      character * 12 user_name
      character * 7  scale_time
      character * 63 nest_file
      character * 9  node_name
      integer dif_time, pos_quote, pos_colon, status
      data rnd /1.0005/, small/1.e-30/
      logical pdl, quarter
      common /scal/ scale, xmn, xdl, ymn, ydl, lu, pdl,
     +     quarter, ixorg, iyorg

      common / nest_common / nest_file
c
      comlog(z) = alog10(amax1(small,abs(rnd*z)))
c
      iscale = 10000
      scale = iscale
      ixorg = 0
      iyorg = 0
      lu = 12
      pdl = .true.
      quarter = .false.
      xdl = 0.1*(xmax-xmin)
      xmx = xmax
      xmn = xmin
      dx = (xmax-xmin)/(npfit-1)
      x0 = xmin - dx
      do 20 i = 1,npfit
        x0 = x0 + dx
        xx(i) = x0
20      continue
      xx(npfit+1) = xmin
      xx(npfit+2) = xdl
      xmlg = amax1( comlog(rnd*xmn), comlog(rnd*xmax))
      iexpon = 3*(ifix(xmlg)/3)
      if(xmlg .lt. 0.0)  iexpon = 3*(ifix(xmlg-3.)/3)
      idpr = xmlg - iexpon
      form = '(F5.' // char(2-idpr+48) // ')'
      factor = 10.0**iexpon
      scale_time = 'MINUTES'
      time_factor = 1.0
      if ( dif_time .gt. seconds_in_hour ) then
          scale_time = 'HOURS'
          time_factor = seconds_in_minute
          time_format = '$HOUR'
      end if 
      if ( dif_time .gt. 2.0 * seconds_in_day ) then
          scale_time = 'DAYS'
          time_format = form
      end if 
      if ( iexpon .gt. 1 ) then
         encode( iexpon+4, 666, factor_string ) factor
666      format( f<iexpon+4>.1 )
         xtitle = xtitle(1:MK$LENGTH(xtitle))
     .        // ' (' //  scale_time(1:MK$LENGTH(scale_time)) // ')* '
     .        // factor_string(1:iexpon+4) // ')'
      else 
         xtitle = xtitle(1:MK$LENGTH(xtitle)) // ' ('
     .            // scale_time(1:MK$LENGTH(scale_time)) // ')'
      endif
      write( lu, 150 ) 'H'
150   format( a, a )
      call date( date_buf )
      status = MK$FILENAME( nest_file )
      pos_quote = index( nest_file, '"' )
      if ( pos_quote .eq. 0 ) then
         pos_colon = index( nest_file, '::' )
         if ( pos_colon .eq. 0 ) then
            status = SYS$TRNLOG( 'SYS$NODE', , node_name, , , )
         else
            node_name = nest_file(1:pos_colon-1) // '::'
         endif
      else
         node_name = nest_file(1:pos_quote-1) // '::'
      endif
      write( lu, 150 ) ' ', node_name(1:MK$LENGTH(node_name)) //
     .                     ' '  // date_buf
      ismin = -iscale / 8
      ismax = 21 * iscale / 20
      write( lu, 160 ) 'W', ismin, ismin, ismax, ismax
160   format( a, 12a4 )
      write( lu, 160 ) 'C', 1
      idelt = iscale / ifix( ( xmx - xmn ) / xdl )
      write( lu, 160 ) 'A', 0, 0, iscale, 120, idelt, 1, 31
      if ( dif_time .lt. 2.0 * seconds_in_day ) then
      write( lu, 160 ) ' ', since_mid_nite , time_factor * xdl / factor
      else
      write( lu, 160 ) ' ', xmn / factor , xdl / factor
      end if
      write( lu, 150 ) ' ', time_format
      write( lu, 160 ) 'S', ( iscale / 2 - 4500 ) , -800, 160, 0
      write( lu, 150 ) ' ', xtitle(1:MK$LENGTH(xtitle))
      if ( ymax .eq. ymin ) then
         ymax = 2.0 * ymax
         ymin = 0.0
      end if
      ydl = 0.1*(ymax-ymin)
      ymx = ymax
      ymn = ymin
      jj = 1
      do 50 j = 1,npfit
50      yy(j) = seval(jj, n, xx(j), x, y, sa, sb, sc)
      yy(npfit+1) = ymin
      yy(npfit+2) = ydl
      ymlg = amax1( comlog(rnd*ymn), comlog(rnd*ymax))
      iexpon = 3*(ifix(ymlg)/3)
      if(ymlg .lt. 0.0)  iexpon = 3*(ifix(ymlg-3.)/3)
      idpr = ymlg - iexpon
      form = '(F5.' // char(2-idpr+48) // ')'
      factor = 10.0**iexpon
      if ( iexpon .gt. 1 ) then
         encode( iexpon+4, 666, factor_string ) factor
         ytitle = ytitle(1:MK$LENGTH(ytitle))
     .        // ' * ' // factor_string(1:MK$LENGTH(factor_string))
      endif
      idelt = iscale / ifix(( ymx - ymn ) / ydl)
      write ( lu, 160 ) 'A', 0, 0, iscale, 120, idelt, 1, 24
      write ( lu, 160 ) ' ', ymn / factor, ydl / factor
      write ( lu, 150 ) ' ', form
      write ( lu, 160 ) 'S', ismin+500, ( iscale / 2 - 2400 ), 200, 900
      write ( lu, 150 ) ' ', ytitle(1:MK$LENGTH(ytitle))
      call draw_line( xx, yy, npfit )
c
      return
      end
