!
!			  COPYRIGHT (c) 1982 BY
!	      MOSTEK CORPORATION, CARROLLTON TX
!
! This software is furnished under a license and may be used and  copied
! only  in  accordance  with  the  terms  of  such  license and with the
! inclusion of the above copyright notice.  This software or  any  other
! copies  thereof may not be provided or otherwise made available to any
! other person.  No title to and ownership of  the  software  is  hereby
! transferred.
!
! The information in this software is subject to change  without  notice
! and  should  not  be  construed  as  a commitment by MOSTEK
! CORPORATION.
!
! MOSTEK assumes no responsibility for the use or  reliability  of  its
! software on equipment which is not supplied by MOSTEK.
!

!++
! FACILITY:
!   MOSTEK CORPORATION
!
! ABSTRACT: Plot the components collected by MOUSE
!   
!
! ENVIRONMENT: VAX VMS unpriv
!   
!
! AUTHOR: Gary Smith, CREATION DATE: 8/23/1982
!
! MODIFIED BY:
!
!--
!
! TABLE OF CONTENTS:
!
!
! INCLUDE FILES: statsubs.for
!                defrec.for
!                mousesubs.for
!                catdef.for
!
!
! MACROS: none
!
!
! EQUATED SYMBOLS: see statsubs.for
!                      mousesubs.for
!                      defrec.for
!
!
! EXTERNAL REFERENCES: read_nest
!

!++
! FUNCTIONAL DESCRIPTION: PLOT plots the information gleaned from
!                         the RATSNEST file and diplays it to the user
!                         in a hopefully comprehensible form. The RATSNEST
!                         file is keyed by time. PLOT uses start_time as
!                         the beginning with keyed read. It then reads 
!                         sequentially until the key returned is greater
!                         than the end_time.
!
!
!
! FORMAL PARAMETERS: start_time -> the beginning key to start reading
!                    end_time   -> the end point of reading
!                    plot_type  -> special for response
!                    category   -> the base class of option
!                    type       -> the particular thing to plot
!                    frame      -> frame number 1 to 4 inclusive
!                    multi_plot -> multiple things to plot flag
!                    interval   -> the sampling interval of MOUSE
!
!
!
! IMPLICIT INPUTS: info read from the RATSNEST file
!
!
!
! IMPLICIT OUTPUTS: a coherent plot of the gathered information
!
!
!
! ROUTINE VALUE and
! COMPLETION CODES: none
!
!
!
! SIDE EFFECTS: none to date
!
!
!
!--
          SUBROUTINE PLOT( start_time, end_time, plot_type, 
     .                     category, type, frame, multi_plot, interval )

          include 'mousesubs.for'
          include 'statsubs.for'
          include 'defrec.for'
          include 'catdef.for'

          common / io_com / in_unit, out_unit 
          common / category / cat_state, cat_title, cat_len

          character * 16  start_time, end_time, nest_key, save_time
          character * 16  sav_start_time, sav_end_time, ascii_mid_nite
          character * 23  begin, finish, ascii_start_time
          character * 23  key_to_date
          character * 80  x_title, y_title
          character * 63  cat_title(rec_size)

          logical multi_plot

          integer category, count, dif_time, deltat, plot_type
          integer in_unit, out_unit, type, frame, status
          integer cat_state(max_cat,max_ent), cat_len(rec_size)
          integer SYS$BINTIM, STR$UPCASE

          real x_values(100000), y_values(100000)

          real     maximum, minimum
          real * 8 mid_nite

          nest_key = start_time

!++
! plot the particular category and type. if the ending time is less
! than the entry in the file, let him know he's starting too early.
!--

          continue

          maximum = 1.0*10E-10
          minimum = maxint
          count = 1
          call read_nest( keyed_read, nest_key, ios )
          sav_start_time = nest_key
          begin = key_to_date( nest_key )
          save_time = nest_key

!++
! Run thru the current nest file until we overstep our bounds.
! Note the special-casing of response
!--

          do while ( key_compare(end_time, nest_key) .eq. greater )
             if ( cat_state(category,type) .eq. c_response ) then
                  current_val = real(rec_response) / 100.0
             else
                  current_val = stats_rec(cat_state(category,type))
             end if

!++
! decide if there has been a lapse in the nest file for some reason such
! as a machine going down or otherwise.
!--

             dif_time = deltat( save_time, nest_key )
             if ( dif_time .gt. (interval + 0.1 * interval) ) then

                do i = count, count + ( dif_time / interval )
                   y_values(i) = 0.0
                   x_values(i) =  ( count * interval ) /
     .                                seconds_in_minute
                   count = count + 1
                end do

             minimum = 0.0
             end if

             y_values(count) = current_val
             if ( current_val .gt. maximum ) 
     .          maximum = current_val
             if ( current_val .lt. minimum )
     .          minimum = current_val
             x_values(count) =  ( count * interval ) /
     .                           seconds_in_minute
             count = count + 1
             save_time = nest_key
             call read_nest( seq_read, nest_key, ios )
             if ( ios .eq. eof ) goto 1001
          end do

!++
! All thru, see how far we went. This includes the time between
! the start and end time and the time since mid-night of the day
! in question.
!--

1001      continue
          finish = key_to_date( nest_key )
          dif_time = deltat( sav_start_time, nest_key )
          ascii_start_time = key_to_date( sav_start_time )
          status = SYS$BINTIM( ascii_start_time(1:12) //
     .                         '00:00:00.00' , mid_nite )
          encode( 16, 1002, ascii_mid_nite ) mid_nite
1002      format( Z16 )
          since_mid_nite = deltat( ascii_mid_nite, sav_start_time ) /
     .                     seconds_in_minute
          since_mid_nite = jint( since_mid_nite )

!++
! depending on the delta time, scale the time into hours or days
!--

          if ( ( dif_time .ge. seconds_in_hour) .and.
     .         ( dif_time .le. 2.0 * seconds_in_day ) ) then 

             do i = 1, count
                x_values(i) = x_values(i) / minutes_in_hour 
             end do

          else if ( ( dif_time .ge. seconds_in_hour ) .and.
     .         ( dif_time .gt. 2.0 * seconds_in_day ) ) then

             do i = 1, count
                x_values(i) = x_values(i) / minutes_in_day
             end do
             
          end if 

          x_title = begin // ' TO ' // finish
          y_title = cat_title(cat_state(category,type))
          status = STR$UPCASE( y_title, y_title )

!++
! now, plot it 
!--

          if ( plot_type .eq. 5 ) then
               call plot_line( x_values, y_values, count-1,
     .                         0.0, x_values(count-1), minimum,
     .                         maximum, x_title, y_title, dif_time,
     .                         since_mid_nite )
          else if ( .not. multi_plot ) then
               call pdl_line(  x_values, y_values, count-1,
     .                         0.0, x_values(count-1), minimum,
     .                         maximum, x_title, y_title, dif_time,
     .                         since_mid_nite )
          else 
               call multi_pdl( x_values, y_values, count-1,
     .                         0.0, x_values(count-1), minimum,
     .                         maximum, x_title, y_title, dif_time,
     .                         frame, since_mid_nite )
          endif

          return

         end
