!
!			  COPYRIGHT (c) 1982 BY
!	      MOSTEK CORPORATION, CARROLLTON TX
!
! This software is furnished under a license and may be used and  copied
! only  in  accordance  with  the  terms  of  such  license and with the
! inclusion of the above copyright notice.  This software or  any  other
! copies  thereof may not be provided or otherwise made available to any
! other person.  No title to and ownership of  the  software  is  hereby
! transferred.
!
! The information in this software is subject to change  without  notice
! and  should  not  be  construed  as  a commitment by MOSTEK
! CORPORATION.
!
! MOSTEK assumes no responsibility for the use or  reliability  of  its
! software on equipment which is not supplied by MOSTEK.
!

!++
! FACILITY:
!   MOSTEK CORPORATION
!
! ABSTRACT: PLOT_LINE interfaces with the neccessary routines to do a
!           on screen plot of the info passed to it
!   
!
! ENVIRONMENT: VAX/VMS unpriv
!   
!
! AUTHOR: Gary Smith, CREATION DATE: 9/20/1982
!
! MODIFIED BY:
!
!--
!
! TABLE OF CONTENTS:
!
!
! INCLUDE FILES: statsubs.for
!
!
! MACROS: none
!
!
! EQUATED SYMBOLS: NPFIT,
!                  see statsubs.for
!
!
! EXTERNAL REFERENCES: none
!

!++
! FUNCTIONAL DESCRIPTION: PLOT_LINE takes the arrays of the information
!                         obtained by PLOT and performs the computations
!                         to interface with the plotting package for VT100
!                         and all it's clones.
!
!
!
! FORMAL PARAMETERS: X              -> Array of X values
!                    Y              -> Array of Y values
!                    N              -> Number of points to plot
!                    XMIN           -> Smallest X value
!                    XMAX           -> Biggest X value
!                    YMIN           -> Smallest y value
!                    YMAX           -> Biggest Y value
!                    XTITLE         -> Title to appear on X axis
!                    YTITLE         -> Title to appear on Y axis
!                    DIF_TIME       -> The delta time between start and end
!                    SINCE_MID_NITE -> Minutes since midnite this plot 
!                                      began
!
!
!
! IMPLICIT INPUTS: none
!
!
!
! IMPLICIT OUTPUTS: a plot of Y versus X
!
!
!
! ROUTINE VALUE and
! COMPLETION CODES: none
!
!
!
! SIDE EFFECTS: none to date
!
!
!
!--
      SUBROUTINE PLOT_LINE (X, Y, N, XMIN, XMAX, YMIN, YMAX, XTITLE,
     .                      YTITLE, DIF_TIME, SINCE_MID_NITE )

      character*80 xtitle, ytitle, factor_string

      dimension x(n), y(n)

      parameter (npfit = 100)

      include 'statsubs.for'

      dimension xx(npfit+2), yy(npfit+2)
      dimension sa(2000), sb(2000), sc(2000)

      character*40  form, time_format
      character*7   scale_time

      integer dif_time, since_mid_nite

      data rnd /1.0005/, small/1.e-30/

      logical pdl, quarter

      common /scal/ scale, xmn, xdl, ymn, ydl, lu, pdl,
     +     quarter, ixorg, iyorg
c
      comlog(z) = alog10(amax1(small,abs(rnd*z)))
c
      ixorg = 0
      iyorg = 0
      lu = 12
      pdl = .false.
      quarter = .false.
      xdl = 0.1*(xmax-xmin)
      xmn = xmin
      dx = (xmax-xmin)/(npfit-1)
      x0 = xmin - dx
      do 20 i = 1,npfit
        x0 = x0 + dx
        xx(i) = x0
20      continue
      xx(npfit+1) = xmin
      xx(npfit+2) = xdl
      xmlg = amax1( comlog(rnd*xmn), comlog(rnd*xmax))
      iexpon = 3*(ifix(xmlg)/3)
      if(xmlg .lt. 0.0)  iexpon = 3*(ifix(xmlg-3.)/3)
      idpr = xmlg - iexpon
      form = '(F5.' // char(2-idpr+48) // ')'
      factor = 10.0**iexpon
      time_factor = 1.0
      time_format = '$HOUR'
      scale_time = 'HOURS'
      if ( dif_time .ge. seconds_in_hour ) then
         time_factor = seconds_in_minute
      end if
      if ( dif_time .gt. 2.0 * seconds_in_day ) then
         scale_time = 'DAYS'
         time_format = form
      end if
      if ( iexpon .gt. 1 ) then
         encode( iexpon+4, 666, factor_string ) factor
666      format( f<iexpon+4>.1 )
         xtitle = xtitle(1:MK$LENGTH(xtitle))
     .        // ' (' //  scale_time(1:MK$LENGTH(scale_time)) // ')* '
     .        // factor_string(1:iexpon+4) // ')'
      else 
         xtitle = xtitle(1:MK$LENGTH(xtitle)) // ' ('
     .            // scale_time(1:MK$LENGTH(scale_time)) // ')'
      endif
      call crt_init
      call crt_origin (1.,1.)

!++
! special case for time less than 2 days
!--

      if ( dif_time .le. 2.0 * seconds_in_day  ) then
         call crt_axis (0., 0., xtitle, 10.0, 'H',
     +            since_mid_nite, time_factor * xdl / factor,
     .            time_format )
      else 
         call crt_axis (0., 0., xtitle, 10.0, 'H',
     +            xx(npfit+1)/factor, xx(npfit+2)/factor, form)
c
      end if

      if ( ymax .eq. ymin ) then
         ymax = 2.0 * ymax
         ymin = 0.0
      end if
      ydl = 0.1*(ymax-ymin)
      ymn = ymin
      jj = 1
      do 50 j = 1,npfit
50      yy(j) = seval(jj, n, xx(j), x, y, sa, sb, sc)
      yy(npfit+1) = ymin
      yy(npfit+2) = ydl
      ymlg = amax1( comlog(rnd*ymn), comlog(rnd*ymax))
      iexpon = 3*(ifix(ymlg)/3)
      if(ymlg .lt. 0.0)  iexpon = 3*(ifix(ymlg-3.)/3)
      idpr = ymlg - iexpon
      form = '(F5.' // char(2-idpr+48) // ')'
      factor = 10.0**iexpon
      if ( iexpon .gt. 1 ) then
         encode( iexpon+4, 666, factor_string ) factor
         ytitle = ytitle(1:MK$LENGTH(ytitle))
     .        // ' * ' // factor_string
      endif
      call crt_axis (0., 0., ytitle, 10., 'V',
     +            yy(npfit+1)/factor, yy(npfit+2)/factor, form)
      call draw_line (xx, yy, npfit)
      call crt_end_job (.true.)
c
      return
      end
