!
!			  COPYRIGHT (c) 1982 BY
!	      MOSTEK CORPORATION, CARROLLTON TX
!
! This software is furnished under a license and may be used and  copied
! only  in  accordance  with  the  terms  of  such  license and with the
! inclusion of the above copyright notice.  This software or  any  other
! copies  thereof may not be provided or otherwise made available to any
! other person.  No title to and ownership of  the  software  is  hereby
! transferred.
!
! The information in this software is subject to change  without  notice
! and  should  not  be  construed  as  a commitment by MOSTEK
! CORPORATION.
!
! MOSTEK assumes no responsibility for the use or  reliability  of  its
! software on equipment which is not supplied by MOSTEK.
!

!++
! FACILITY:
!   MOSTEK CORPORATION
!
! ABSTRACT: READ_NEST is the interface to the RATSNEST file created by
!           MOUSE. All the different pieces of the stats record are broken
!           up form the single stats record.
!   
!
! ENVIRONMENT: VAX VMS 11 unpriveleged
!   
!
! AUTHOR: Gary Smith, CREATION DATE: 8/4/1982
!
! MODIFIED BY:
!
!--
!
! TABLE OF CONTENTS:
!
!
! INCLUDE FILES: mousesubs.for, defrec.for
!
!
! MACROS: none
!
!
! EQUATED SYMBOLS: see mousesubs.for, defrec.for
!
!
! EXTERNAL REFERENCES: msg_perfilerr, msg_filenotset, SYS$GETMSG
!

!++
! FUNCTIONAL DESCRIPTION: READ_NEST performs all I/O on the RATSNEST file
!                         left by MOUSE. It has four functions, open the 
!                         nest file, do keyed reads, do sequential reads,
!                         and close the nest file. An ancillary function
!                         of READ_NEST is to decompose the stats record into
!                         the different fields of information.
!
!
!
! FORMAL PARAMETERS: read_code -> operation that READ_NEST is to perform
!                    key       -> the 16 byte key that is the system time
!                                 in CHAR form for when the 
!                                 glomming of information took
!                                 place
!                    ios       -> status of operation
!
!
!
! IMPLICIT INPUTS: the performance file created by MOUSE
!
!
!
! IMPLICIT OUTPUTS: The various decomposed parts of the stats record.
!                   In addition, the key value for each stats record
!                   is returned in key.
!
!
!
! ROUTINE VALUE and
! COMPLETION CODES: none
!
!
!
! SIDE EFFECTS: none to date
!
!
!
!--
          SUBROUTINE read_nest( read_code, key, ios )

          include 'mousesubs.for'

          include 'defrec.for'

          character * 16 key
          character * 64 nest_file
          character * 80 msg

          integer  read_code, rms_status, status, SYS$GETMSG
          integer  out_unit

          logical batch, file_set

          common / nest_common / nest_file
          common / io_com / in_unit, out_unit, batch

          data mouse_unit / 42 /
          data file_set / .false. /

          external msg_perfilerr, msg_filenotset, SYS$GETMSG
!++
! based on the value of the read code, perform the read operation
!--
          goto ( 100,   200,         300,       400 )  read_code

!                open   keyed read   seq read   close

!+
! Open up the stats file. Note that it is keyed by system time in the
! first sixteen bytes. Note also that is opened SHARED so that STATS can
! read the file as MOUSE is writing new info into it.
!-

100       continue

          open( unit=mouse_unit, status='OLD',
     .          organization='INDEXED', access='KEYED',
     .          recordtype='FIXED', form='UNFORMATTED',
     .          recl=(key_size+rec_size), key=(1:16:character),
     .          iostat=ios, err=999, shared, readonly,
     .          carriagecontrol='NONE',
     .          file=nest_file )
          file_set = .true.
          return

!++
! keyed read, note the key greater than or equal keyword on the
! read 
!--

200       continue

          if ( file_set ) then
             read( unit=mouse_unit, iostat=ios, err=999, keyge=key )
     .             key, stats_rec
             goto 310
          else
             call LIB$SIGNAL( msg_filenotset )
             return
          end if

!++
! sequential read, note the abscence of the keyge keyword
!--

300       continue

          if ( file_set ) then
             read( unit=mouse_unit, iostat=ios, end=666, err=999 ) key,
     .                                                  stats_rec
          else
             call LIB$SIGNAL( msg_filenotset )
             return
          end if

310       continue
          unlock( unit=mouse_unit, iostat=ios )

          return

!++
! close the RATSNEST file
!--

400       continue

          close( unit=mouse_unit )
          file_set = .false.
          return

         
!++
! end of file occurred
!--

666       continue
          return

!++
! severe error occurred, shut down operations
!--

999       continue

          call ERRSNS( ios, rms_status , , mouse_unit, )
          status = SYS$GETMSG( %VAL(rms_status), , msg, , )
          write( unit=out_unit, fmt=9991 ) msg(1:MK$LENGTH(msg)), 
     .                                   rms_status
9991      format( ' ', A, '. RMS status is', z8 )
          call LIB$SIGNAL( msg_perfilerr )
          file_set = .false.
          return
          end
