!
!			  COPYRIGHT (c) 1982 BY
!	      MOSTEK CORPORATION, CARROLLTON TX
!
! This software is furnished under a license and may be used and  copied
! only  in  accordance  with  the  terms  of  such  license and with the
! inclusion of the above copyright notice.  This software or  any  other
! copies  thereof may not be provided or otherwise made available to any
! other person.  No title to and ownership of  the  software  is  hereby
! transferred.
!
! The information in this software is subject to change  without  notice
! and  should  not  be  construed  as  a commitment by MOSTEK
! CORPORATION.
!
! MOSTEK assumes no responsibility for the use or  reliability  of  its
! software on equipment which is not supplied by MOSTEK.
!

!++
! FACILITY:
!   MOSTEK CORPORATION
!
! ABSTRACT: Replay the components collected by MOUSE
!   
!
! ENVIRONMENT: VAX VMS unpriv
!   
!
! AUTHOR: Gary Smith, CREATION DATE: 8/23/1982
!
! MODIFIED BY:
!
!--
!
! TABLE OF CONTENTS:
!
!
! INCLUDE FILES: statsubs.for
!                defrec.for
!                mousesubs.for
!
!
! MACROS: none
!
!
! EQUATED SYMBOLS: see statsubs.for
!                      mousesubs.for
!                      defrec.for
!
!
! EXTERNAL REFERENCES: read_nest
!

!++
! FUNCTIONAL DESCRIPTION: REPLAY replays the information gleaned from
!                         the RATSNEST file and diplays it to the user
!                         in a hopefully comprehensible form. The RATSNEST
!                         file is keyed by time. REPLAY uses start_time as
!                         the beginning with keyed read. It then reads 
!                         sequentially until the key returned is greater
!                         than the end_time.
!
!
!
! FORMAL PARAMETERS: start_time -> the beginning key to start reading
!                    end_time   -> the end point of reading
!                    category   -> the base class of option
!                    type       -> the particular thing to plot
!
!
! IMPLICIT INPUTS: info read from the RATSNEST file
!
!
!
! IMPLICIT OUTPUTS: a replay of the values gathered
!
!
!
! ROUTINE VALUE and
! COMPLETION CODES: none
!
!
!
! SIDE EFFECTS: none to date
!
!
!
!--
          SUBROUTINE REPLAY( start_time, end_time, category, type )

          include 'mousesubs.for'
          include 'statsubs.for'
          include 'defrec.for'
          include 'catdef.for'

          common / io_com / in_unit, out_unit 
          common / category / cat_state, cat_title, cat_len

          character * 16  start_time, end_time, nest_key
          character * 16  sav_start_time, sav_end_time, ascii_mid_nite
          character * 23  begin, finish, ascii_start_time
          character * 23  key_to_date
          character * 80  x_title, y_title
          character * 63  cat_title(rec_size)

          integer category, count
          integer in_unit, out_unit, type
          integer cat_state(max_cat,max_ent), cat_len(rec_size)

          real     maximum, minimum

          nest_key = start_time

!++
! replay the particular category and type
!--

          continue

          maximum = 1.0*10E-10
          minimum = maxint
          call read_nest( keyed_read, nest_key, ios )
          sav_start_time = nest_key
          begin = key_to_date( nest_key )


!++
! fiddle about so that the thing is centered
!--

          iindxx = cat_state(category, type)
          column = 39 - MK$LENGTH(cat_title(iindxx)) / 2
          write( out_unit, 100 ) cat_title(iindxx)
     .           (1:MK$LENGTH(cat_title(iindxx)))
100       format( /, ' ', t<column>, A )
          write( out_unit, 105 )
105       format( /, t11, 'Time', t33, 'Current', t52, 'Max', t67,
     .            'Min', / )
!++
! Run thru the current nest file until we overstep our bounds.
! Note the special-casing of response.
!--

          do while ( key_compare(end_time, nest_key) .eq. greater )
             if ( cat_state(category,type) .ne. c_response) then
                current_val = stats_rec(cat_state(category,type))
             else
                current_val = real(rec_response) / 100.0
             end if
             if ( current_val .gt. maximum ) 
     .          maximum = current_val

             if ( current_val .lt. minimum ) 
     .          minimum = current_val

!++
! write out the time and the current value, maximum and minimum
!--

             write( out_unit, 110 ) begin, current_val, maximum,
     .                              minimum
110          format( ' ', A t30, f10.2, t45, f10.2, t60, f10.2 )
             call read_nest( seq_read, nest_key, ios )
             begin = key_to_date( nest_key )
             if ( ios .eq. eof ) goto 1001
          end do

1001      continue

          return

         end
