!
!			  COPYRIGHT (c) 1982 BY
!	      MOSTEK CORPORATION, CARROLLTON TX
!
! This software is furnished under a license and may be used and  copied
! only  in  accordance  with  the  terms  of  such  license and with the
! inclusion of the above copyright notice.  This software or  any  other
! copies  thereof may not be provided or otherwise made available to any
! other person.  No title to and ownership of  the  software  is  hereby
! transferred.
!
! The information in this software is subject to change  without  notice
! and  should  not  be  construed  as  a commitment by MOSTEK
! CORPORATION.
!
! MOSTEK assumes no responsibility for the use or  reliability  of  its
! software on equipment which is not supplied by MOSTEK.
!

!++
! FACILITY:
!   MOSTEK CORPORATION
!
! ABSTRACT: SET_TIME sets the starting time for reading the stats 
!           information in the RATSNEST file.
!   
!
! ENVIRONMENT: VAX VMS unpriv
!   
!
! AUTHOR: Gary Smith, CREATION DATE: 8/23/1982
!
! MODIFIED BY:
!
!--
!
! TABLE OF CONTENTS:
!
!
! INCLUDE FILES: statsubs.for
!
!
! MACROS: none
!
!
! EQUATED SYMBOLS: str_len (local)
!                  see statsubs.for
!
!
! EXTERNAL REFERENCES: SYS$BINTIM
!

!++
! FUNCTIONAL DESCRIPTION: SET_TIME receives from its caller the pcb
!                         (parse control block) and an 16 byte variable
!                         that will contain the time in DEC system format 
!                         after conversion by SYS$BINTIM in CHAR form.
!
!
!
! FORMAL PARAMETERS: pcb      -> the parse control block
!                    time     -> the converted time
!                    ret_code -> return code
!
!
!
! IMPLICIT INPUTS: none
!
!
!
! IMPLICIT OUTPUTS: none
!
!
!
! ROUTINE VALUE and
! COMPLETION CODES: ret_code  -> 0 success
!                   ret_code  -> 1 failure
!
!
!
! SIDE EFFECTS: none to date
!
!
!
!--
          SUBROUTINE SET_TIME( pcb, time, ret_code )
 
          include 'statsubs.for'

          parameter str_len = 5  

          integer pcb(30)
          integer SYS$BINTIM, SYS$GETMSG
          integer status
          integer ret_code
          integer out_unit
          integer i_systime(2), i_mid_nite(2), i_one_day(2)
          integer i_yesterday(2), i_offset(2), temp

          character * 16  time
          character * 23  time_buffer
          character * 23  mid_nite / '-- 00:00:00.00' /
          character * 23  one_day  / '1 00:00:00.00' /
          character * 80  msg_buffer
          character * 132 compress

          real * 8 systime, bin_mid_nite, bin_one_day, bin_yesterday
          real * 8 offset

          logical batch

          equivalence ( i_mid_nite, bin_mid_nite )
          equivalence ( i_one_day, bin_one_day )
          equivalence ( i_yesterday, bin_yesterday )
          equivalence ( i_systime, systime )
          equivalence ( i_offset, offset )

          external SYS$BINTIM, SS$_NORMAL, msg_invalidtime
          external msg_setstart, LIB$ADDX, LIB$SUBX

          common / io_com / in_unit, out_unit, batch


!++
! set return code to nominal success and figure what to do
!--

          ret_code = success
          if ( pcb(4) .eq. 5 ) then
               goto ( 100, 200, 300, 400, 500 ) pcb(6)

!++
! time was "START".
! Do this by closing the nest file, opening it again, and doing a sequential
! read of the first record. 
!--

100            continue
               call read_nest( nest_close, time, ios )
               call read_nest( nest_open, time, ios )
               call read_nest( seq_read, time, ios )
               if ( ios .ne. success ) then
                    call LIB$SIGNAL( msg_setstart )
               end if
               goto 600

!++
! time was "END". Insure the end really is the end by making the end
! the absolute latest time available.
!--

200            continue
               time = '7FFFFFFFFFFFFFFF'
               goto 600

!++
! time was "YESTERDAY" when all my troubles seemed so far away...
!--

300            continue
               status = SYS$BINTIM( mid_nite, bin_mid_nite )
               status = SYS$BINTIM( one_day, bin_one_day )
               status = LIB$ADDX( i_mid_nite, i_one_day, i_systime )
               goto 600

!++
! time was today.
!--

400            continue
               status = SYS$BINTIM( mid_nite, i_systime )
               goto 600

!++
! time was "TOMORROW"
!--

500            continue
               status = SYS$BINTIM( mid_nite, bin_mid_nite )
               status = SYS$BINTIM( one_day, bin_one_day )
               status = LIB$SUBX( i_mid_nite, i_one_day, i_systime )
               goto 600
!++
! if there is an expression following the special keyword, merge it
! into the time accordingly.
!--

600            continue
               if ( pcb(7) .eq. 5 ) then
                  time_buffer = compress( pcb(11) )
                  status = SYS$BINTIM( time_buffer, offset )

!++
! if all was not ok with the off set string let them know
!--

                  if ( status .ne. %LOC(SS$_NORMAL) ) then

                     call LIB$SIGNAL( msg_invalidtime, %VAL(1),
     .                    time_buffer(1:MK$LENGTH(time_buffer)) )
                     ret_code = failure
                     return

                  end if

!++
! off set was OK. If the keyword was START or END, convert the charcter
! representation of the key to a its binary form so that it can be added
! or subtracted with the binary form of the delta time.
!--

                  if ( (pcb(6) .eq. 1) .or. (pcb(6) .eq. 2) ) then
                     decode( 16, 610, time, err=999 ) i_systime
610                  format( 2Z8 )

!++
! put into funny DEC format
!--
                     temp = i_systime(2)
                     i_systime(2) = i_systime(1)
                     i_systime(1) = temp
                  end if


!++
! at last, merge it in
!--

                  goto ( 620, 630 ) pcb(9)

620               continue                    ! was a +
                  call LIB$SUBX( i_systime, i_offset, i_systime )
                  goto 700

630               continue                    ! was a -
                  call LIB$ADDX( i_systime, i_offset, i_systime )
                  goto 700

               end if

               goto ( 900, 900, 700, 700, 700 ) pcb(6)
!++
! ELSE condition of the above.
! Reconstitute the individual character of the time into a single
! string
!--
          else

               time_buffer = compress( pcb(str_len) )

          end if 

          status = SYS$BINTIM( time_buffer, systime )

!++
! if all was not ok with the input string give a msg
!--

          if ( status .ne. %LOC(SS$_NORMAL) ) then

             call LIB$SIGNAL( msg_invalidtime, %VAL(1),
     .                        time_buffer(1:MK$LENGTH(time_buffer)) )
             ret_code = failure
             return

          end if

!++
! turn the binary form of the time into a character form
!--

700       continue
          encode ( 16, 710, time, err=999 ) systime
710       format ( z16 )

900       continue
          return

!++
! error trap on encode or decode
!--

999       continue
          call LIB$SIGNAL( msg_invalidtime, %VAL(1),
     .         time_buffer(1:MK$LENGTH(time_buffer)) )
          ret_code = failure
          return
          end
