!
!			  COPYRIGHT (c) 1982 BY
!	      MOSTEK CORPORATION, CARROLLTON TX
!
! This software is furnished under a license and may be used and  copied
! only  in  accordance  with  the  terms  of  such  license and with the
! inclusion of the above copyright notice.  This software or  any  other
! copies  thereof may not be provided or otherwise made available to any
! other person.  No title to and ownership of  the  software  is  hereby
! transferred.
!
! The information in this software is subject to change  without  notice
! and  should  not  be  construed  as  a commitment by MOSTEK
! CORPORATION.
!
! MOSTEK assumes no responsibility for the use or  reliability  of  its
! software on equipment which is not supplied by MOSTEK.
!

!++
! FACILITY:
!   MOSTEK CORPORATION
!
! ABSTRACT:
!   STATCPU gathers the percentage of time that the cpu spends in the 
!   various states. It also gathers the number of batch, interactive
!   and network jobs in the system.
!
! ENVIRONMENT:
!   VAX11 VMS   
!
! AUTHOR: Gary Smith, CREATION DATE: 6/24/82
!
! MODIFIED BY:
!
!--
!
! TABLE OF CONTENTS:
!
!
! INCLUDE FILES:
!   mousesubs.for
!
! MACROS:
!   none
!
! EQUATED SYMBOLS:
!   see mousesubs.for
!
! EXTERNAL REFERENCES:
!   PMS$GL_KERNEL,SYS$GW_BJOBCNT,SYS$GW_IJOBCNT,SYS$GW_NJOBCNT

!++
! FUNCTIONAL DESCRIPTION:
!   Using PMS$GL_KERNEL as a base, STATCPU gets the total time for each
!   of the modes of the cpu. This is used to calculate the percent spent
!   in each mode since the last inspection by STATCPU
!
!
! FORMAL PARAMETERS:
!   none
!
!
! IMPLICIT INPUTS:
!   rec_cpu_modes
!
!
! IMPLICIT OUTPUTS:
!   rec_cpu_modes
!
!
! ROUTINE VALUE and
! COMPLETION CODES:
!   none
!
!
! SIDE EFFECTS:
!   none to date
!
!
!--
        SUBROUTINE STATCPU              !! CPU activity

        external PMS$GL_KERNEL          !! Tot. times in states
        external SYS$GW_BJOBCNT         !! # of Batch Jobs
        external SYS$GW_IJOBCNT         !! # of Interactive jobs
        external SYS$GW_NJOBCNT         !! # of DECnet jobs
        external SCH$GL_NULLPCB         !! addr of null pcb
        external PCB$L_PHD              !! offset to PHD
        external PHD$L_CPUTIM           !! offset to cpu time

        include 'mousesubs.for'
        include 'defrec.for'

        integer last_mode(cpu_size), phd


!++
! gather up the numbers of the time spent in each mode
!--

          do i = 1, num_modes
             rec_cpu_modes(i) =  ML$L_PEEK(%LOC(PMS$GL_KERNEL)+4*(i-1))
          enddo


          phd = ML$L_PEEK( %LOC(SCH$GL_NULLPCB) +  %LOC(PCB$L_PHD) ) 
          rec_cpu_modes(idle) = ML$L_PEEK( ML$L_PEEK(%LOC(phd)) + 
     .                      %LOC(PHD$L_CPUTIM) )

	  total_time = 0.0

	  do i = 1, num_modes
	    total_time = total_time + ( rec_cpu_modes(i) - 
     .                   last_mode(i) )
	  enddo

	  if ( total_time .le. 0.1 ) total_time = 1.0

	  do i = 1, num_modes
	    temp = rec_cpu_modes(i)
	    rec_cpu_modes(i) = nint( 100*( ( temp - last_mode(i) ) / 
     .                    total_time ) )
	    last_mode(i) = temp
	  enddo

!++
! peek at the batch and interactive counts
!--

	  rec_cpu_modes(bjobcnt) = 'ffff'x .and. 
     .                        ML$L_PEEK(%LOC(SYS$GW_BJOBCNT))
	  rec_cpu_modes(ijobcnt) = 'ffff'x .and. 
     .                        ML$L_PEEK(%LOC(SYS$GW_IJOBCNT))

          return
          end
