!
!			  COPYRIGHT (c) 1982 BY
!	      MOSTEK CORPORATION, CARROLLTON TX
!
! This software is furnished under a license and may be used and  copied
! only  in  accordance  with  the  terms  of  such  license and with the
! inclusion of the above copyright notice.  This software or  any  other
! copies  thereof may not be provided or otherwise made available to any
! other person.  No title to and ownership of  the  software  is  hereby
! transferred.
!
! The information in this software is subject to change  without  notice
! and  should  not  be  construed  as  a commitment by MOSTEK
! CORPORATION.
!
! MOSTEK assumes no responsibility for the use or  reliability  of  its
! software on equipment which is not supplied by MOSTEK.
!

!++
! FACILITY:
!   MOSTEK CORPORATION
!
! ABSTRACT: STATLOCK gathers the ENQ/DEQ info that is kept by VMS.
!   
!
! ENVIRONMENT: VAX VMS unprivileged
!   
!
! AUTHOR: Gary Smith, CREATION DATE: 8/18/1982
!
! MODIFIED BY:
!
!--
!
! TABLE OF CONTENTS:
!
!
! INCLUDE FILES: mousesubs.for
!                defrec.for
!
!
! MACROS: none
!
!
! EQUATED SYMBOLS: see mousesubs.for
!
!
! EXTERNAL REFERENCES:
!
!          PMS$GL_ENQNEW   -> New ENQ's
!          PMS$GL_ENQWAIT  -> ENQ's waiting to complete
!          PMS$GL_DEQ      -> DEQ's
!          PMS$GL_DLCKFND  -> DEQ's locked to find
!          PMS$GL_DLCKSRCH -> DEQ's locked to search
!          PMS$GL_ENQCVT   -> Converted ENQ
!          PMS$GL_ENQNOTQD -> ENQ not queued

!++
! FUNCTIONAL DESCRIPTION: ENQ/DEQ is now an integral part of VMS with 3.0
!                         and thus the OS keeps stats on it. STATLOCK gathers
!                         these stats with the aid of ML$L_PEEK. These are
!                         kept as a difference measurement.
!
!
! FORMAL PARAMETERS: none
!
!
!
! IMPLICIT INPUTS: none
!
!
!
! IMPLICIT OUTPUTS: lock_modes
!
!
!
! ROUTINE VALUE and
! COMPLETION CODES: none
!
!
!
! SIDE EFFECTS: none to date
!
!
!
!--
          SUBROUTINE STATLOCK

          include 'mousesubs.for'
          include 'defrec.for'

          external PMS$GL_ENQNEW   
          external PMS$GL_ENQWAIT  
          external PMS$GL_DEQ      
          external PMS$GL_DLCKFND  
          external PMS$GL_DLCKSRCH 
          external PMS$GL_ENQCVT   
          external PMS$GL_ENQNOTQD 

          external ML$L_PEEK

          integer  ML$L_PEEK
          integer  last(lock_size), temp

!++
! Gather up the ENQ/DEQ stats
!--

          rec_lock_modes(enqnew   ) = 
     .                    ML$L_PEEK( %LOC( PMS$GL_ENQNEW ) )
          rec_lock_modes(enqwait  ) = 
     .                    ML$L_PEEK( %LOC( PMS$GL_ENQWAIT ) )
          rec_lock_modes(deq      ) = 
     .                    ML$L_PEEK( %LOC( PMS$GL_DEQ ) )
          rec_lock_modes(dlckfnd  ) = 
     .                    ML$L_PEEK( %LOC( PMS$GL_DLCKFND  ) )
          rec_lock_modes(dlcksrch ) = 
     .                    ML$L_PEEK( %LOC( PMS$GL_DLCKSRCH ) )
          rec_lock_modes(enqcvt   ) = 
     .                    ML$L_PEEK( %LOC( PMS$GL_ENQCVT   ) )
          rec_lock_modes(enqnotqd ) = 
     .                    ML$L_PEEK( %LOC( PMS$GL_ENQNOTQD ) )

!++
! Compute the delta from the last time
!--

          do i = 1, lock_size
               temp = rec_lock_modes(i)
               rec_lock_modes(i) = rec_lock_modes(i) - last(i)
               last(i) = temp
          end do

          return
          end
