!
!			  COPYRIGHT (c) 1982 BY
!	      MOSTEK CORPORATION, CARROLLTON TX
!
! This software is furnished under a license and may be used and  copied
! only  in  accordance  with  the  terms  of  such  license and with the
! inclusion of the above copyright notice.  This software or  any  other
! copies  thereof may not be provided or otherwise made available to any
! other person.  No title to and ownership of  the  software  is  hereby
! transferred.
!
! The information in this software is subject to change  without  notice
! and  should  not  be  construed  as  a commitment by MOSTEK
! CORPORATION.
!
! MOSTEK assumes no responsibility for the use or  reliability  of  its
! software on equipment which is not supplied by MOSTEK.
!

!++
! FACILITY:
!   MOSTEK CORPORATION
!
! ABSTRACT:  STATS is the second half of the dichotomy of a stats reporting
!            system. STATS reads the RATSNEST file left by MOUSE for 
!            information about the performance of the system. STATS forms
!            the user interface also
!            
!
! ENVIRONMENT: VAX/VMS unpriveleged
!   
!
! AUTHOR: Gary Smith, CREATION DATE: Aug 2 1982
!
! MODIFIED BY:
!
!--
!
! TABLE OF CONTENTS:
!
!
! INCLUDE FILES:
!   mousesubs.for
!   statsubs.for
!
! MACROS:
!   none
!
! EQUATED SYMBOLS:
!   see mousesubs.for
!   see statsubs.for
!
! EXTERNAL REFERENCES:
!                     SYS$BINTIM, SYS$SCHDWK, SYS$HIBER, SS$_NORMAL
!                     JPI$_STS, PCB$V_BATCH, SYS$GETJPI, SYS$GETMSG
!
!++
! FUNCTIONAL DESCRIPTION: STATS reads the user commands. The command line
!                         is passed to PARSER which does the hard part of 
!                         parsing input. Based on the success or failure 
!                         of the parse, STATS reads information from the 
!                         RATSNEST file left by MOUSE. This information
!                         is either presented in tabular form or is presented
!                         in a plot ( Communist or otherwise ).
!
!
!
! FORMAL PARAMETERS:
!   none
!
!
! IMPLICIT INPUTS:
!   RATSNEST file
!
!
! IMPLICIT OUTPUTS:
!   Tabular information or plots
!
!
! ROUTINE VALUE and
! COMPLETION CODES:
!   none
!
!
! SIDE EFFECTS:
!   none to date
!
!
!--
          program stats

!++
! subs for the sizes of local things in STATS
!--

          parameter lx_size        = 10000     ! size of grammar lexicon
          parameter in_buffer_size = 256       ! size of input buffer
          parameter pcb_size       = 256       ! size of parse control blk
          parameter stack_size     = 500       ! size of parser work space
          parameter keyword_size   = 9         ! size of root node
          parameter file_len       = 64        ! size of file names
          parameter interrupt_bell = 7         ! alarm bell
          parameter command_size   = 11        ! number of commands
          parameter index1         = 1         ! index for equivalences
          parameter index2         = 990 * 1 + 1 ! index for equivalences
          parameter index3         = 990 * 2 + 1 ! index for equivalences
          parameter index4         = 990 * 3 + 1 ! index for equivalences

          include 'statsubs.for'
          include 'mousesubs.for'

          dimension lx1(990), lx2(990), lx3(990), lx4(310)
          dimension lx(lx_size), in_buffer(in_buffer_size)
          dimension stack(stack_size), pcb(pcb_size)
          dimension keyword(keyword_size)
          
          character *  9  stats_commands(command_size)
          character * 16  start_time, end_time, saved_start, saved_end
          character * 16  time_1, time_2
          character * 23  key_to_date
          character * 23  time_of_day, snooze_spec
          character * 27  greeting / ' STATS Version 1.1 here at ' /
          character * 63  cat_title(rec_size)
          character * 80  out_buffer
          character * 132 compress

          character * (file_len)  nest_file, old_nest_file
          character * (file_len)  file_spec

          logical start_set, end_set, file_present, alarm_set
          logical multi_plot, batch

          integer in_unit, out_unit, pcb, stack
          integer end, start, trace, ret_code
          integer key_compare, entries, pcb_ptr
          integer status, frame, frames_to_do
          integer item_list(3), delta_time(2), deltat
          integer cat_state(max_cat, max_ent), cat_len(rec_size)
          integer SYS$BINTIM, SYS$SCHDWK, SYS$HIBER, SYS$GETMSG

          real * 8 snooze_time

          common / io_com / in_unit, out_unit, batch
          common / nest_common / nest_file
          common / category / cat_state, cat_title, cat_len

          equivalence ( lx1(1), lx(index1) )
          equivalence ( lx2(1), lx(index2) )
          equivalence ( lx3(1), lx(index3) )
          equivalence ( lx4(1), lx(index4) )

          data start_set / .false. /, end_set / .false. /

          data stats_commands( 1) / 'SET START' /
          data stats_commands( 2) / 'SET END'   /
          data stats_commands( 3) / 'DISPLAY' /
          data stats_commands( 4) / 'EXIT' /
          data stats_commands( 5) / 'PLOT' /
          data stats_commands( 6) / 'USE' /
          data stats_commands( 7) / 'PDL' /
          data stats_commands( 8) / 'SHOW' /
          data stats_commands( 9) / 'REPLAY' /
          data stats_commands(10) / 'SNOOZE' /
          data stats_commands(11) / 'HELP' /

          data nest_file / standard_nest_file /

          data lx1 /
     .    1,  503, 3279,    1,    0,    0,    0,    0,    0,    0,          0
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,         10
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,         20
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,         30
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,         40
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,         50
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,  508,         60
     .    0,  758, 1263, 1403, 1473,    0, 1557,    0,    0,    0,         70
     . 1756, 2019,    0, 1276, 2462,    0, 2554, 2745,    0,  790,         80
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,         90
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        100
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        110
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        120
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        130
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        140
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        150
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        160
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        170
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        180
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        190
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        200
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        210
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        220
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        230
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        240
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        250
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        260
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        270
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        280
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        290
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        300
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        310
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        320
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        330
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        340
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        350
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        360
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        370
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        380
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        390
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        400
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        410
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        420
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        430
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        440
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        450
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        460
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        470
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        480
     .    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,        490
     .    0,    0,    0,    0,    0,    0,    0,    0,    5,   60,        500
     .   65,   67,   84,   62,  583,    0,    0,    9,   60,   68,        510
     .   73,   83,   80,   76,   65,   89,   62, 1386,    0,    0,        520
     .    6,   60,   80,   76,   79,   84,   62, 2537,    0,  530,        530
     .    5,   60,   80,   68,   76,   62, 2508,    0,    0,    6,        540
     .   60,   83,   72,   79,   87,   62, 3254,    0,    0,    8,        550
     .   60,   82,   69,   80,   76,   65,   89,   62, 2633,    0,        560
     .  549,    8,   60,   83,   78,   79,   79,   90,   69,   62,        570
     . 3209,    0,   32,    5,    1,    5,    1,    3,  518,    1,        580
     .    9,    3,  531,    1,   13,    3,  541,    1,   17,    3,        590
     .  550,    1,   21,    3,  560,    1,   25,    3,  572,    1,        600
     .   29,    6,    1,    1,    0,    0,    5,   60,   67,   80,        610
     .   85,   62,  713,    0,    6,    3,   75,   69,   82,   78,        620
     .   69,   76,    2,    1,    1,    5,    3,   83,   85,   80,        630
     .   69,   82,    2,    1,    2,    4,    3,   69,   88,   69,        640
     .   67,    2,    1,    3,    4,    3,   85,   83,   69,   82,        650
     .    2,    1,    4,    3,    3,   73,   78,   84,    2,    1,        660
     .    5,    6,    6,   67,   79,   77,   80,   65,   84,    2,        670
     .    1,    6,    4,    3,   73,   68,   76,   69,    2,    1,        680
     .    7,    5,    3,   66,   65,   84,   67,   72,    2,    1,        690
     .    8,    6,    3,   79,   78,   76,   73,   78,   69,    2,        700
     .    1,    9,   44,    5,    1,    5,    1,   15,  625,    1,        710
     .    9,   15,  636,    1,   13,   15,  646,    1,   17,   15,        720
     .  655,    1,   21,   15,  664,    1,   25,   15,  672,    1,        730
     .   29,   15,  683,    1,   33,   15,  692,    1,   37,   15,        740
     .  702,    1,   41,    6,    1,    1,    0,  616,    9,   60,        750
     .   67,   79,   77,   77,   65,   78,   68,   62,  818,    0,        760
     .  517,    5,   60,   68,   69,   67,   62, 1354,    0,    0,        770
     .    6,   60,   69,   88,   73,   84,   62, 1456,    0,    0,        780
     .    5,   60,   85,   83,   69,   62, 3270,    0,    0,    6,        790
     .   60,   72,   69,   76,   80,   62, 1739,    0,  771,    5,        800
     .   60,   68,   67,   76,   62, 1311,    0,   32,    5,    1,        810
     .    5,    1,    3,  772,    1,    9,    3,  509,    1,   13,        820
     .    3,  781,    1,   17,    3,  791,    1,   21,    3,  800,        830
     .    1,   25,    3,  810,    1,   29,    6,    1,    1,    0,        840
     .  809,    9,   60,   68,   73,   95,   76,   73,   83,   84,        850
     .   62,  880,    0,  851,    8,   60,   68,   73,   95,   79,        860
     .   80,   84,   62, 1137,    0,    1,    1,   44,    0,   32,        870
     .    3,  865,    5,    0,    5,   20,    9,   21,   15,  876,        880
     .   13,   17,    3,  865,    1,   17,    6,    0,   21,    0,        890
     .    5,    1,   25,    1,   13,    0,    1,   29,    6,    0,        900
     .    1,    0,  864,    8,   60,   68,   69,   67,   78,   69,        910
     .   84,   62,  990,    0,    8,    4,   65,   82,   82,   76,        920
     .   79,   67,   80,   75,    2,    9,    1,    8,    4,   65,        930
     .   82,   82,   84,   82,   65,   80,   75,    2,    9,    2,        940
     .    8,    3,   68,   69,   80,   76,   79,   67,   80,   75,        950
     .    2,    9,    3,    8,    3,   82,   67,   86,   66,   85,        960
     .   70,   70,   76,    2,    9,    4,    8,    3,   84,   82,        970
     .   67,   78,   71,   76,   79,   83,    2,    9,    5,   28/        980
 

          data lx2 /
     .    5,    1,    5,    1,   15,  925,    1,    9,   15,  938,        990
     .    1,   13,   15,  951,    1,   17,   15,  964,    1,   21,       1000
     .   15,  977,    1,   25,    6,    1,    1,    0,    3,    3,       1010
     .   67,   80,   85,    2,    3,    1,    3,    3,   70,   67,       1020
     .   80,    2,    3,    2,    4,    4,   77,   73,   83,   67,       1030
     .    2,    3,    3,    6,    3,   77,   69,   77,   79,   82,       1040
     .   89,    2,    3,    4,    5,    5,   80,   71,   70,   76,       1050
     .   84,    2,    3,    5,    5,    3,   83,   67,   72,   69,       1060
     .   68,    2,    3,    6,    4,    4,   83,   87,   65,   80,       1070
     .    2,    3,    7,    6,    2,   85,   80,   84,   73,   77,       1080
     .   69,    2,    3,    8,    6,    3,   68,   69,   67,   78,       1090
     .   69,   84,    2,    3,    9,    4,    4,   76,   79,   67,       1100
     .   75,    2,    3,   10,    8,    3,   82,   69,   83,   80,       1110
     .   79,   78,   83,   69,    2,    3,   11,    4,    4,   80,       1120
     .   79,   79,   76,    2,    3,   12,   56,    5,    1,    5,       1130
     .    1,   15, 1019,    1,    9,   15, 1027,    1,   13,   15,       1140
     . 1035,    1,   17,   15, 1044,    1,   21,   15, 1055,    1,       1150
     .   25,   15, 1065,    1,   29,   15, 1075,    1,   33,   15,       1160
     . 1084,    1,   37,   15, 1095,    1,   41,   15, 1106,    1,       1170
     .   45,   15, 1115,    1,   49,   15, 1128,    1,   53,    6,       1180
     .    1,    1,    0,  913,   10,   60,   68,   65,   89,   95,       1190
     .   78,   65,   77,   69,   62, 1242,    0,    9,    3,   89,       1200
     .   69,   83,   84,   69,   82,   68,   65,   89,    1,    3,       1210
     .    5,    3,   84,   79,   68,   65,   89,    1,    4,    8,       1220
     .    3,   84,   79,   77,   79,   82,   82,   79,   87,    1,       1230
     .    5,   20,    5,    1,    5,    1,   15, 1208,    1,    9,       1240
     .   15, 1221,    1,   13,   15, 1230,    1,   17,    6,    1,       1250
     .    1,    0, 1194,    9,   60,   68,   65,   89,   95,   69,       1260
     .   88,   80,   62, 1289,    0,    0,    9,   60,   79,   70,       1270
     .   70,   95,   83,   69,   84,   62, 2078,    0,   16,    3,       1280
     . 1195,    5,    0,    5,    1,    9,    1,    3, 1277,    1,       1290
     .   13,    6,    0,    1,    0,    1,    1,   36,    1,   12,       1300
     .    8,   15, 1306,    5,    0,   13,    0,    1,    0,    3,       1310
     .    2,   83,   69,   84,    0,  571,   11,   60,   83,   69,       1320
     .   84,   95,   83,   84,   65,   82,   84,   62, 3144,    0,       1330
     . 1326,    9,   60,   83,   69,   84,   95,   69,   78,   68,       1340
     .   62, 3096,    0,   20,   15, 1320,    5,    0,    5,    1,       1350
     .    9,    1,    3, 1327,    1,   13,    3, 1342,    1,   17,       1360
     .    6,    1,    1,    0,    7,    2,   68,   73,   83,   80,       1370
     .   76,   65,   89,    1,    3,   16,    5,    1,    5,    1,       1380
     .   15, 1375,    9,   13,    3,  852,    1,   13,    6,    1,       1390
     .    1,    0,  780,    9,   60,   69,   78,   68,   95,   69,       1400
     .   88,   80,   62, 1423,    0,    3,    3,   69,   78,   68,       1410
     .    1,    2,   16,   15, 1416,    5,    0,    5,    1,    9,       1420
     .    1,    3, 1277,    1,   13,    6,    0,    1,    0,    4,       1430
     .    2,   69,   88,   73,   84,    1,    4,    4,    1,   81,       1440
     .   85,   73,   84,    1,    4,   16,    5,    1,    5,    1,       1450
     .   15, 1440,    1,    9,   15, 1448,    1,   13,    6,    1,       1460
     .    1,    0,    0,    5,   60,   70,   67,   80,   62, 1528,       1470
     .    0,    4,    3,   67,   65,   76,   76,    2,    2,    1,       1480
     .    5,    3,   67,   65,   67,   72,   69,    2,    2,    2,       1490
     .    3,    3,   67,   80,   85,    2,    2,    3,    4,    3,       1500
     .   82,   69,   65,   68,    2,    2,    4,    5,    3,   87,       1510
     .   82,   73,   84,   69,    2,    2,    5,   28,    5,    1,       1520
     .    5,    1,   15, 1482,    1,    9,   15, 1491,    1,   13,       1530
     .   15, 1501,    1,   17,   15, 1509,    1,   21,   15, 1518,       1540
     .    1,   25,    6,    1,    1,    0,  799,   10,   60,   72,       1550
     .   73,   84,   95,   76,   73,   83,   84,   62, 1682,    0,       1560
     .    5,    5,   79,   80,   69,   78,   83,    2,    3,    9,       1570
     .    4,    4,   79,   80,   69,   78,    2,    3,   10,    6,       1580
     .    4,   68,   73,   82,   72,   73,   84,    2,    3,   11,       1590
     .    7,    4,   68,   73,   82,   77,   73,   83,   83,    2,       1600
     .    3,   12,    6,    4,   69,   88,   84,   72,   73,   84,       1610
     .    2,    3,   13,    7,    4,   69,   88,   84,   77,   73,       1620
     .   83,   83,    2,    3,   14,    6,    4,   70,   73,   68,       1630
     .   72,   73,   84,    2,    3,   15,    7,    4,   70,   73,       1640
     .   68,   77,   73,   83,   83,    2,    3,   16,    6,    4,       1650
     .   81,   85,   79,   72,   73,   84,    2,    3,   17,    7,       1660
     .    4,   81,   85,   79,   77,   73,   83,   83,    2,    3,       1670
     .   18,   48,    5,    1,    5,    1,   15, 1571,    1,    9,       1680
     .   15, 1581,    1,   13,   15, 1590,    1,   17,   15, 1601,       1690
     .    1,   21,   15, 1613,    1,   25,   15, 1624,    1,   29,       1700
     .   15, 1636,    1,   33,   15, 1647,    1,   37,   15, 1659,       1710
     .    1,   41,   15, 1670,    1,   45,    6,    1,    1,    0,       1720
     .    4,    2,   72,   69,   76,   80,    1,   11,   16,   15,       1730
     . 1731,    5,    0,    5,    1,    9,    1,   13,    0,    1,       1740
     .   13,    6,    0,    1,    0,    0,    6,   60,   76,   79,       1750
     .   67,   75,   62, 1846,    0,    6,    5,   69,   78,   81,       1760
     .   78,   69,   87,    2,   10,    1,    7,    4,   69,   78,       1770
     .   81,   87,   65,   73,   84,    2,   10,    2,    3,    3,       1780
     .   68,   69,   81,    2,   10,    3,    7,    5,   68,   76,       1790
     .   67,   75,   70,   78,   68,    2,   10,    4,    8,    5,       1800
     .   68,   76,   67,   75,   83,   82,   67,   72,    2,   10,       1810
     .    5,    6,    4,   69,   78,   81,   67,   78,   84,    2,       1820
     .   10,    6,    8,    4,   69,   78,   81,   78,   79,   84,       1830
     .   81,   68,    2,   10,    7,   36,    5,    1,    5,    1,       1840
     .   15, 1766,    1,    9,   15, 1777,    1,   13,   15, 1789,       1850
     .    1,   17,   15, 1797,    1,   21,   15, 1809,    1,   25,       1860
     .   15, 1822,    1,   29,   15, 1833,    1,   33,    6,    1,       1870
     .    1,    0,    0,    6,   60,   77,   73,   83,   67,   62,       1880
     . 1974,    0,    5,    5,   68,   73,   82,   73,   79,    2,       1890
     .    3,    1,    5,    5,   66,   85,   70,   73,   79,    2,       1900
     .    3,    2,    6,    3,   76,   79,   71,   78,   65,   77,       1910
     .    2,    3,    3,    6,    3,   77,   66,   82,   69,   65,       1920
     .   68,    2,    3,    4,    7,    3,   77,   66,   87,   82,       1930
     .   73,   84,   69,    2,    3,    5,    4,    3,   84,   85,       1940
     .   82,   78,    2,    3,    6,    5,    3,   83,   80,   76,       1950
     .   73,   84,    2,    3,    7,    3,    3,   72,   73,   84,       1960
     .    2,    3,    8,   44,    5,    1,    5,    1,   15, 1893/       1970
 

          data lx3 / 
     .    1,    9,   15, 1903,    1,   13,   15, 1913,    1,   17,       1980
     .   15, 1924,    1,   21,   15, 1935,    1,   25,   15, 1947,       1990
     .    1,   29,   15, 1956,    1,   33,   15, 1966,    1,   37,       2000
     .    3, 1558,    1,   41,    6,    1,    1,    0, 1883,    5,       2010
     .   60,   77,   69,   77,   62, 2051,    0,    7,    3,   70,       2020
     .   82,   69,   69,   67,   78,   84,    2,    4,    1,    6,       2030
     .    3,   77,   70,   89,   67,   78,   84,    2,    4,    2,       2040
     .   16,    5,    1,    5,    1,   15, 2028,    1,    9,   15,       2050
     . 2040,    1,   13,    6,    1,    1,    0,    1,    1,   43,       2060
     .    1,    1,    1,    1,   45,    1,    2,   20,    5,    1,       2070
     .    5,   17,   15, 2068,    1,    9,   15, 2073,    1,   13,       2080
     .    6,    1,   17,    0,   13,    0,    1,    0,  540,    6,       2090
     .   60,   80,   79,   79,   76,   62, 2198,    0,    6,    3,       2100
     .   73,   82,   80,   67,   78,   84,    2,   12,    1,    5,       2110
     .    3,   72,   79,   76,   69,   83,    2,   12,    2,    6,       2120
     .    3,   76,   66,   76,   79,   67,   75,    2,   12,    3,       2130
     .    6,    3,   83,   66,   76,   79,   67,   75,    2,   12,       2140
     .    4,    8,    3,   83,   77,   65,   76,   76,   67,   78,       2150
     .   84,    2,   12,    5,    6,    3,   84,   83,   80,   65,       2160
     .   67,   69,    2,   12,    6,    6,    3,   76,   82,   80,       2170
     .   67,   78,   84,    2,   12,    7,    6,    3,   83,   82,       2180
     .   80,   67,   78,   84,    2,   12,    8,   40,    5,    1,       2190
     .    5,    1,   15, 2109,    1,    9,   15, 2120,    1,   13,       2200
     .   15, 2130,    1,   17,   15, 2141,    1,   21,   15, 2152,       2210
     .    1,   25,   15, 2165,    1,   29,   15, 2176,    1,   33,       2220
     .   15, 2187,    1,   37,    6,    1,    1,    0, 2099,    7,       2230
     .   60,   80,   71,   70,   76,   84,   62, 2332,    0,    8,       2240
     .    3,   68,   90,   82,   79,   70,   76,   84,   83,    2,       2250
     .    5,    1,    6,    3,   70,   65,   85,   76,   84,   83,       2260
     .    2,    5,    2,    7,    7,   80,   82,   69,   65,   68,       2270
     .   73,   79,    2,    5,    3,    7,    7,   80,   87,   82,       2280
     .   73,   84,   73,   79,    2,    5,    4,    6,    3,   80,       2290
     .   82,   69,   65,   68,   83,    2,    5,    5,    7,    3,       2300
     .   80,   87,   82,   73,   84,   69,   83,    2,    5,    6,       2310
     .    6,    3,   82,   68,   70,   76,   84,   83,    2,    5,       2320
     .    7,   36,    5,    1,    5,    1,   15, 2250,    1,    9,       2330
     .   15, 2263,    1,   13,   15, 2274,    1,   17,   15, 2286,       2340
     .    1,   21,   15, 2298,    1,   25,   15, 2309,    1,   29,       2350
     .   15, 2321,    1,   33,    6,    1,    1,    0, 2239,    8,       2360
     .   60,   80,   76,   95,   79,   80,   84,   62, 2409,    0,       2370
     . 1341,    5,   60,   83,   67,   72,   62, 3016,    0, 2381,       2380
     .    6,   60,   83,   87,   65,   80,   62, 2861,    0,  559,       2390
     .    5,   60,   82,   69,   83,   62, 2618,    0,   52,    5,       2400
     .    1,    5,    1,    3,  617,    1,    9,    3, 1474,    1,       2410
     .   13,    3, 1884,    1,   17,    3, 2020,    1,   21,    3,       2420
     . 2240,    1,   25,    3, 2382,    1,   29,    3, 2391,    1,       2430
     .   33,    3, 2401,    1,   37,    3,  914,    1,   41,    3,       2440
     . 1757,    1,   45,    3, 2100,    1,   49,    6,    1,    1,       2450
     .    0, 2369,   10,   60,   80,   68,   76,   95,   76,   73,       2460
     .   83,   84,   62, 2480,    0,    1,    1,   44,    0,   20,       2470
     .    3, 2370,    5,    0,    5,    3,    9,    1,   15, 2476,       2480
     .   13,   17,    3, 2370,    1,   17,    6,    0,    1,    0,       2490
     .    3,    3,   80,   68,   76,    1,    7,   20,    5,    1,       2500
     .    5,    1,   15, 2501,    9,   17,    3, 2463,   13,   17,       2510
     .   13,    0,    1,   17,    6,    1,    1,    0,    4,    2,       2520
     .   80,   76,   79,   84,    1,    5,   16,    5,    1,    5,       2530
     .    1,   15, 2529,    9,   13,    3, 2370,    1,   13,    6,       2540
     .    1,    1,    0, 2400,   10,   60,   82,   69,   80,   95,       2550
     .   76,   73,   83,   84,   62, 2572,    0,    1,    1,   44,       2560
     .    0,   32,    3, 2370,    5,    0,    5,   20,    9,   21,       2570
     .   15, 2568,   13,   17,    3, 2370,    1,   17,    6,    0,       2580
     .   21,    0,    5,    1,   25,    1,   13,    0,    1,   29,       2590
     .    6,    0,    1,    0,    8,    3,   82,   69,   83,   80,       2600
     .   79,   78,   83,   69,    2,   11,    1,    4,   15, 2605,       2610
     .    1,    0,    6,    3,   82,   69,   80,   76,   65,   89,       2620
     .    1,    9,   16,    5,    1,    5,    1,   15, 2623,    9,       2630
     .   13,    3, 2555,    1,   13,    6,    1,    1,    0, 2390,       2640
     .    8,   60,   83,   72,   95,   79,   80,   84,   62, 2686,       2650
     .    0,    5,    2,   83,   84,   65,   82,   84,    1,    1,       2660
     .    3,    3,   69,   78,   68,    1,    2,    4,    4,   70,       2670
     .   73,   76,   69,    1,    3,   20,    5,    1,    5,    1,       2680
     .   15, 2662,    1,    9,   15, 2671,    1,   13,   15, 2678,       2690
     .    1,   17,    6,    1,    1,    0, 2650,    9,   60,   83,       2700
     .   72,   95,   76,   73,   83,   84,   62, 2724,    0,    1,       2710
     .    1,   44,    0,   20,    3, 2651,    5,    0,    5,    4,       2720
     .    9,    1,   15, 2720,   13,   17,    3, 2708,    1,   17,       2730
     .    6,    0,    1,    0, 2707,   11,   60,   83,   84,   65,       2740
     .   82,   84,   95,   69,   88,   80,   62, 2769,    0,    5,       2750
     .    2,   83,   84,   65,   82,   84,    1,    1,   16,   15,       2760
     . 2760,    5,    0,    5,    1,    9,    1,    3, 1277,    1,       2770
     .   13,    6,    0,    1,    0,    8,    3,   73,   83,   80,       2780
     .   65,   71,   67,   78,   84,    2,    7,    1,    7,    3,       2790
     .   73,   83,   87,   80,   67,   78,   84,    2,    7,    2,       2800
     .    7,    3,   80,   82,   79,   67,   67,   78,   84,    2,       2810
     .    7,    3,    7,    3,   79,   83,   87,   80,   67,   78,       2820
     .   84,    2,    7,    4,    8,    3,   72,   79,   83,   87,       2830
     .   80,   67,   78,   84,    2,    7,    5,    8,    3,   72,       2840
     .   73,   83,   87,   80,   67,   78,   84,    2,    7,    6,       2850
     .   32,    5,    1,    5,    1,   15, 2786,    1,    9,   15,       2860
     . 2799,    1,   13,   15, 2811,    1,   17,   15, 2823,    1,       2870
     .   21,   15, 2835,    1,   25,   15, 2848,    1,   29,    6,       2880
     .    1,    1,    0,    5,    3,   67,   79,   76,   80,   71,       2890
     .    2,    6,    1,    5,    3,   77,   87,   65,   73,   84,       2900
     .    2,    6,    2,    3,    3,   67,   69,   70,    2,    6,       2910
     .    3,    3,    3,   80,   70,   87,    2,    6,    4,    3,       2920
     .    3,   76,   69,   70,    2,    6,    5,    4,    4,   76,       2930
     .   69,   70,   79,    2,    6,    6,    3,    3,   72,   73,       2940
     .   66,    2,    6,    7,    4,    4,   72,   73,   66,   79,       2950
     .    2,    6,    8,    5,    5,   83,   85,   83,   80,   79/       2960
 

          data lx4 /
     .    2,    6,   10,    4,    4,   83,   85,   83,   80,    2,       2970
     .    6,    9,    3,    3,   70,   80,   71,    2,    6,   11,       2980
     .    3,    3,   67,   79,   77,    2,    6,   12,    4,    4,       2990
     .   67,   79,   77,   79,    2,    6,   13,    3,    3,   67,       3000
     .   85,   82,    2,    6,   14,   64,    5,    1,    5,    1,       3010
     .   15, 2894,    1,    9,   15, 2904,    1,   13,   15, 2914,       3020
     .    1,   17,   15, 2922,    1,   21,   15, 2930,    1,   25,       3030
     .   15, 2938,    1,   29,   15, 2947,    1,   33,   15, 2955,       3040
     .    1,   37,   15, 2964,    1,   41,   15, 2974,    1,   45,       3050
     .   15, 2983,    1,   49,   15, 2991,    1,   53,   15, 2999,       3060
     .    1,   57,   15, 3008,    1,   61,    6,    1,    1,    0,       3070
     .    3,    3,   69,   78,   68,    1,    2,    5,    2,   83,       3080
     .   84,   65,   82,   84,    0,   32,   15, 3081,    5,    0,       3090
     .    5,    1,    9,    1,    3, 1404,    1,   13,   15, 3088,       3100
     .   17,   21,    9,    0,    1,   29,    3, 1264,    1,   25,       3110
     .   13,    0,    1,   29,    6,    1,    1,    0,    5,    2,       3120
     .   83,   84,   65,   82,   84,    1,    1,    3,    3,   69,       3130
     .   78,   68,    0,   32,   15, 3129,    5,    0,    5,    1,       3140
     .    9,    1,    3, 2746,    1,   13,   15, 3138,   17,   21,       3150
     .    9,    0,    1,   29,    3, 1264,    1,   25,   13,    0,       3160
     .    1,   29,    6,    1,    1,    0,    6,    2,   83,   78,       3170
     .   79,   79,   90,   69,    1,   10,    5,    2,   65,   76,       3180
     .   65,   82,   77,    1,    1,    4,    4,   84,   73,   76,       3190
     .   76,    0,    3,    3,   70,   79,   82,    0,   36,   15,       3200
     . 3177,    5,    0,    5,    1,    9,   17,   15, 3187,    1,       3210
     .   13,    6,    0,   17,    0,    5,    1,   21,   33,   15,       3220
     . 3196,    1,   25,   15, 3203,    1,   29,    6,    1,   33,       3230
     .    0,   13,    0,    1,    0,    4,    2,   83,   72,   79,       3240
     .   87,    1,    8,    8,   15, 3246,    5,    0,    3, 2708,       3250
     .    1,    0,    3,    3,   85,   83,   69,    1,    6,    8,       3260
     .   15, 3263,    5,    0,   13,    0,    1,    0,   16,   60/       3270



          external SYS$BINTIM, SYS$SCHDWK, SYS$HIBER, SS$_NORMAL
          external JPI$_STS, PCB$V_BATCH, SYS$GETJPI, SYS$GETMSG

          external msg_setstart, msg_nochange, msg_endequal
          external msg_notset, msg_endless, msg_opdipfil
          external msg_oppdlfil, msg_opperfil, msg_startset
          external msg_endset, msg_oprepfil, msg_invalidtime
          external msg_syntax, msg_inlilenex, msg_unknown, msg_tooearly

!++
! start type or root node
!--
          data keyword/ 60, 67, 79, 77, 77, 65, 78, 68, 62 /
!                       <   C   O   M   M   A   N   D   >
          in_unit = 5
          out_unit = 6

!++
! open wide for input and output
!--

          open( unit=in_unit, file='SYS$INPUT:', recl=256, iostat=ios,
     .          carriagecontrol='NONE', status='OLD' )
          open( unit=out_unit, file='SYS$OUTPUT:', recl=256,
     .          iostat=ios, carriagecontrol='FORTRAN', status='OLD' )

!++
! determine if STATS is being run as a batch job
!--
          batch = .true.
          item_list(1) = 4 + %LOC(JPI$_STS)*(2**16) 
          item_list(2) = %LOC(jpi_sts)
          status = SYS$GETJPI( %VAL(0), %VAL(0), %VAL(0),
     .                         item_list, %VAL(0), %VAL(0), %VAL(0) )
          is_batch = 2**%LOC(PCB$V_BATCH)
          if ( iand(jpi_sts, is_batch) .eq. 0 ) then 
               call controlt
               batch = .false.
          end if


!++
! get time of day and date for later use and write a cheery msg
!--

          status = SYS$ASCTIM( , time_of_day, , )
          out_buffer = greeting // time_of_day 
          write( out_unit, 5 ) out_buffer(1:MK$LENGTH(out_buffer))
5         format( /, A )
          call read_nest( nest_open, start_time, ios )

!++
! Determine what the sampling ratio is by the difference in time keys )
!--

          call read_nest( seq_read, time_1, ios )
          call read_nest( seq_read, time_2, ios )
          interval = deltat( time_1, time_2 )
!++
! build up the correspondence table between the lengths of the titles
! in category common and the titles. I know this is a bit stupid to
! do this each time the program is done, but I would rather the program
! do it than I do it. So there!
!--

          do i = 1, rec_size
             cat_len(i) = MK$LENGTH(cat_title(i))
          end do

!++
! set trace flag off
!--
          trace = 0
!++
! read user's request and zero pcb
!--

10        continue
          i = 1
          call terminal_read( in_buffer, i, end )
          do j = 1, pcb_size
             pcb(j) = 0
          end do
!++
! if request is null, prompt again
!--
          if ( end .eq. 0 ) goto 10
!++
! determine the statement type
!--
          start = 1
          l = keyword_size
          call parser( keyword, l, in_buffer, start, end, lx, stack,
     .                 trace, irkf, pcb, ltr, ret_code )

!++
! check that statement type was valid
!--
          if ( ret_code .eq. success ) then
!++
! pick up statement type
! 1  = set start 
! 2  = set end
! 3  = display
! 4  = end
! 5  = plot
! 6  = use
! 7  = pdl
! 8  = show
! 9  = replay
! 10 = snooze
! 11 = help
! 12 = a DCL command
! note that we are unconcerned with the first two elements in stack
! they indicate that a literal was found and how many
!--
             i = pcb(3)
             start = irkf
             goto ( 100, 200, 300, 400, 500, 600, 500, 800, 900,
     .              1000, 1100, 1200 ) i

!++
! command was SET START
!--

100          continue
             saved_start = start_time
             start_set = .false.
!++
! Determine if the starting time is the first time in the file or
! if it is some other time. 
!--
             call set_time( pcb, start_time, ret_code )

             if ( ret_code .ne. success ) then
                  start_time = saved_start
             else
                  start_set = .true.
             endif

             goto 10

!++
! command was SET END
!--

200          continue

!++
! determine if the end is to be the end of the file or a regular time
!--

             end_set = .false.
             saved_end = end_time

             call set_time( pcb, end_time, ret_code )

             if ( ret_code .ne. success ) then
                  end_time = saved_end
             else
                  end_set = .true.
             endif

             if ( end_set ) then
                if ( key_compare(end_time, start_time) ) 210, 220, 230

!++
! end_time < start_time 
!--

210                continue
                   call LIB$SIGNAL( msg_endless )
                   call LIB$SIGNAL( msg_nochange )
                   end_time = saved_end
                   end_set = .false.
                   goto 10

!++
! end_time = start_time
!--

220                continue
                   call LIB$SIGNAL( msg_endequal )
                   call LIB$SIGNAL( msg_nochange )
                   end_time = saved_end
                   end_set = .false.
                   goto 10

!++
! end_time > start_time, ok. See if the times are coincident with the 
! times of the current performance file.
!--

230                continue
                   time_1 = start_time
                   call read_nest( keyed_read, time_1, ios )
                   if ( key_compare( time_1, end_time )
     .                                      .eq. greater ) then
                      call LIB$SIGNAL( msg_tooearly )
                      start_set = .false.
                      end_set = .false.
                   end if
                   goto 10

             endif
             goto 10

!++
! command was DISPLAY something
!--

300          continue

!++
! determine how many entries to process and if a file name is present
!--

             entries = 0
             pcb_ptr = 4

             do while ( ( pcb(pcb_ptr) .eq. 5 ) .or. 
     .                  ( pcb(pcb_ptr) .eq. 4 ) )
                entries = entries + 1
                pcb_ptr = pcb_ptr + pcb(pcb_ptr + 1) + 2
             end do

!++
! see if a file name was present, ie, pcb(pcb_ptr) = 3
!--

             if ( pcb(pcb_ptr) .eq. 3 ) then
                 file_spec = compress( pcb(pcb_ptr+1) )
                 out_unit = 7
                 open( unit=out_unit, status='NEW',
     .                 file=file_spec(1:MK$LENGTH(file_spec)),
     .                 carriagecontrol='LIST', err=320 )
                 file_present = .true.
             end if

             if ( start_set .and. end_set ) then
                 j = 7

                 do i = 1, entries
                    call display( start_time, end_time, pcb(j),
     .                            pcb(j-1) )
                    j = j + 4
                 end do

             else
                call LIB$SIGNAL( msg_notset )
             end if

!++
! if an output file was speciifed, reset the output and close the file
!--

             if ( file_present ) then
                close( unit=out_unit )
                out_unit = 6
                file_present = .false.
             end if

             goto 10
!++
! error occurred on a DISPLAY file
!--

320          continue
             out_unit = 6

             call LIB$SIGNAL( msg_opdipfil, %VAL(1), 
     .                        file_spec(1:MK$LENGTH(file_spec)) )
             goto 10

!++
! command was EXIT
!--

400          continue
             call read_nest( nest_close, end_time, ios )
             close ( unit=in_unit )
             close ( unit=out_unit )
             stop 'STATS normal exit...'

!++
! command was PLOT or PDL
!--

500          continue
             if ( start_set .and. end_set ) then

!++
! create PDL file according to the specification made in the user line
!--
               multi_plot = .false.
               frames_to_do = 1

               if ( pcb(3) .eq. 7 ) then

!++
! determine if this is a multiple plot invocation
!--

                 index_lit = 8
                 frames_to_do = 1
                 if ( pcb(index_lit) .eq. 5 ) multi_plot = .true.

                 do while ( pcb(index_lit) .eq.  5 )
                    frames_to_do = frames_to_do + 1
                    index_lit = index_lit + 4
                 end do 

                 file_spec = compress( pcb(index_lit+1) )
                 open( unit=12, file=file_spec(1:MK$LENGTH(file_spec)),
     .                 status='NEW', err=520 )
               end if

               do j = 1, frames_to_do
                 call plot( start_time, end_time, i, pcb(6+(j-1)*4),
     .                      pcb(7+(j-1)*4), j, multi_plot, interval )
               end do

               if (  pcb(3) .eq. 7 ) close( unit=12 )
             else
                 call LIB$SIGNAL( msg_notset )
             end if
             goto 10

!++
! error occurred in the creation of the PDL file. Give'em a message
!--

520          call LIB$SIGNAL( msg_oppdlfil, %VAL(1),
     .                        file_spec(1:MK$LENGTH(file_spec)) )
             goto 10

!++
! use command 
!--

600       continue

!++
! first close down the old nest file
!--
          call read_nest( nest_close, end_time, ios )

          old_nest_file = nest_file
          nest_file = ' '
          nest_file = compress( pcb(5) )

!++
! try to open up the new nest file
!--
          call read_nest( nest_open, time_1, ios )
          if ( ios .ne. 0 ) then
             call LIB$SIGNAL( msg_opperfil, %VAL(1),
     .                        nest_file(1:MK$LENGTH(nest_file)) )
             nest_file = old_nest_file
             call read_nest( nest_open, time_1, ios )
             goto 10
          end if

!++
! The alternate nest file is there; tell'em where it starts. 
!--

          call read_nest( seq_read, time_1, ios )
          status = MK$FILENAME( nest_file )
          write( out_unit, 620 ) nest_file(1:MK$LENGTH(nest_file)),
     .                           key_to_date(time_1)
620       format( ' ', A, ' begins at ', A )
          goto 10

!++
! command was SHOW something or the other
!--
          
800       continue
          j = 6
          do while ( pcb(j) .ne. 0 )
             goto ( 810, 820, 830 ) pcb(j)
!++
! show start
!--
810          continue
             if ( start_set ) then
                write( out_unit, 815 ) key_to_date( start_time )
815             format( ' The starting time is ', A, '.' )
             else
                call LIB$SIGNAL( msg_startset )
             end if
             goto 899
!++
! show end
!--
820          continue
             if ( end_set ) then
                write( out_unit, 825 ) key_to_date( end_time )
825             format( ' The ending time is ', A, '.' )
             else
                call LIB$SIGNAL( msg_endset )
             end if
             goto 899

!++
! show file
!--
830          continue
             status = MK$FILENAME( nest_file )
             write( out_unit, 835 ) nest_file(1:MK$LENGTH(nest_file))
835          format( ' The current STATS file is ', A, '.' )

899          j = j + 3
          end do
          goto 10          

!++
! command was replay
!--

900       continue

!++
! determine how many entries to process and if a file name is present
!--

             entries = 0
             pcb_ptr = 4

             do while ( pcb(pcb_ptr) .eq. 5 )
                entries = entries + 1
                pcb_ptr = pcb_ptr + pcb(pcb_ptr + 1) + 2
             end do

!++
! see if a file name was present, ie, pcb(pcb_ptr) = 3
!--

             if ( pcb(pcb_ptr) .eq. 3 ) then
                 file_spec = compress( pcb(pcb_ptr+1) )
                 out_unit = 7
                 open( unit=out_unit, status='NEW',
     .                 file=file_spec(1:MK$LENGTH(file_spec)),
     .                 carriagecontrol='LIST', err=920 )
                 file_present = .true.
             end if

             if ( start_set .and. end_set ) then
                 j = 6

                 do i = 1, entries
                    call replay( start_time, end_time, pcb(j),
     .                           pcb(j+1) )
                    j = j + 4
                 end do

             else
                 call LIB$SIGNAL( msg_notset )
             end if

!++
! if an output file was speciifed, reset the output and close the file
!--

             if ( file_present ) then
                close( unit=out_unit )
                out_unit = 6
                file_present = .false.
             end if

             goto 10
!++
! error occurred on a REPLAY file
!--

920          continue
             out_unit = 6
             call LIB$SIGNAL( msg_oprepfil, %VAL(1),
     .                        file_spec(1:MK$LENGTH(file_spec)) )
             goto 10
          
!++
! command was snooze
!--
 
1000         continue
             alarm_set = .false.

             if ( pcb(4) .eq. 3 ) then
                pcb_ptr = 5
             else
                pcb_ptr = 8
                alarm_set = .true.
             end if 

             snooze_spec = compress( pcb(pcb_ptr) )
             status = SYS$BINTIM( snooze_spec, snooze_time )

             if ( status .ne. %LOC(SS$_NORMAL) ) then
                call LIB$SIGNAL( msg_invalidtime, %VAL(1),
     .                           snooze_spec(1:MK$LENGTH(snooze_spec)) )
                goto 10
             endif

             status = SYS$SCHDWK( , ,snooze_time, )
             write( out_unit, 1020 )
1020         format( ' ', 'Zzzz...' )
             status = SYS$HIBER()
             if ( alarm_set ) then
                do j = 1, 7
                   write( out_unit, 1030 ) interrupt_bell
1030               format( '$', A )
                end do
             end if

             goto 10

!++
! command was help
!--

1100         continue

             call statshlp( compress( pcb(5) ) )
             goto 10

!++
! do a DCL command
!--

1200      continue

          status = LIB$SPAWN( '$' // compress( pcb(5) ) )
          if ( .not. status ) then
             out_buffer = ' '
             status = SYS$GETMSG( %VAL(status), %VAL(0), out_buffer,
     .                            %VAL(0), %VAL(0) )
             write( out_unit, 1210 ) out_buffer(1:MK$LENGTH(out_buffer))
1210         format( ' ', A )
          end if

          goto 10

!++
! Syntax error occurred
!--

          else
             if ( pcb(3) .lt. 1 ) then
                call LIB$SIGNAL( msg_unknown )
             else
                call LIB$SIGNAL( msg_syntax, %VAL(1),
     .                         stats_commands(pcb(3))
     .                         (1:MK$LENGTH(stats_commands(pcb(3)))) )
             end if 

          end if
          goto 10
          end
