$ !--------------------------------------------------------------------!
$ !     Procedure:      AMDAHLRJE                                      !
$ !--------------------------------------------------------------------!
$ !     Programmer:     Kevin Klughart                                 !
$ !--------------------------------------------------------------------!
$ !     Date:           03-FEB-83 @ 16:00 CDST                         !
$ !--------------------------------------------------------------------!
$ !     Language:       VAX/VMS V3.1 DCL                               !
$ !--------------------------------------------------------------------!
$ !     System:         VAX-11/780                                     !
$ !                     Mostek Corporation                             !
$ !                     Operations Planning Systems                    !
$ !                     1215 West Crosby Road                          !
$ !                     Carrollton, Texas  75006                       !
$ !                     Mail Station 2306                              !
$ !                     (214) 466-7506                                 !
$ !--------------------------------------------------------------------!
$ !     Purpose:        This command procedure allows remote VAX       !
$ !                     users to transmit job files through a VAX      !
$ !                     node on the Mostek Engineering Network to the  !
$ !                     AMDAHL system served by that remote VAX node.  !
$ !--------------------------------------------------------------------!
$ !     Notes:          The remote VAX AMDAHL server node must have the!
$ !                     VAX/VMS 2780/3780 software installed and must  !
$ !                     have an account set up to enable remote users  !
$ !                     to submit batch jobs to trigger the transmit   !
$ !                     to the AMDAHL.  The format for the username of !
$ !                     this access account is the character "X" plus  !
$ !                     the first three characters of the local node   !
$ !                     name plus "RJE".  Thus users on node CADEGR    !
$ !                     would access account XCADRJE with password     !
$ !                     XCADRJE.  The AMDAHL server node and the       !
$ !                     the RJE 2780/3780 job queue name are defined   !
$ !                     below and may be changed to reflect your       !
$ !                     specific configuration.                        !
$ !--------------------------------------------------------------------!
$ !     Methodology:    This procedure processes two distinct and      !
$ !                     different conditions:                          !
$ !                                                                    !
$ !                     1.  The job deck is resident on the local node !
$ !                         and must simply be transferred to the      !
$ !                         AMDAHL via a job sent to the appropriate   !
$ !                         2780/3780 job submission batch queue.      !
$ !                                                                    !
$ !                     2.  The user and job deck are resident on a    !
$ !                         different node than the server.  In this   !
$ !                         case one additional file must be created:  !
$ !                         a file to submit the file created in (1)   !
$ !                         to the appropriate queue on the remote     !
$ !                         server node.  In this case the job to be   !
$ !                         submitted must also be transmitted to the  !
$ !                         remote server node prior to execution of   !
$ !                         the batch procedures.                      !
$ !--------------------------------------------------------------------!
$ !
$ !--------------------------------------------------------------------!
$ !                                                                    !
$ !     AMDAHL Extended Remote Batch job submission procedure          !
$ !                                                                    !
$ !--------------------------------------------------------------------!
$ !     Inform user of procedure function                              !
$ !--------------------------------------------------------------------!
$       WRITE SYS$OUTPUT " "
$       WRITE SYS$OUTPUT "	***  VAX/VMS V3.1 AMDAHL RJE Job Submission Procedure  ***"
$       WRITE SYS$OUTPUT "				Version 2.1"
$       WRITE SYS$OUTPUT "				03-FEB-1983"
$       WRITE SYS$OUTPUT " "
$       WRITE SYS$OUTPUT " "
$       WRITE SYS$OUTPUT " "
$ !--------------------------------------------------------------------!
$ !     Initialize warning, error, and control-y processing            !
$ !--------------------------------------------------------------------!
$       ON ERROR        THEN GOTO QUIT
$       ON SEVERE_ERROR THEN GOTO QUIT
$       ON WARNING      THEN GOTO QUIT
$       ON CONTROL_Y    THEN GOTO QUIT
$ !--------------------------------------------------------------------!
$ !     Initialize site dependent default network parameters           !
$ !--------------------------------------------------------------------!
$       SERVER = "OPSPLN"                       ! REMOTE 2780/3780 SERVER
$       JOBQUE = "AMDAHL"                       ! RJE BATCH QUEUE FOR AMDAHL JOB
$ !
$ !--------------------------------------------------------------------!
$ !     Initialize procedure debug flag for debugging purposes         !
$ !--------------------------------------------------------------------!
$                             TRACE = "/DELETE /NOPRINTER /NOLOG_FILE"
$       IF (P2 .NES. "") THEN TRACE = ""
$ !--------------------------------------------------------------------!
$ !     Determine if the network software is functional - this is a    !
$ !     requirement by this procedure so an informational message must !
$ !     be displayed if this is not the case!                          !
$ !--------------------------------------------------------------------!
$       LCLNOD = F$LOGICAL("SYS$NODE")          ! LOCAL NODE NAME
$       IF (LCLNOD .NES. "") THEN GOTO NETOK    ! CONTINUE IF NETWORK UP
$
$       WRITE SYS$OUTPUT "***  ERROR:  DECnet must be operational for this procedure to function properly  ***"
$       EXIT
$ !--------------------------------------------------------------------!
$ !     Determine if this is a local job submission or a remote job.   !
$ !     Set LCLJOB flag to be true/false based on this condition.      !
$ !     Set SERVER to be either server node plus access control or     !
$ !     null, depending on the server node being the local node.       !
$ !--------------------------------------------------------------------!
$ NETOK:NODNAM = LCLNOD - "_" - "::"            ! STRIP UNDERSCORE + COLONS
$       LCLJOB = (NODNAM .EQS. SERVER)          ! SET LOCAL JOB FLAG
$
$       NODNAM = F$EXTRACT(0,3,NODNAM)          ! GET LOCAL NODE ID
$       IF (.NOT. LCLJOB) THEN NODSPC = "''SERVER'""X''NODNAM'RJE X''NODNAM'RJE""::"
$       IF (      LCLJOB) THEN NODSPC = ""
$ !--------------------------------------------------------------------!
$ !     Create three unique file specifications of the form HHMMSSCC.- !
$ !--------------------------------------------------------------------!
$ !     The TMP file is required for remote node job submissions       !
$ !     The AMD file is required for submission to the RJE job queue   !
$ !     The RJE file is required for remote node copy of local job deck!
$ !     The TXT file is required for the mail utility (notify user)    !
$ !--------------------------------------------------------------------!
$       UNIQUE = F$TIME()
$       UNIQUE = F$EXTRACT(12,2,UNIQUE)+-
                 F$EXTRACT(15,2,UNIQUE)+-
                 F$EXTRACT(18,2,UNIQUE)+-
                 F$EXTRACT(21,2,UNIQUE)
$
$       SRVNAM = UNIQUE+".TMP"
$       AMDNAM = UNIQUE+".AMD"
$       RJENAM = UNIQUE+".RJE"
$       TXTNAM = UNIQUE+".MAI"
$ !
$ !--------------------------------------------------------------------!
$ !                                                                    !
$ !     Get name of file to transmit to AMDAHL & validate syntax       !
$ !                                                                    !
$ !--------------------------------------------------------------------!
$ !     Insure that the specified file exists and that the format      !
$ !     of the file specification has correct syntax.  Obtain the      !
$ !     fully qualified file specification in symbol FILNAM.           !
$ !--------------------------------------------------------------------!
$ !     Process cases in which filename has been passed as a parameter !
$ !--------------------------------------------------------------------!
$ FCHK: FILNAM = "''P1'"
$       IF (FILNAM .EQS. "") THEN GOTO ENAM
$ !--------------------------------------------------------------------!
$ !     Parse and search for the file and continue processing if found !
$ !--------------------------------------------------------------------!
$       FILNAM = F$PARSE(FILNAM,".RJE")
$       FILNAM = F$SEARCH(FILNAM)
$
$       IF (FILNAM .NES. "") THEN GOTO SEND
$ !--------------------------------------------------------------------!
$ !     File not found or incorrectly specified - reenter file spec    !
$ !--------------------------------------------------------------------!
$ ENAM: INQUIRE P1 "Enter AMDAHL job filename"
$       GOTO FCHK
$ !
$ !--------------------------------------------------------------------!
$ !                                                                    !
$ !     Create remote batch stream to submit job deck to AMDAHL queue  !
$ !                                                                    !
$ !--------------------------------------------------------------------!
$ !     If this is a remote node we must first create a trigger job    !
$ !     on the server node to submit a job stream to the RJE batch     !
$ !     queue so that the user job deck may be transmitted to the      !
$ !     AMDAHL.  This trigger job merely redirects the actual job      !
$ !     submission procedure to the appropriate batch queue then exits.!
$ !--------------------------------------------------------------------!
$ SEND: WRITE SYS$OUTPUT " "
$       WRITE SYS$OUTPUT "	Submitting file  ''FILNAM'"
$       WRITE SYS$OUTPUT "	to Amdahl server _''SERVER':: (''JOBQUE' job queue)"
$       WRITE SYS$OUTPUT " "
$
$       IF (LCLJOB) THEN RJENAM = FILNAM
$       IF (LCLJOB) THEN GOTO QJOB
$
$       OPEN /WRITE  SRVJOB 'NODSPC''SRVNAM'
$
$       WRITE SRVJOB "$ !----------------------------------------------!"
$       WRITE SRVJOB "$ !                                              !"
$       WRITE SRVJOB "$ !     Remote node job trigger to AMDAHL server !"
$       WRITE SRVJOB "$ !                                              !"
$       WRITE SRVJOB "$ !----------------------------------------------!"
$       WRITE SRVJOB "$ !     Disable printing of batch log file       !"
$       WRITE SRVJOB "$ !----------------------------------------------!"
$       WRITE SRVJOB "$       ASSIGN NLA0: SYS$PRINT"
$       WRITE SRVJOB "$ !----------------------------------------------!"
$       WRITE SRVJOB "$ !     User name           = ''F$GETJPI("","USERNAME")'"
$       WRITE SRVJOB "$ !     Process name        = ''F$GETJPI("","PRCNAM")'"
$       WRITE SRVJOB "$ !     Process UIC         = ''F$GETJPI("","UIC")'"
$       WRITE SRVJOB "$ !     Process default     = ''F$LOGICAL("SYS$DISK")'''F$DIRECTORY()'"
$       WRITE SRVJOB "$ !     File specification  = ''F$LOGICAL("SYS$NODE")'''FILNAM'"
$       WRITE SRVJOB "$ !----------------------------------------------!"
$       WRITE SRVJOB "$ !     Queue AMDAHL job submission batch queue  !"
$       WRITE SRVJOB "$ !----------------------------------------------!"
$       WRITE SRVJOB "$       SUBMIT ''AMDNAM' /QUEUE=''JOBQUE' /PARAMETERS=(''RJENAM') /NOTIFY ''TRACE'"
$       WRITE SRVJOB "$ !----------------------------------------------!"
$       WRITE SRVJOB "$ !     End of remote AMDAHL job submit trigger  !"
$       WRITE SRVJOB "$ !----------------------------------------------!"
$
$       CLOSE SRVJOB
$ ! 
$ !--------------------------------------------------------------------!
$ !                                                                    !
$ !     Create 2780/3780 AMDAHL queue mini job stream to transfer job  !
$ !                                                                    !
$ !--------------------------------------------------------------------!
$ !     This file is created for the expressed purpose of transmitting !
$ !     the user job file (now resident on the server node) to the     !
$ !     AMDAHL via the use of the RJE 2780/3780 software.  This        !
$ !     procedure MUST be executed from the RJE batch queue (specified !
$ !     by the JOBQUE symbol) for it to properly transmit the file to  !
$ !     the AMDAHL.  To accomplish this, the job is submitted with a   !
$ !     /QUEUE= qualifier from either the remote RJE trigger job or    !
$ !     interactively (if the local node is also the server node).     !
$ !--------------------------------------------------------------------!
$ QJOB: OPEN  /WRITE AMDJOB 'NODSPC''AMDNAM'
$
$       WRITE AMDJOB "$ !----------------------------------------------!"
$       WRITE AMDJOB "$ !                                              !"
$       WRITE AMDJOB "$ !     Submit job deck to RJE transmit queue    !"
$       WRITE AMDJOB "$ !                                              !"
$       WRITE AMDJOB "$ !----------------------------------------------!"
$       WRITE AMDJOB "$ !     Initialize error processing              !"
$       WRITE AMDJOB "$ !----------------------------------------------!"
$       WRITE AMDJOB "$       ON ERROR        THEN CONTINUE"
$       WRITE AMDJOB "$       ON SEVERE_ERROR THEN CONTINUE"
$       WRITE AMDJOB "$       ON WARNING      THEN CONTINUE"
$       WRITE AMDJOB "$       ON CONTROL_Y    THEN CONTINUE"
$       WRITE AMDJOB "$ !----------------------------------------------!"
$       WRITE AMDJOB "$ !     User name           = ''F$GETJPI("","USERNAME")'"
$       WRITE AMDJOB "$ !     Process name        = ''F$GETJPI("","PRCNAM")'"
$       WRITE AMDJOB "$ !     Process UIC         = ''F$GETJPI("","UIC")'"
$       WRITE AMDJOB "$ !     Process default     = ''F$LOGICAL("SYS$DISK")'''F$DIRECTORY()'"
$       WRITE AMDJOB "$ !     File specification  = ''F$LOGICAL("SYS$NODE")'''FILNAM'"
$       WRITE AMDJOB "$ !----------------------------------------------!"
$       WRITE AMDJOB "$ !     Execute RJE emulator to submit job deck  !"
$       WRITE AMDJOB "$ !----------------------------------------------!"
$       WRITE AMDJOB "$       RUN SYS$SYSTEM:REMOTE"
$       WRITE AMDJOB "SEND ''RJENAM' /TIMEOUT=01:00:00"
$       WRITE AMDJOB "WAIT /TIMEOUT=01:00:00"
$       WRITE AMDJOB "$ !----------------------------------------------!"
$       WRITE AMDJOB "$ !     Issue mail to user indicating status     !"
$       WRITE AMDJOB "$ !----------------------------------------------!"
$       WRITE AMDJOB "$       STATUS = $STATUS"
$       WRITE AMDJOB "$       STSVAL = 'STATUS'"
$       WRITE AMDJOB "$       FAOVAL = F$FAO(""%!ZL (hex):"",STSVAL)"
$       WRITE AMDJOB "$       FAOMSG = F$MESSAGE(STATUS)"
$       WRITE AMDJOB "$"
$       WRITE AMDJOB "$       OPEN /WRITE MAIL ''TXTNAM'"
$       WRITE AMDJOB "$       WRITE MAIL ""Your job ''FILNAM' submitted on"""
$       WRITE AMDJOB "$       WRITE MAIL ""''F$TIME()' completed with a status of "",FAOVAL"
$       WRITE AMDJOB "$       WRITE MAIL "" """
$       WRITE AMDJOB "$       WRITE MAIL ""	***  "",FAOMSG,""  ***"""
$       WRITE AMDJOB "$       WRITE MAIL "" """
$       WRITE AMDJOB "$       WRITE MAIL "" """
$       WRITE AMDJOB "$       WRITE MAIL ""        		OPS AMDAHLRJE V2.1"""
$       WRITE AMDJOB "$       CLOSE MAIL"
$       WRITE AMDJOB "$"
$       WRITE AMDJOB "$       MAIL"
$       WRITE AMDJOB "SEND ''TXTNAM'"
$       WRITE AMDJOB "''LCLNOD'''F$GETJPI("","USERNAME")'"
$       WRITE AMDJOB "RJE job ''LCLNOD'''FILNAM' completion status"
$       WRITE AMDJOB "$       DELETE ''TXTNAM';*"
$
$       IF (LCLJOB) THEN GOTO QEND
$       WRITE AMDJOB "$ !----------------------------------------------!"
$       WRITE AMDJOB "$ !     Clean up temporary files                 !"
$       WRITE AMDJOB "$ !----------------------------------------------!"
$       WRITE AMDJOB "$       DELETE ''RJENAM';*"
$       WRITE AMDJOB "$       DELETE ''SRVNAM';*"
$       WRITE AMDJOB "$       DELETE   FAL.LOG;*"
$
$ QEND: WRITE AMDJOB "$ !----------------------------------------------!"
$       WRITE AMDJOB "$ !     End of 2780/3780 job submission          !"
$       WRITE AMDJOB "$ !----------------------------------------------!"
$
$       CLOSE AMDJOB
$ !--------------------------------------------------------------------!
$ !                                                                    !
$ !     Copy job file to remote server node and submit trigger job     !
$ !     or submit local job to AMDAHL queue which will then tranfer    !
$ !     job to remote host.                                            !
$ !                                                                    !
$ !--------------------------------------------------------------------!
$       IF (LCLJOB) THEN GOTO QLCL
$
$       COPY   'FILNAM' 'NODSPC''RJENAM'                ! COPY AMDAHL JOB JCL
$       SUBMIT 'NODSPC''SRVNAM' /REMOTE                 ! SUBMIT TRIGGER JOB
$
$       GOTO DONE
$ !--------------------------------------------------------------------!
$ !     Process local job submissions to the AMDAHL                    !
$ !--------------------------------------------------------------------!
$ QLCL: SUBMIT 'AMDNAM' /QUEUE='JOBQUE' /PARAMETERS=('FILNAM') /NOTIFY 'TRACE'
$
$       GOTO DONE
$ !
$ !--------------------------------------------------------------------!
$ !                                                                    !
$ !     Error Trap:     Close all output files and abort processing    !
$ !                                                                    !
$ !--------------------------------------------------------------------!
$ QUIT: IF (F$LOGICAL("SRVJOB") .NES. "") THEN CLOSE SRVJOB
$       IF (F$LOGICAL("AMDJOB") .NES. "") THEN CLOSE AMDJOB
$
$ DONE: WRITE SYS$OUTPUT " "
$       WRITE SYS$OUTPUT " "
$       WRITE SYS$OUTPUT "	***  End of AMDAHL Job Submission Processing  ***"
$ !--------------------------------------------------------------------!
$ !     End of AMDAHL job submission procedure                         !
$ !--------------------------------------------------------------------!
