$ !--------------------------------------------------------------------!
$ !                                                                    !
$ !     NETINS.COM                                                     !
$ !                                                                    !
$ !--------------------------------------------------------------------!
$ !     Programmer:     Kevin Klughart                                 !
$ !     Date:           30-JAN-1983 @ 11:00 CDST                       !
$ !--------------------------------------------------------------------!
$ !     VMS V3.0 command procedure to build necessary accounts for     !
$ !     Executor node definition on Mostek Engineering network.        !
$ !--------------------------------------------------------------------!
$ !     Enable errors to be trapped and determine current defaults     !
$ !--------------------------------------------------------------------!
$       ON ERROR     THEN GOTO QUIT
$       ON WARNING   THEN CONTINUE
$       ON CONTROL_Y THEN GOTO QUIT
$
$       CUR_DEFAULT = F$LOGICAL("SYS$DISK")+F$DIRECTORY()
$       SET DEFAULT SYS$SYSTEM
$ !
$ !--------------------------------------------------------------------!
$ !     Perform preliminary error checking on network logical names    !
$ !--------------------------------------------------------------------!
$ !     Determine the Mostek global root directory tree                !
$ !--------------------------------------------------------------------!
$       MKGTOP = F$LOGICAL("MKG$SYSROOT")
$       MKGLEN = F$LENGTH(MKGTOP)
$       IF (MKGLEN .NE. 0) THEN GOTO MKG
$       WRITE SYS$OUTPUT "***  Mostek Global Tree (MKG) not defined  ***"
$       EXIT
$
$ MKG:  MKGLEN = MKGLEN-2
$       MKGTOP = F$EXTRACT(0,MKGLEN,MKGTOP)+"]"
$ !--------------------------------------------------------------------!
$ !     Determine the Mostek local root directory tree                 !
$ !--------------------------------------------------------------------!
$       MKLTOP = F$LOGICAL("MKL$SYSROOT")
$       MKLLEN = F$LENGTH(MKLTOP)
$       IF (MKLLEN .NE. 0) THEN GOTO MKL
$       WRITE SYS$OUTPUT "***  Mostek Global Tree (MKL) not defined  ***"
$       EXIT
$
$ MKL:  MKLLEN = MKLLEN-2
$       MKLTOP = F$EXTRACT(0,MKLLEN,MKLTOP)+"]"
$ !--------------------------------------------------------------------!
$ !     Insure that network manager logical names are defined          !
$ !--------------------------------------------------------------------!
$ MGR:  NETMGR = F$LOGICAL("MKG$NETWORK")
$       NETLEN = F$LENGTH(NETMGR)
$       IF (NETLEN .NE. 0) THEN GOTO EXE
$
$       WRITE SYS$OUTPUT "***  Network Manager logical name not defined  ***"
$       WRITE SYS$OUTPUT "***  MKG$NETWORK  must be defined for network  ***"
$       EXIT
$ !--------------------------------------------------------------------!
$ !     Insure that executor network manager logical names are defined !
$ !--------------------------------------------------------------------!
$ EXE:  NETMGR = F$LOGICAL("MKL$NETWORK")
$       NETLEN = F$LENGTH(NETMGR)
$       IF (NETLEN .NE. 0) THEN GOTO PRV
$
$       WRITE SYS$OUTPUT "***  Network Manager logical name not defined  ***"
$       WRITE SYS$OUTPUT "***  MKL$NETWORK  must be defined for network  ***"
$       EXIT
$ !--------------------------------------------------------------------!
$ !     Insure that network privileged user logical names are defined  !
$ !--------------------------------------------------------------------!
$ PRV:  NETPRV = F$LOGICAL("MKG$NETPRIV")
$       NETLEN = F$LENGTH(NETPRV)
$       IF (NETLEN .NE. 0) THEN GOTO USR
$
$       WRITE SYS$OUTPUT "***  Privileged user logical name not defined  ***"
$       WRITE SYS$OUTPUT "***  MKG$NETPRIV  must be defined for network  ***"
$       EXIT
$ !--------------------------------------------------------------------!
$ !     Insure that network manager logical names are defined          !
$ !--------------------------------------------------------------------!
$ USR:  NETUSR = F$LOGICAL("MKG$NETUSER")
$       NETLEN = F$LENGTH(NETUSR)
$       IF (NETLEN .NE. 0) THEN GOTO PMGR
$
$       WRITE SYS$OUTPUT "***  Network nonpriv logical name not defined  ***"
$       WRITE SYS$OUTPUT "***  MKG$NETUSER  must be defined for network  ***"
$       EXIT
$ !
$ !--------------------------------------------------------------------!
$ !     Request passwords for privileged network and network manager   !
$ !--------------------------------------------------------------------!
$ PMGR: INQUIRE NETMGR_PSWD "Enter Network Manager Password"
$
$ PPRV: INQUIRE NETPRV_PSWD "Enter Privileged Network Password"
$ !--------------------------------------------------------------------!
$ !     Build the DECnet network manager account                       !
$ !--------------------------------------------------------------------!
$ !     Create the network manager global directory                    !
$ !--------------------------------------------------------------------!
$       CREATE MKG$SYSROOT:[NETMGR] /DIRECTORY /OWNER_UIC=[20,10] -
                /PROTECTION=(SYSTEM:RW,OWNER:RW,GROUP:R,WORLD)
$
$
$       SET FILE 'MKGTOP'NETMGR.DIR /OWNER_UIC=[20,10]
$       SET PROTECTION=(SYSTEM:RW,OWNER:RW,GROUP:R,WORLD) 'MKGTOP'NETMGR.DIR
$
$       SET FILE MKG$NETWORK:*.*;*  /OWNER_UIC=[20,10]
$       SET PROTECTION=(SYSTEM:RW,OWNER:RW,GROUP:R,WORLD) MKG$NETWORK:*.*;*
$
$       SET PROTECTION=(WORLD:R)                          MKG$NETWORK:LOGIN.*
$ !--------------------------------------------------------------------!
$ !     Create the network manager local directory                     !
$ !--------------------------------------------------------------------!
$       CREATE MKL$SYSROOT:[NETMGR] /DIRECTORY /OWNER_UIC=[20,10] -
                /PROTECTION=(SYSTEM:RW,OWNER:RW,GROUP:R,WORLD)
$
$
$       SET FILE 'MKLTOP'NETMGR.DIR /OWNER_UIC=[20,10]
$       SET PROTECTION=(SYSTEM:RW,OWNER:RW,GROUP:R,WORLD) 'MKLTOP'NETMGR.DIR
$
$       SET FILE MKL$NETWORK:*.*;*  /OWNER_UIC=[20,10]
$       SET PROTECTION=(SYSTEM:RW,OWNER:RW,GROUP:R,WORLD) MKL$NETWORK:*.*;*
$ !--------------------------------------------------------------------!
$ !     Create the network manager UAF entry                           !
$ !--------------------------------------------------------------------!
$       RUN SYS$SYSTEM:AUTHORIZE
                REMOVE NETMGR
                ADD    NETMGR
                MODIFY NETMGR /ACCOUNT=DECNET
                MODIFY NETMGR /ASTLM=20
                MODIFY NETMGR /BIOLM=6
                MODIFY NETMGR /BYTLM=8192
                MODIFY NETMGR /CLI=DCL
                MODIFY NETMGR /DEVICE=MKG$SYSROOT:
                MODIFY NETMGR /DIOLM=6
                MODIFY NETMGR /DIRECTORY=[NETMGR]
                MODIFY NETMGR /ENQLM=20
                MODIFY NETMGR /FILLM=20
                MODIFY NETMGR /FLAGS=(DEFCLI,LOCKPWD,DISUSER)
                MODIFY NETMGR /LGICMD=MKG$NETWORK:LOGIN
                MODIFY NETMGR /MAXJOBS=0
                MODIFY NETMGR /OWNER="DECnet network manager account"
                MODIFY NETMGR /PASSWORD=""
                MODIFY NETMGR /PFLAGS=(NODISDIALUP,NODISNETWORK)
                MODIFY NETMGR /P_RESTRICT
                MODIFY NETMGR /PRIMEDAYS=(MON,TUE,WED,THU,FRI,SAT,SUN)
                MODIFY NETMGR /PRIO=4
                MODIFY NETMGR /PRIVILEGES=(ACNT,NOALLSPOOL,NOALTPRI,NOBUGCHK)
                MODIFY NETMGR /PRIVILEGES=(BYPASS,NOCMEXEC,CMKRNL,DETACH)
                MODIFY NETMGR /PRIVILEGES=(NODIAGNOSE,NOEXQUOTA,NOGROUP,NOGRPNAM)
                MODIFY NETMGR /PRIVILEGES=(NOLOG_IO,NOMOUNT,NETMBX,OPER)
                MODIFY NETMGR /PRIVILEGES=(NOPFNMAP,NOPHY_IO,NOPRMCEB,NOPRMGBL)
                MODIFY NETMGR /PRIVILEGES=(NOPRMMBX,NOPSWAPM,SETPRV,NOSHMEM)
                MODIFY NETMGR /PRIVILEGES=(NOSYSGBL,NOSYSLCK,SYSNAM,SYSPRV)
                MODIFY NETMGR /PRIVILEGES=(TMPMBX,NOVOLPRO,NOWORLD)
                MODIFY NETMGR /SFLAGS=(NODISDIALUP,NODISNETWORK)
                MODIFY NETMGR /S_RESTRICT
                MODIFY NETMGR /TQELM=10
                MODIFY NETMGR /UIC=[20,10]
                MODIFY NETMGR /WSDEFAULT=150
                MODIFY NETMGR /WSEXTENT=500
                MODIFY NETMGR /WSQUOTA=200
                CREATE /PROXY
                ADD    /PROXY CADEGR::NETMGR NETMGR
                ADD    /PROXY CADGPX::NETMGR NETMGR
                ADD    /PROXY CAMVAX::NETMGR NETMGR
                ADD    /PROXY ALPD01::NETMGR NETMGR
                ADD    /PROXY OPSPLN::NETMGR NETMGR
                ADD    /PROXY CMS::NETMGR    NETMGR
$ !--------------------------------------------------------------------!
$ !     Modify the password on the network manager UAF entry           !
$ !--------------------------------------------------------------------!
$       OPEN /WRITE NETUAF MKG$NETWORK:NETMGR.UAF
$       WRITE NETUAF "MODIFY NETMGR /PASSWORD=''NETMGR_PSWD'"
$       WRITE NETUAF "MODIFY NETMGR /FLAGS=NODISUSER"
$       CLOSE NETUAF
$
$       ASSIGN /USER_MODE MKG$NETWORK:NETMGR.UAF SYS$INPUT:
$       RUN SYS$SYSTEM:AUTHORIZE
$       DELETE MKG$NETWORK:NETMGR.UAF;*
$ !
$ !--------------------------------------------------------------------!
$ !     Build the privileged DECnet network account                    !
$ !--------------------------------------------------------------------!
$ !     Create the network privileged user directory                   !
$ !--------------------------------------------------------------------!
$       CREATE MKG$SYSROOT:[NETPRV] /DIRECTORY /OWNER_UIC=[20,20] -
                /PROTECTION=(SYSTEM:RW,OWNER:RW,GROUP,WORLD)
$
$
$       SET FILE 'MKGTOP'NETPRV.DIR /OWNER_UIC=[20,20]
$       SET PROTECTION=(SYSTEM:RW,OWNER:RW,GROUP,WORLD) 'MKGTOP'NETPRV.DIR 
$
$       COPY MKG$NETWORK:LOGIN.PRV MKG$NETPRIV:*
$       PURGE MKG$NETPRIV:LOGIN.PRV
$       SET FILE MKG$NETPRIV:*.*;*  /OWNER_UIC=[20,20]
$       SET PROTECTION=(SYSTEM:RW,OWNER:RW,GROUP,WORLD) MKG$NETPRIV:*.*;*
$ !--------------------------------------------------------------------!
$ !     Create the network privileged user UAF entry                   !
$ !--------------------------------------------------------------------!
$       RUN SYS$SYSTEM:AUTHORIZE
                REMOVE NETPRV
                ADD    NETPRV
                MODIFY NETPRV /ACCOUNT=DECNET
                MODIFY NETPRV /ASTLM=20
                MODIFY NETPRV /BIOLM=6
                MODIFY NETPRV /BYTLM=8192
                MODIFY NETPRV /CLI=DCL
                MODIFY NETPRV /DEVICE=MKG$SYSROOT:
                MODIFY NETPRV /DIOLM=6
                MODIFY NETPRV /DIRECTORY=[NETPRV]
                MODIFY NETPRV /ENQLM=20
                MODIFY NETPRV /FILLM=20
                MODIFY NETPRV /FLAGS=(CAPTIVE,DEFCLI,DISCTLY,LOCKPWD,NODISUSER)
                MODIFY NETPRV /LGICMD=MKG$NETWORK:LOGIN.PRV
                MODIFY NETPRV /MAXJOBS=0
                MODIFY NETPRV /OWNER="DECnet privileged user account"
                MODIFY NETPRV /PASSWORD=""
                MODIFY NETPRV /PFLAGS=(DISDIALUP,DISNETWORK)
                MODIFY NETPRV /P_RESTRICT
                MODIFY NETPRV /PRIMEDAYS=(MON,TUE,WED,THU,FRI,SAT,SUN)
                MODIFY NETPRV /PRIO=4
                MODIFY NETPRV /PRIVILEGES=(NOACNT,NOALLSPOOL,NOALTPRI,NOBUGCHK)
                MODIFY NETPRV /PRIVILEGES=(NOBYPASS,NOCMEXEC,NOCMKRNL,NODETACH)
                MODIFY NETPRV /PRIVILEGES=(DIAGNOSE,NOEXQUOTA,NOGROUP,NOGRPNAM)
                MODIFY NETPRV /PRIVILEGES=(NOLOG_IO,NOMOUNT,NETMBX,OPER)
                MODIFY NETPRV /PRIVILEGES=(NOPFNMAP,NOPHY_IO,NOPRMCEB,NOPRMGBL)
                MODIFY NETPRV /PRIVILEGES=(NOPRMMBX,NOPSWAPM,NOSETPRV,NOSHMEM)
                MODIFY NETPRV /PRIVILEGES=(NOSYSGBL,NOSYSLCK,NOSYSNAM,NOSYSPRV)
                MODIFY NETPRV /PRIVILEGES=(TMPMBX,NOVOLPRO,NOWORLD)
                MODIFY NETPRV /SFLAGS=(DISDIALUP,DISNETWORK)
                MODIFY NETPRV /S_RESTRICT
                MODIFY NETPRV /TQELM=10
                MODIFY NETPRV /UIC=[20,20]
                MODIFY NETPRV /WSDEFAULT=150
                MODIFY NETPRV /WSEXTENT=500
                MODIFY NETPRV /WSQUOTA=200
$ !--------------------------------------------------------------------!
$ !     Modify the password on the network privilieged user UAF entry  !
$ !--------------------------------------------------------------------!
$       OPEN /WRITE NETUAF MKG$NETWORK:NETPRV.UAF
$       WRITE NETUAF "MODIFY NETPRV /PASSWORD=''NETPRV_PSWD'"
$       WRITE NETUAF "MODIFY NETPRV /FLAGS=NODISUSER"
$       CLOSE NETUAF
$
$       ASSIGN /USER_MODE MKG$NETWORK:NETPRV.UAF SYS$INPUT:
$       RUN SYS$SYSTEM:AUTHORIZE
$       DELETE MKG$NETWORK:NETPRV.UAF;*
$ !
$ !--------------------------------------------------------------------!
$ !     Build the DECnet nonprivileged user network account            !
$ !--------------------------------------------------------------------!
$ !     Create the network nonprivileged user directory                !
$ !--------------------------------------------------------------------!
$       CREATE MKG$SYSROOT:[NETUSR] /DIRECTORY /OWNER_UIC=[20,30] -
                /PROTECTION=(SYSTEM:RWE,OWNER:RWE,GROUP:RWE,WORLD:RE)
$
$
$       SET FILE 'MKGTOP'NETUSR.DIR /OWNER_UIC=[20,30]
$       SET PROTECTION=(SYSTEM:RWE,OWNER:RWE,GROUP:RWE,WORLD:RE) 'MKGTOP'NETUSR.DIR 
$
$       COPY MKG$NETWORK:LOGIN.USR MKG$NETUSER:
$       PURGE MKG$NETUSER:LOGIN.USR
$       SET FILE MKG$NETUSER:*.*;*  /OWNER_UIC=[20,30]
$       SET PROTECTION=(SYSTEM:RWE,OWNER:RWE,GROUP:RWE,WORLD:RE) MKG$NETUSER:*.*;*
$ !--------------------------------------------------------------------!
$ !     Create the network nonprivileged UAF entry                     !
$ !--------------------------------------------------------------------!
$       RUN SYS$SYSTEM:AUTHORIZE
                REMOVE NETUSR
                ADD    NETUSR
                MODIFY NETUSR /ACCOUNT=DECNET
                MODIFY NETUSR /ASTLM=20
                MODIFY NETUSR /BIOLM=6
                MODIFY NETUSR /BYTLM=8192
                MODIFY NETUSR /CLI=DCL
                MODIFY NETUSR /DEVICE=MKG$SYSROOT:
                MODIFY NETUSR /DIOLM=6
                MODIFY NETUSR /DIRECTORY=[NETUSR]
                MODIFY NETUSR /ENQLM=20
                MODIFY NETUSR /FILLM=20
                MODIFY NETUSR /FLAGS=(CAPTIVE,DEFCLI,DISCTLY,LOCKPWD,NODISUSER)
                MODIFY NETUSR /LGICMD=MKG$NETWORK:LOGIN.USR
                MODIFY NETUSR /MAXJOBS=0
                MODIFY NETUSR /OWNER="DECnet nonprivileged account"
                MODIFY NETUSR /PASSWORD=NETUSR
                MODIFY NETUSR /PFLAGS=(DISDIALUP,DISNETWORK)
                MODIFY NETUSR /P_RESTRICT
                MODIFY NETUSR /PRIMEDAYS=(MON,TUE,WED,THU,FRI,SAT,SUN)
                MODIFY NETUSR /PRIO=4
                MODIFY NETUSR /PRIVILEGES=(NOACNT,NOALLSPOOL,NOALTPRI,NOBUGCHK)
                MODIFY NETUSR /PRIVILEGES=(NOBYPASS,NOCMEXEC,NOCMKRNL,NODETACH)
                MODIFY NETUSR /PRIVILEGES=(NODIAGNOSE,NOEXQUOTA,NOGROUP,NOGRPNAM)
                MODIFY NETUSR /PRIVILEGES=(NOLOG_IO,NOMOUNT,NETMBX,NOOPER)
                MODIFY NETUSR /PRIVILEGES=(NOPFNMAP,NOPHY_IO,NOPRMCEB,NOPRMGBL)
                MODIFY NETUSR /PRIVILEGES=(NOPRMMBX,NOPSWAPM,NOSETPRV,NOSHMEM)
                MODIFY NETUSR /PRIVILEGES=(NOSYSGBL,NOSYSLCK,NOSYSNAM,NOSYSPRV)
                MODIFY NETUSR /PRIVILEGES=(TMPMBX,NOVOLPRO,NOWORLD)
                MODIFY NETUSR /SFLAGS=(DISDIALUP,DISNETWORK)
                MODIFY NETUSR /S_RESTRICT
                MODIFY NETUSR /TQELM=10
                MODIFY NETUSR /UIC=[20,30]
                MODIFY NETUSR /WSDEFAULT=150
                MODIFY NETUSR /WSEXTENT=500
                MODIFY NETUSR /WSQUOTA=200
$ !
$ !--------------------------------------------------------------------!
$ !     Exit network account generation procedure                      !
$ !--------------------------------------------------------------------!
$ QUIT: SET DEFAULT 'CUR_DEFAULT'
$ !--------------------------------------------------------------------!
$ !     End of network account generation procedure                    !
$ !--------------------------------------------------------------------!
